<?php
/**
 * <--- Copyright 2005-2011 de Solis - Cooperativa de Solues Livres Ltda. e
 * Univates - Centro Universitrio.
 * 
 * Este arquivo  parte do programa Gnuteca.
 * 
 * O Gnuteca  um software livre; voc pode redistribu-lo e/ou modific-lo
 * dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao
 * do Software Livre (FSF); na verso 2 da Licena.
 * 
 * Este programa  distribudo na esperana que possa ser til, mas SEM
 * NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO
 * ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU/GPL em
 * portugus para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo
 * "LICENCA.txt", junto com este programa, se no, acesse o Portal do Software
 * Pblico Brasileiro no endereo www.softwarepublico.gov.br ou escreva para a
 * Fundao do Software Livre (FSF) Inc., 51 Franklin St, Fifth Floor, Boston,
 * MA 02110-1301, USA --->
 * 
 * Class Dependency Check
 *
 * @author Eduardo Bonfandini [eduardo@solis.coop.br]
 *
 * @version $Id$
 *
 * \b Maintainers: \n
 * Eduardo Bonfandini [eduardo@solis.coop.br]
 * Jamiel Spezia [jamiel@solis.coop.br]
 * Moises Heberle [moises@solis.coop.br]
 * Jader Osvino Fiegenbaum [jader@solis.coop.br]
 *
 * @since
 * Class created on 06/10/2010
 *
 **/

class DependencyCheck
{
    private $message;

    public function setMessage($message)
    {
        $this->message = $message;
    }

    public function getMessage()
    {
        return $this->message;
    }

    public function listDependency()
    {
        $module = MIOLO::getCurrentModule();

        $list[] = array('gdInstalled', 		_M('Gd (Image library)', $module));
        $list[] = array('yazInstalled', 	_M('Yaz installed (used by Z3950)', $module));
        $list[] = array('ldapInstalled', 	_M('LDAP installed', $module));
        $list[] = array('filePermission', 	_M('Default file permission', $module));
        $list[] = array('logPermission', 	_M('Log write Permission', $module));
        $list[] = array('linkedFiles', 		_M('Gnuteca files that must be inside miolo/html/', $module));
        $list[] = array('emailConfigured', 	_M('Email is configured and sending', $module));
        $list[] = array('printerSocket', 	_M('Printer using socket system', $module));
        $list[] = array('gCron', 		_M('GCron', $module));
        $list[] = array('apacheVersion', 	_M('Apache version', $module));
        $list[] = array('phpVersion', 		_M('Php version', $module));
        $list[] = array('postgresVersion', 	_M('Postgres version', $module));
        $list[] = array('registerGlobals', 	_M('Register globals - security', $module));
        $list[] = array('magicQuotesGpc', 	_M('Magic quotes - security', $module));
        $list[] = array('magicQuotesSybase', 	_M('Magic quotes sybase', $mdule));
        $list[] = array('sessionGcMaxLifeTime', _M('Session - garbage max life time', $mdule));
        $list[] = array('mioloTimeOut',         _M('Configurao do Miolo - session.timeout', $mdule));
        
        

        return $list;
    }

    public function getDependencyLabel($dependency)
    {
        $list = $this->listDependency();

        if ( is_array($list))
        {
            foreach ( $list as $line => $info )
            {
                if ( $info[0] == $dependency)
                {
                    return $info[1];
                }
            }
        }
    }

    public function gdInstalled()
    {
        $this->setMessage( GD_VERSION );

        return extension_loaded ('gd') && function_exists ('gd_info');
    }

    public function yazInstalled()
    {
        $MIOLO      = MIOLO::getInstance();

        $MIOLO->getClass('gnuteca3', 'GnutecaZ3950');

    	$z3950 = new GnutecaZ3950();

        return $z3950->isInstalled();
    }

    public function filePermission()
    {
        $busFile = new BusinessGnuteca3BusFile();
        $this->setMessage('/html/files/');
        return $busFile->isWritable();
    }

    public function logPermission()
    {
        $MIOLO      = MIOLO::getInstance();
        $logPath    = $MIOLO->getConf('home.logs');

        $this->setMessage($logPath);

        return is_writable($logPath);
    }

    public function printerSocket()
    {
        $MIOLO = MIOLO::getInstance();

        if ( PRINT_MODE == 1 )
        {
            $MIOLO->getClass('gnuteca3', 'GnutecaPrinterClient');
            $socket = new GnutecaPrinterClient();
            $ok = $socket->isConnect();

            if ( $ok )
            {
                $this->setMessage(_M('The printer server is running.', 'gnuteca3'));
            }
            else
            {
                $aviso = _M('No existe servidor de impresso rodando nesta mquina. - ', 'gnuteca3');
                $this->setMessage( $aviso .$socket->getSocketLastError() );
            }
        }
        else
        {
            $ok = true;
            $this->setMessage( _M('Mode de impresso definido como : Navegador.', 'gnuteca3'));
        }

        return $ok;
    }
    
    
    /**
     * Checa se a GCron est sendo executada
     */
    public function gCron()
    {
        $MIOLO = MIOLO::getInstance();
        $module = MIOLO::getCurrentModule();
        $modulesPath = $MIOLO->getConf('home.modules');
        $content = file_get_contents($modulesPath . '/gnuteca3/html/files/gcron.log');
        
        $ok = false;
        if ( strlen($content) > 0 )
        {
            $dateNow = GDate::now();
            $dateRunning = new GDate($content);
            $diff = $dateNow->diffDates($dateRunning);
            
            //se faz mais de 2 minutos desde a ltima execuo, o gcron est parado
            if ( $diff->minutes <= 2)
            {
                $ok = true;
            }
        }
        
        if ($ok)
        {
            $this->setMessage(_M('GCron est sendo executado.', 'gnuteca3'));
            return true;
        }
        else 
        {
            $this->setMessage(_M('GCron no est sendo executado.', 'gnuteca3'));
            return false;
        }
    }

    
    public function emailConfigured()
    {
        $MIOLO = MIOLO::getInstance();
        $module = MIOLO::getCurrentModule();
        $MIOLO->getClass( $module , 'GnutecaMail');

        $mail = new GnutecaMail();
        $mail->setAddress(EMAIL_ADMIN);
        $mail->setSubject( _M('Teste de dependncias', $module));
        $mail->setContent( _M( 'Se voc esta lendo esta mensagem  porque seu sistema de emails esta funcionando corretamente.',$module ) ) ;

        $send =  $mail->send();

        if ( $send )
        {
            $this->setMessage( _M('Email enviado para ', $module) . EMAIL_ADMIN . ' . ' .  _M('Por favor, confira o recebimento.') );
        }
        else
        {
            $this->setMessage( $mail->getError() );
        }

        return $send;

    }

    public function linkedFiles()
    {
        $MIOLO      = MIOLO::getInstance();
        $htmlPath   = $MIOLO->getConf('home.html');

        $file   = file_exists($htmlPath.'/file.php');
        $google = file_exists($htmlPath.'/googleViewer.php');
        $upload = file_exists($htmlPath.'/uploadFile.php');

        $ok = $file && $google && $upload;

        if (!$ok)
        {
            $aviso = _M('Files that needed to be in ', 'gnuteca3');
            $this->setMessage($aviso. $htmlPath . ' : ' . 'file.php, googleViewer.php, uploadFile.php');
        }

        return $ok;
    }

    public function ldapInstalled()
    {
        return (function_exists('ldap_connect'));
    }

    public function apacheVersion()
    {
        $version = apache_get_version(); #Apache/2.2.14 (Ubuntu)

        $version = explode('/', $version);
        $version = $version[1]; //2.2.14 (Ubuntu)
        $version = explode('.', $version);

        $this->setMessage(apache_get_version() . ' - ' . _M('2 or more', 'gnuteca3'));

        return $version[0] >=2;
    }

    public function phpVersion()
    {
        $version = phpversion();
        $version = explode('.', $version);

        $this->setMessage( phpversion(). ' - ' ._M('5.3 or more', 'gnuteca3'));

        return $version[0] >= 5 && $version[1] >= 3;
    }

    public function postgresVersion()
    {
        $version = pg_version();

        $server = explode('.', $version['server']);
        $client = explode('.', $version['client']);

        $aviso = _M('Client/Protocol/Server : ', 'gnuteca3');
        $this->setMessage( $aviso. implode(pg_version() , '/' ) .  _M('5.2 or more', 'gnuteca3'));

        return $client[0] >= 8 && $client[1] >= 3 && $server[0] >= 8 && $server[1] >= 3;
    }

    public function registerGlobals()
    {
        $globals = ini_get('register_globals');

        if ( $globals )
        {
            $this->setMessage(_M("Register globals is activated in php.ini, it's a security problem. ", 'gnuteca3')) ;
        }

        return !$globals;
    }

    public function magicQuotesGpc()
    {
        $magicQuotes = get_magic_quotes_gpc();

        if ( !$magicQuotes )
        {
            $this->setMessage(_M("Magic quotes must be activated, it's a security issue.", 'gnuteca3')) ;
        }

        return $magicQuotes;
    }

    public function magicQuotesSybase()
    {
		$magicQuotesSybase = strtolower(ini_get('magic_quotes_sybase'));

		//Para mostrar a imagem correta,  necessrio alterar o valor da varivel
		$magicQuotesSybase = $magicQuotesSybase ? false : true;

        if ( !$magicQuotesSybase )
        {
            $this->setMessage(_M("Magic quotes sybase no pode estar ativa.", 'gnuteca3')) ;
        }

        return $magicQuotesSybase;
    }
    
    public function sessionGcMaxLifeTime()
    {
        $gc_maxlifetime = strtolower(ini_get('session.gc_maxlifetime'));

        //Para mostrar a imagem correta,  necessrio alterar o valor da varivel
        $gc_maxlifetime = ($gc_maxlifetime >= 18000);

        if ( !$gc_maxlifetime )
        {
            $this->setMessage(_M("Sugere-se que o valor de session.gc_maxlifetime seja maior ou igual a 18000 (5 horas).", 'gnuteca3')) ;
        }

        return $gc_maxlifetime;
    }
    
    public function mioloTimeOut()
    {
        $MIOLO = MIOLO::getInstance();
        $mTimeOut = strtolower($MIOLO->getConf('session.timeout'));

        //Para mostrar a imagem correta,  necessrio alterar o valor da varivel
        $mTimeOut = ($mTimeOut >= 300);

        if ( !$mTimeOut )
        {
            $this->setMessage(_M("Sugere-se que o valor de session.timeout do miolo seja maior ou igual a 300 (5 horas).", 'gnuteca3')) ;
        }

        return $mTimeOut;
    }
    

}
?>
