<?php
/**
 * <--- Copyright 2005-2011 de Solis - Cooperativa de Solues Livres Ltda. e
 * Univates - Centro Universitrio.
 * 
 * Este arquivo  parte do programa Gnuteca.
 * 
 * O Gnuteca  um software livre; voc pode redistribu-lo e/ou modific-lo
 * dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao
 * do Software Livre (FSF); na verso 2 da Licena.
 * 
 * Este programa  distribudo na esperana que possa ser til, mas SEM
 * NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO
 * ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU/GPL em
 * portugus para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo
 * "LICENCA.txt", junto com este programa, se no, acesse o Portal do Software
 * Pblico Brasileiro no endereo www.softwarepublico.gov.br ou escreva para a
 * Fundao do Software Livre (FSF) Inc., 51 Franklin St, Fifth Floor, Boston,
 * MA 02110-1301, USA --->
 * 
 * Class GnutecaBussines, extends the default MBussines,
 * including default database configuration and some usefull functions.
 *
 * @author Eduardo Bonfandini [eduardo@solis.coop.br]
 *
 * @version $Id$
 *
 * \b Maintainers: \n
 * Eduardo Bonfandini [eduardo@solis.coop.br]
 * Jamiel Spezia [jamiel@solis.coop.br]
 * Luiz Gregory Filho [luiz@solis.coop.br]
 * Moises Heberle [moises@solis.coop.br]
 *
 * @since
 * Class created on 28/07/2008
 *
 **/
class GnutecaBusiness extends MBusiness
{
    public $MIOLO;
    public $module;
    public $db;
    public $MSQL;
    public $columns, $columnsNoId, $id;
    public $tables;
    public $sql;
    public $removeData;


    /**
    * Class constructor. Create default variables and connects with gnuteca3 database.
    *
    * @param $tables        the table 's that the class will manage
    * @param $id            the id (primary key) of class table, can be comma separated, if has more than one
    * @Param $columnsNoId   the rest of table columsn that class will manage, can be comma separated.
    *
    **/
    function __construct($tables=NULL, $id=NULL, $columnsNoId=NULL, $db = 'gnuteca3')
    {
        parent::__construct();
        $this->MIOLO        = MIOLO::getInstance();
        $this->module       = 'gnuteca3';

        $this->setData(null);
        $this->setDb($db);
        $this->MSQL         = new MSQL();
        $this->tables       = $tables;
        $this->id           = $id;
        $this->columnsNoId  = $columnsNoId;
        $this->columns      = $id;
        
        if (strlen($columnsNoId) > 0)
        {
        	$this->columns .= ',' . $columnsNoId;
        }
    }

    public function setDb($db = 'gnuteca3')
    {
        $this->db = $this->MIOLO->getDatabase($db);
    }


    public function setData($data)
    {
    	//Define removeData como false para nao ter que limpar manualmente em todos business que usam GnutecaRepetitiveField
        $this->removeData = false;
    	return parent::setData($data);
    }


    /**
    * Define the table id (code).
    *
    * ATTENTION: this function automatically mounts the columns using $this->columnsNoId
    *
    * @param $id the string that define the id, can be comma separated, if has more than one
    */
    public function setId($id)
    {
        $this->id = $id;
        $this->setColumns($id .','. $this->columnsNoId);
    }


    /**
    * Define the table columns (exept the id that is defined in other function)
    *
    * ATTENTION: this function automatically mounts the columns using $this->id
    *
    * @param $columnsNoId the string that define the columns, can be comma separated, if has more than one
    */
    public function setColumnsNoid($columnsNoId)
    {
        $this->columnsNoId = $columnsNoId;
        $this->setColumns($this->id .','. $this->columnsNoId);
    }


    /**
    * Define the Bussines Columns
    *
    * @param $columns the string with columns (separated by ',' )
    */
    public function setColumns($columns, $distinct = false)
    {
        if(is_array($columns))
        {
            $columns = $this->getColumnsString($columns);
        }

        $this->columns = $columns;
        $this->MSQL->setColumns($columns, $distinct);
    }


    /**
    * Define the Bussines tables
    *
    * @param $tables the string with tables
    */
    public function setTables($tables)
    {
        $this->tables = $tables;
        $this->MSQL->setTables($tables);
    }


    /**
    * Define the Bussines where
    *
    * @param $where the string with where
    * @param mixed $value Value(s) of SQL
    */
    public function setWhere($where, $value = NULL)
    {
        $this->MSQL->setWhere($where);

        if ($value)
        {
	        foreach ((array)$value as $val)
	        {
	        	$this->MSQL->addParameter($val);
	        }
        }
    }

    public function setLimit( $limit )
    {
    	$this->MSQL->setRange( $limit );
    }

    public function setWhereOr($where)
    {
        $this->MSQL->setWhereOr($where);
    }

    /**
     * Define SQL order
     *
     * @param $orderBy (String): Order by
     */
    public function setOrderBy($orderBy)
    {
        $this->MSQL->setOrderBy($orderBy);
    }


    /**
    * Clear the MSQL object and sql string;
    *
    */
    public function clear()
    {
        $this->MSQL->clear();
        unset($this->sql);
    }


    /**
    * Create a select sql.
    *
    * @param $args the args to prepare sql.
    */
    public function select($args = null)
    {
        $this->sql = $this->MSQL->select($args);
        return $this->sql;
    }

    /**
    * Automatically mount a simple list function
    * that can be used in a MSelection be example
    *
    * @param $object boolean, if you want to use in MSelection please pass $object as false;
    *
    */
    protected function autoList($object=FALSE, $associative=FALSE)
    {
        //$this->clear();
        $this->setColumns($this->columns);
        $this->setTables($this->tables);
        $sql = $this->select();
        $rs  = $this->query($sql, $object);
        if ($associative)
        {
            $out = array();
            if ($rs)
            {
                foreach ($rs as $v)
                {
                    list($id, $value) = $v;
                    $out[ $id ] = $value;
                }
            }
            return $out;
        }
        return $rs;
    }

    /**
    * This functions manage the filters used in autoSelect function
    *
    * @param filters  You have to pass a array, by example
                     $filters  = array(
                    'classificationAreaId'  => 'equals',
                    'areaName'              => 'ilike' ,
                    'classification'        => 'ilike' ,
                    'ignoreClassification'  => 'ilike'
                    );
                    The function will search by classificationAreaId or classificationAreaIdS
                    it add "S" and try to find the information
    *
    */
    protected function addFilters($filters=NULL)
    {
        if ($filters)
        {
            foreach ($filters as $line => $info)
            {
                $temp = $line;
                if (!$this->$temp)
                {
                    $temp = $line.'S';
                }

                if ($info == 'like')
                {
                    if ($this->$temp)
                    {
                        $this->$temp = str_replace(' ', '%', $this->$temp);
                        $this->setWhere($line . ' like ?');
                        $args[] = '%'.$this->$temp.'%';
                    }
                }
                else
                if ($info == 'ilike')
                {
                    if ($this->$temp)
                    {
                        $this->$temp = str_replace(' ', '%', $this->$temp);
                        $this->setWhere('lower('.$line . ') like lower(?)');
                        $args[] = '%'.$this->$temp.'%';
                    }
                }
                else //($info == 'equals')
                {
                    if ($this->$temp)
                    {
                        $this->setWhere($line . ' = ?');
                        $args[] = $this->$temp;
                    }
                }


            }
        }
        return $args;
    }

    /**
    * Automatically makes a search in database
    *
    * @param $filters you can pass the filters using AddFilter format
    *
    */
    protected function autoSearch($filters, $object=FALSE)
    {
        $args = $this->addFilters($filters);
        $this->setColumns($this->columns);
        $this->setTables($this->tables);
        $sql = $this->select($args);
        $rs  = $this->query($sql, $object);
        return $rs;
    }


    /**
    * Function that implements a automatically get Function using $this->columns and $this->tables
    *
    * @param $args you can pass how many arga you want, but it need to be exactely the same order and quantity of $this->id;
    *
    */
    protected function autoGet()
    {
        $data = func_get_args();

        $this->clear();
        $this->setColumns($this->columns);
        $this->setTables($this->tables);
        $temp = explode(',', $this->id);
        foreach ($temp as $line => $info)
        {
            $this->setWhere($info. ' = ?');
        }
        $sql = $this->select($data);
        $rs  = $this->query($sql, $toObject=true);
        $this->setData($rs[0]);
        return $rs[0];
    }


    /**
    * Create a insert sql.
    *
    * @param $args the args to prepare sql.
    */
    public function insert($args)
    {
        $this->sql = $this->MSQL->insert($args);
        return $this->sql;
    }

    /**
    * Automatically make an insert in databasem, it generate the sql and call databse function
    *
    * ATTENTION: if $this->id has a value it will use it to do the insert, otherwise the function do not will pass id , and consider an auto id generated by the database
    *
    */
    protected function autoInsert()
    {
        $ids = explode(',', $this->id);
        if ($ids)
        {
            foreach ($ids as $line =>$info)
            {
                $info = trim($info);
                if ($this->$info)
                {
                    $idValue[] = $this->$info;
                }
            }
        }
        $this->clear();
        //aqui entra se tem que preencher um valor para os Id
        if ($idValue)
        {
        	$columns = $this->getInverseColumns();
            $data = $this->associateData( $columns );
            $this->setColumns( $columns );
        }
        else //aqui entra quando o Id  auto gerado pelo banco
        {
            if ($this->columnsNoId)
            {
            	$data = $this->associateData( $this->columnsNoId );
                $this->setColumns($this->columnsNoId);
            }
            else
            {
            	return false;
            }
        }
        if ($this->tables)
        {
	        $this->setTables($this->tables);
	        $sql = $this->insert($data);
	        $rs  = $this->execute($sql);

	        return $rs;
        }
        else
        {
        	return false;
        }
    }


    /**
    * Create a update sql.
    *
    * @param $args the args to prepare sql.
    */
    public function update($args)
    {
        $this->sql = $this->MSQL->update($args);
        return $this->sql;
    }

    /**
    * Makes an automatically update in this table in database.
    *
    * @param no no param needed, you must set class atributes
    *
    */
    protected function autoUpdate()
    {
        $data = $this->associateData( $this->getInverseColumns() );
        $this->clear();
        $this->setColumns($this->columnsNoId);
        $this->setTables($this->tables);
        $arrayId = explode(',', $this->id);
        foreach ($arrayId as $line => $info)
        {
            $this->setWhere($info . '= ?');
        }
        $sql = $this->update($data);
        $rs  = $this->execute($sql);
        return $rs;
    }


    /**
    * Create a delete sql.
    *
    * @param $args the args to prepare sql.
    */
    public function delete($args)
    {
        $this->sql = $this->MSQL->delete($args);
        return $this->sql;
    }

    /**
    * Make an automatically delete on base, it make the sql and execute it.
    * ATTENTION: you have to pass the functions args in the same quantity and order that $this->id
    *
    * TODO:: INFORMAR ERROS MELHOR PARA O PROGRAMADOR
    *
    * @param $args can be various args in quantity you need, but you must respect the $this->id quantity and order
    *
    */
    protected function autoDelete()
    {
        $this->clear();
        $this->setTables($this->tables);
        $arrayId = explode(',', $this->id);
        foreach ($arrayId as $line => $info)
        {
            $this->setWhere($info . '= ?');
        }
        $sql = $this->delete(func_get_args());
        $rs  = $this->execute($sql);
        return $rs;
    }


    /**
    * Executes an sql using query function, if you not pass the $sql param it will take $this->sql, insert, update, delete and select functions set it.
    *
    * @param $sql the sql to query.
    * @param #queryToObj if is to apply queryToObj function that convert the result to a object
    */
    public function query($sql=NULL, $queryToObj=NULL)
    {
        if ($sql)
        {
            $query =  $this->db->query($sql);
        }
        else
        {
            $query = $this->db->query($this->sql);
        }

        if(!$query->result)
        {
            return false;
        }

        if ($queryToObj)
        {
            return $this->queryToObj($query->result);
        }

        return $query->result;
   }


    /**
    * Executes a sql.
    *
    * @param $sql the sql to execute.
    */
    public function execute($sql=NULL)
    {
        $sql = $sql ? $sql : $this->sql;
        return $this->db->execute($sql);
    }


    /**
    * Convert an string separated by ',' to an array, if you not pass
    * $columnsString parameters it will take the default $this->columns
    *
    * @param $columnsString the string separated by ',' with all columns
    *
    * @return array - the array with columns
    */
    public function getColumnsArray($columnsString=NULL)
    {
        if ($columnsString)
        {
            $columns = explode(',', $columnsString);
        }
        else
        {
            $columns = explode(',', $this->columns);
        }
        if (is_array($columns) )
        {
            foreach ($columns as $line => $info)
            {
                $columns[$line] = trim($info);
            }
        }
        return $columns;
    }


    /**
     * Transforma um array de colunas em uma string preparada para MSQL
     *
     * @param (Array) $columns
     * @return String
     */

    public function getColumnsString($columns = null)
    {
        if(!is_array($columns))
        {
            return false;
        }

        return implode(", ", $columns);
    }


    /**
     * Strip "AS" and "Ref." SQL column
     *
     * @param $column (String): Column to parse
     *
     * @return $column (String): Column parsed
     */
    public function stripSQLColumn($column)
    {
        $col = $column;

        if (($pos = stripos($column, ' AS ')) > 0)
        {
            $col = substr($column, ($pos + 3), strlen($column));
            $col = trim($col);
        }
        else
        {
            //Remove table name/alias from SQL column (eg. A.personId, strip "A.")
            $col = explode('.', $column);
            $col = $col[(count($col)-1)];
        }
        $col = str_replace('"','', $col ); //extract " when it needed

        return $col;
    }


    /**
    * Convert a MSQL query result to an object or array of objects (if is an multilpe dimension array)
    *
    * @param $query the result of the query
    * @param $columnsString the string separated by ',' with columns, you can pass null, the function will take the default $this->columns
    *
    * @return an array of object or a object, according passed data, can return false if has some erros or query is not an array
    */
    public function queryToObj($query, $columnsString=NULL)
    {
        if (!is_array($query))
        {
            return false;
        }

        if (is_array($query[0]))
        {
            foreach ($query as $line => $info)
            {
                $data[] = $this->queryToObj($info, $columnsString);
            }
            return $data;
        }

        $columns = $this->getColumnsArray($columnsString);

        if (count($query) != count($columns))
        {
            echo ( _M('QueryToObj - Error: Differences between columns and query.', 'gnuteca3') );
            return false;
        }

        foreach ($query as $line => $info)
        {
            $column = $this->stripSQLColumn($columns[$line]);
            $data->$column = $info;
        }

        return $data;
    }


    /**
     * Associate SQL columns with business data
     *
     * @param $columns (String): SQL columns string, separate by comma
     *
     * @return $array (Array)
     */
    public function associateData($columns = NULL)
    {
        if ($columns == NULL)
        {
            $columns = $this->columns;
        }

        $cols  = explode(',', $columns);
        if (!$cols)
        {
        	return false;
        }
        $array = array();
        $data  = $this->getData();
        if (!$data)
        {
        	return false;
        }
        foreach ($cols as $line)
        {
            $line = $this->stripSQLColumn( trim($line) );
            $array[] = $data->$line;
        }
        return $array;
    }


    /*
     * Get columns starting with normal columns and end by primary keys
     */
    public function getInverseColumns()
    {
	    $columns = '';
	    if ($this->columnsNoId)
	    {
	        $columns = $this->columnsNoId.',';
	    }
	    $columns .= $this->id;
	    return $columns;
    }


    public function columnsToArray($columns)
    {
    	$exp  = explode(',', $columns);
    	$cols = array();
    	if (count($exp))
    	{
    		foreach ($exp as $val)
    		{
    			$cols[] = trim($val);
    		}
    	}
    	return $cols;
    }

    /**
     * Extract the columns part of a sql and explode it and return as a array
     *
     * @param string $sql
     * @return array the array with columns name
     */
    public function parseSqlToColumnsArray( $sql )
    {
        $sql = preg_replace("/\((.*)[from|FROM](.*)\)/", "", $sql);

        $sql   = ' ' . trim( $sql );
        $posIni = stripos( $sql, 'select') + 6;
        $posFim = stripos( $sql, 'from');

        if ( $posIni && $posFim)
        {
            $sql = substr( $sql, $posIni, $posFim-$posIni );
        }

        $columns = explode(',',$sql);

        if ( $columns )
        {
            $count = count($columns)-1;

            foreach ( $columns as $line => $info)
            {
                $columns[$line] = $this->stripSQLColumn( $columns[$line] );
            }            $sql       = substr( $string, 4, strlen($string) );

            /*$result    = $this->executeSelect( $sql );

            if ( count( $result ) == 1 && count( $result[0] ) ==1 )
            {
                return $result[0][0];
            }
            else
            {
                return $result;
            }*/
        }

        return $columns;
    }


    /**
     * Verifies if sql is valid to execute (or is only a select, not insert, update or delete)
     *
     * @param unknown_type $sql
     */
    public function verifySql( $sql )
    {
        $posDelete = stripos( ' '.$sql, 'delete');
        $posInsert = stripos( ' '.$sql, 'insert');
        $posUpdate = stripos( ' '.$sql, 'update');

        if ($posDelete > 0 || $posInsert > 0 || $posUpdate > 0)
        {
            return false;
        }

        return true;
    }


    /**
     * Execute an sql, changing the sql by the form data
     *
     * The sub sql is executed for each result of sql.
     *
     * @param string $sql the sql string
     * @param string $subSql the sub sql string
     * @param object $data the default ajax miolo data
     * @return array the database result
     */
    public function executeSelect( $sql , $subSql = null, $data = null)
    {
        //faz replace das variveis
        if ($data)
        {
            $data = (array) ( $data );
            foreach ( $data as $line => $info)
            {
                $sql = str_replace( '$'.$line, $info, $sql);
            }
        }

        if ( $sql && $this->verifySql( $sql ))
        {
            $result = $this->query($sql);

            //faz a subsql caso necessrio
            if ( $result && $subSql && $this->verifySql( $subSql ) )
            {
                foreach ( $result as $line => $info )
                {
                    $tempSubSql    = $subSql;
                    if ( is_array( $info ))
                    {
                        foreach ( $info as $l => $i )
                        {
                            $tempSubSql    = str_replace( '$'.$l, $info[$l], $tempSubSql);
                        }
                    }
                    $subResult      = $this->query( $tempSubSql );
                    if ( is_array($subResult) )
                    {
                        $result[$line] = array_merge($result[$line], $subResult[0] );
                    }
                }
            }
        }

        return $result;
    }

    public function beginTransaction()
    {
        $this->execute("begin transaction;");
    }
    
    public function commitTransaction()
    {
        $this->execute("commit;");
    }
    
    public function rollbackTransaction()
    {
    	$this->execute("rollback;");
    }
}
?>
