<?php

/**
 * <--- Copyright 2005-2011 de Solis - Cooperativa de Solues Livres Ltda. e
 * Univates - Centro Universitrio.
 * 
 * Este arquivo  parte do programa Gnuteca.
 * 
 * O Gnuteca  um software livre; voc pode redistribu-lo e/ou modific-lo
 * dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao
 * do Software Livre (FSF); na verso 2 da Licena.
 * 
 * Este programa  distribudo na esperana que possa ser til, mas SEM
 * NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO
 * ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU/GPL em
 * portugus para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo
 * "LICENCA.txt", junto com este programa, se no, acesse o Portal do Software
 * Pblico Brasileiro no endereo www.softwarepublico.gov.br ou escreva para a
 * Fundao do Software Livre (FSF) Inc., 51 Franklin St, Fifth Floor, Boston,
 * MA 02110-1301, USA --->
 * 
 * Class
 *
 * @author Luiz Gilberto Gregory F [luiz@solis.coop.br]
 *
 * @version $Id$
 *
 * \b Maintainers: \n
 * Eduardo Bonfandini [eduardo@solis.coop.br]
 * Jamiel Spezia [jamiel@solis.coop.br]
 * Luiz Gregory Filho [luiz@solis.coop.br]
 * Moises Heberle [moises@solis.coop.br]
 *
 * @since
 * Class created on 09/03/2009
 *
 **/

class GnutecaDate2
{

    /**
     * Attributes
     */
    private $original       = null,
            $mask           = null,
            $H              = null,
            $i              = null,
            $s              = null,
            $m              = null,
            $d              = null,
            $Y              = null,
            $defaultMask    = null,
            $calculate      = null,
            $error          = null,
            $timeStampUnix  = null;

    public $MIOLO, $module;

    /**
     * Constructor Method
     */
    function __construct($date = null, $defaultMask = "d/m/Y H:i:s")
    {
        $this->MIOLO    = MIOLO::getInstance();
        $this->module   = MIOLO::getCurrentModule();

        $this->setOriginalDate($date, $defaultMask);
    }


    /**
     * Seta a data que ser trabalhada na classe
     *
     * @param timestamp $date
     */
    public function setOriginalDate($date = null, $defaultMask = "d/m/Y H:i:s")
    {
        $this->clean();

        if(is_null($date))
        {
            $date = date($defaultMask);
        }
        else
        {
            $d      = explode(".", $date);
            $date   = $d[0];
        }

        $this->defaultMask  = $defaultMask;
        $this->original     = trim($date);

        $this->explodeDate();
    }


    /**
     * Verifica se  uma data valida.
     *
     * @return boolean
     */
    public function isValid()
    {
        if(!is_null($this->error))
        {
            return false;
        }

        if(!checkdate($this->m, $this->d, $this->Y))
        {
            $this->error = 2;
            return false;
        }

        return true;
    }


    /**
     * Verifica se a mascara da data original  igual a uma determinada data passada por parametro
     *
     * @param date mask $mask
     * @return boolean
     */
    public function checkMask($mask)
    {
        if(!is_null($this->error))
        {
            return false;
        }

        return $this->mask == $mask;
    }


    /**
     *  Este metodo retorna a data do objeto.
     *  Podemos informar a mascara desejada no primeiro parametro. Deixando este nullo, retorna a data original.
     *
     * @param mascara desejada $mask
     * @return datetime/false
     */
    public function getDate($mask = null)
    {
        if($mask == null)
        {
            $mask = $this->defaultMask;
        }

        if(!is_null($this->error))
        {
            return false;
        }

        return $this->getDateFromTimestampUnix($mask);
    }



    /**
     * Retorna data com mascara para o banco
     *
     * @param string $type
     * @return date string
     */
    public function getDBDate($type = '')
    {
        return $this->getDate($this->getDBMask($type));
    }


    /**
     * Retorna data com mascara para o banco
     *
     * @param string $type
     * @return date string
     */
    public function getUserDate($type = '')
    {
        return $this->getDate($this->getUserMask($type));
    }
    
    /**
     * Funo chamada automaticamente pelo PHP quando precisa converter dado para String
     * 
     * @return a data no formato do usurio
     */
    public function __toString()
    {
    	return $this->getUserDate();
    }
    
    
    /**
     * Funo que o miolo chama automaticamente, convertendo o objeto para string
     * 
     * @return a data no formato do usurio
     */
    public function generate()
    {
    	return $this->getUserDate();
    }



    /**
     * Retorna a diferena entre a data do objeto e a data do objeto do parametro.
     *
     * @param Object GnutecaDate2 $dateSubtract
     */
    public function subtractDate($dateSubtract)
    {
        $unixTypeDateSubtract   = $dateSubtract->getTimestampUnix();
        $unixTypeDateLocal      = $this->getTimestampUnix();
        $this->timeStampUnix    = ($unixTypeDateLocal - $unixTypeDateSubtract);

        return $this->timeStampUnix;
    }


    /**
     * retorna diferena de dias entre as datas
     * esta funcao trabalha somente com data. exemplo: 02/02/2008
     */
    public function diffDays($dateDiff)
    {
        $unixTypeDateDiff   = $dateDiff->getTimestampUnix();
        $unixTypeDateLocal  = $this->getTimestampUnix();

        $unixDiff = ($unixTypeDateDiff - $unixTypeDateLocal);

        return ($unixDiff / 86400);
    }


    /**
     * Soma minutos a data original
     *
     * @param integer $minutes
     */
    public function sumMinutes($minutes)
    {
        if(!is_null($this->error))
        {
            return false;
        }

        $this->calculate = true;
        $this->i       += $minutes;
    }



    /**
     * Soma horas a data original
     *
     * @param integer $hours
     */
    public function sumHours($hours)
    {
        if(!is_null($this->error))
        {
            return false;
        }

        $this->calculate = true;
        $this->H        += $hours;
    }


    /**
     * Soma dias a data original
     *
     * @param integer $days
     */
    public function sumDays($days)
    {
        if(!is_null($this->error))
        {
            return false;
        }

        $this->calculate = true;
        $this->d        += $days;
    }

    /**
     * diminui dias a data original
     *
     * @param integer $days
     */
    public function subtractDays($days)
    {
        if(!is_null($this->error))
        {
            return false;
        }

        $this->calculate = true;
        $this->d        -= $days;
    }


    /**
     * Soma dias a data original
     *
     * @param integer $days
     */
    public function sumYears($years)
    {
        if(!is_null($this->error))
        {
            return false;
        }

        $this->calculate = true;
        $this->Y       += $years;
    }

    /**
     * retorna o time stamp da data original
     *
     * @return long int
     */
    public function getTimestampUnix()
    {
        if(!is_null($this->error))
        {
            return false;
        }

        $this->timeStampUnix = mktime($this->H, $this->i, $this->s, $this->m, $this->d, $this->Y);

        return $this->timeStampUnix;
    }



    /**
     * Transforma o time stamp unix em uma data
     *
     * @param string $mask
     * @return string date
     */
    public function getDateFromTimestampUnix($mask, $unixTimeStamp = null)
    {
        if(is_null($unixTimeStamp))
        {
            $unixTimeStamp = $this->getTimestampUnix();
        }

        if($unixTimeStamp)
        {
            $mask = str_ireplace(array('dd', 'mm', 'yyyy', 'hh', 'ii', 'ss'), array('d', 'm', 'Y', 'H', 'i', 's'), $mask);
            return date($mask, $unixTimeStamp);
        }

        return false;
    }



    /**
     * Compara dois obejetos gnutecaDate2
     *
     * @param GnutecaDate2 Object $date
     * @param string $operation
     * @return boolean
     */
    public function compare($date, $operation = '=')
    {
        switch ($operation)
        {
            case '>'    :
                return ($this->getTimestampUnix() > $date->getTimestampUnix());

            case '<'    :
                return ($this->getTimestampUnix() < $date->getTimestampUnix());

            case '>='   :
                return (($this->getTimestampUnix() == $date->getTimestampUnix()) || ($this->getTimestampUnix() > $date->getTimestampUnix()));

            case '<='   :
                return (($this->getTimestampUnix() == $date->getTimestampUnix()) || ($this->getTimestampUnix() < $date->getTimestampUnix()));

            default     :
                return ($this->getTimestampUnix() == $date->getTimestampUnix());
        }

    }



    /**
     * Este metodo verifica se a data passada por parametro esta em um formato vlido.
     * Estando em um formato valido, esta seta os attributos internos e retorna true.
     *
     * @return Boolean
     */
    private function explodeDate()
    {
        $this->error        = null;
        $this->calculate    = null;



        // formato = dd/mm/yyyy hh:ii:ss
        if(ereg("^([0-9]{2})\/([0-9]{2})\/([0-9]{4}) ([0-9]{2})\:([0-9]{2})\:([0-9]{2})\$", $this->original, $reg))
        {
            $this->H    = $reg[4];
            $this->i    = $reg[5];
            $this->s    = $reg[6];
            $this->m    = $reg[2];
            $this->d    = $reg[1];
            $this->Y    = $reg[3];
            $this->mask = "dd/mm/yyyy hh:ii:ss";
            return true;
        }
        // formato = dd/mm/yyyy hh:ii
        if(ereg("^([0-9]{2})\/([0-9]{2})\/([0-9]{4}) ([0-9]{2})\:([0-9]{2})\$", $this->original, $reg))
        {
            $this->H    = $reg[4];
            $this->i    = $reg[5];
            $this->s    = '00';
            $this->m    = $reg[2];
            $this->d    = $reg[1];
            $this->Y    = $reg[3];
            $this->mask = "dd/mm/yyyy hh:ii";
            return true;
        }
        // formato = dd/mm/yyyy
        if(ereg("^([0-9]{2})\/([0-9]{2})\/([0-9]{4})\$", $this->original, $reg))
        {
            $this->H    = '00';
            $this->i    = '00';
            $this->s    = '00';
            $this->m    = $reg[2];
            $this->d    = $reg[1];
            $this->Y    = $reg[3];
            $this->mask = "dd/mm/yyyy";
            return true;
        }
        // formato = dd/mm
        if(ereg("^([0-9]{2})\/([0-9]{2})\$", $this->original, $reg))
        {
            $this->H    = '00';
            $this->i    = '00';
            $this->s    = '00';
            $this->m    = $reg[2];
            $this->d    = $reg[1];
            $this->Y    = date("Y");
            $this->mask = "dd/mm/yyyy";
            return true;
        }
        // formato = yyyy-mm-dd hh:ii:ss
        if(ereg("^([0-9]{4})-([0-9]{2})-([0-9]{2}) ([0-9]{2})\:([0-9]{2})\:([0-9]{2})\$", $this->original, $reg))
        {
            $this->H    = $reg[4];
            $this->i    = $reg[5];
            $this->s    = $reg[6];
            $this->m    = $reg[2];
            $this->d    = $reg[3];
            $this->Y    = $reg[1];
            $this->mask = "yyyy-mm-dd hh:ii:ss";
            return true;
        }
        // formato = yyyy-mm-dd hh:ii
        if(ereg("^([0-9]{4})-([0-9]{2})-([0-9]{2}) ([0-9]{2})\:([0-9]{2})\$", $this->original, $reg))
        {
            $this->H    = $reg[4];
            $this->i    = $reg[5];
            $this->s    = '00';
            $this->m    = $reg[2];
            $this->d    = $reg[3];
            $this->Y    = $reg[1];
            $this->mask = "yyyy-mm-dd hh:ii";
            return true;
        }
        // formato = yyyy-mm-dd
        if(ereg("^([0-9]{4})-([0-9]{2})-([0-9]{2})\$", $this->original, $reg))
        {
            $this->H    = $reg[4];
            $this->i    = $reg[5];
            $this->s    = '00';
            $this->m    = $reg[2];
            $this->d    = $reg[3];
            $this->Y    = $reg[1];
            $this->mask = "yyyy-mm-dd";
            return true;
        }

        $this->error = 1;
        return false;
    }



    /**
     * Limpa os atributos do objeto
     */
    public function clean()
    {
        $this->original       =
        $this->mask           =
        $this->H              =
        $this->i              =
        $this->s              =
        $this->m              =
        $this->d              =
        $this->Y              =
        $this->defaultMask    =
        $this->calculate      =
        $this->error          = null;
    }


    /**
     * Retorna a mascara padro do usurio
     *
     * @param string $type
     * @return string date mask
     */
    public function getUserMask($type)
    {
        if(!defined('MASK_DATE_USER'))
        {
            @define('MASK_DATE_USER', 'dd/mm/yyyy');
        }
        if(!defined('MASK_TIME_USER'))
        {
            @define('MASK_TIME_USER', 'hh:ii:ss');
        }

        switch (strtoupper($type))
        {
            case 'DATE':      return MASK_DATE_USER;
            case 'TIME':      return MASK_TIME_USER;
            case 'TIMESTAMP':
            default         : return MASK_DATE_USER . ' ' . MASK_TIME_USER;
        }
    }


    /**
     * Retorna a mascara padro do banco
     *
     * @param string $type
     * @return string date mask
     */
    public function getDBMask($type)
    {
        if(!defined('MASK_DATE_DB'))
        {
            @define('MASK_DATE_DB', 'yyyy-mm-dd');
        }
        if(!defined('MASK_TIME_DB'))
        {
            @define('MASK_TIME_DB', 'hh:ii:ss');
        }

        switch (strtoupper($type))
        {
            case 'DATE':      return MASK_DATE_DB;
            case 'TIME':      return MASK_TIME_DB;
            case 'TIMESTAMP':
            default         : return MASK_DATE_DB . ' ' . MASK_TIME_DB;
        }
    }


    /**
     * Retorna a descrio do erro
     */
    public function getErrorDescription()
    {
        switch($this->error)
        {
            case 1: return _M("O formato da data '@1'  invlido!",     $this->module, $this->original);
            case 2: return _M("A data '@1'  invlida!",                $this->module, $this->original);
        }
    }

}// final da classe


?>
