<?php

/**
 * <--- Copyright 2005-2011 de Solis - Cooperativa de Solues Livres Ltda. e
 * Univates - Centro Universitrio.
 * 
 * Este arquivo  parte do programa Gnuteca.
 * 
 * O Gnuteca  um software livre; voc pode redistribu-lo e/ou modific-lo
 * dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao
 * do Software Livre (FSF); na verso 2 da Licena.
 * 
 * Este programa  distribudo na esperana que possa ser til, mas SEM
 * NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO
 * ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU/GPL em
 * portugus para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo
 * "LICENCA.txt", junto com este programa, se no, acesse o Portal do Software
 * Pblico Brasileiro no endereo www.softwarepublico.gov.br ou escreva para a
 * Fundao do Software Livre (FSF) Inc., 51 Franklin St, Fifth Floor, Boston,
 * MA 02110-1301, USA --->
 * 
 * Class
 *
 * @author Jamiel Spezia [jamiel@solis.coop.br]
 *
 * @version $Id$
 *
 * \b Maintainers: \n
 * Eduardo Bonfandini [eduardo@solis.coop.br]
 * Jamiel Spezia [jamiel@solis.coop.br]
 * Luiz Gregory Filho [luiz@solis.coop.br]
 * Moises Heberle [moises@solis.coop.br]
 *
 * @since
 * Class created on 26/11/2008
 *
 **/
class GnutecaFunction
{
    private $MIOLO,
    $module,
    $variable,
    $executeFunctions=false,
    $busMaterial;
    
    //usado para informar para o gnutecaFunction qual a linha do exemplar a pegar no caso do getTagDescription
    public $line;


    /**
     * Class constructor
     **/
    function __construct()
    {
        $this->MIOLO = MIOLO::getInstance();
        $this->module = $this->MIOLO->getCurrentModule();
        $this->setVariable('$LN', "\n");

        $this->busMaterial = $this->MIOLO->getBusiness($this->module, 'BusMaterial');
    }


    public function setVariable( $variable , $value )
    {
        $this->variable[ $variable ] = $value;
    }

    public function clearVariables()
    {
        unset($this->variable);
    }

    public function replaceVariables( $content )
    {
        $content = strtr( $content, $this->variable );
        // REMOVE AS VARIAVEIS TAG QUE NAO FORAM TROCADAS
        $content = preg_replace('/\$[0-9]{3}.[a-zA-Z0-9]{1}/', '', $content);
        return $content;
    }


    /**
     * Determine if can do execute functions
     *
     * @param boolean $executeFunctions Determine if can do execute functions
     */
    public function SetExecuteFunctions( $executeFunctions )
    {
        $this->executeFunctions  = $executeFunctions;
    }


    /**
     *
     *
     * @param $content (String)
     *
     * @return String
     */
    public function interpret($content, $replaceTextDefaultFormated = true)
    {
        if($replaceTextDefaultFormated)
        {
            $content = str_replace(array("\n", "\t", "\r"), '', $content);
        }

        if ( count($this->variable) > 0 )
        {
            $content = $this->replaceVariables( $content );
        }

        $explode = preg_split('/<([^<]*)>/U', $content, -1, PREG_SPLIT_DELIM_CAPTURE);

        $level = 0;
        foreach ($explode as $key=>$exp)
        {
            if (($key % 2) == 0) //Se for 0  o texto do usurio
            {
                $text[$level] .= $exp;
            }
            else //Se for 1  a tag
            {
                if (substr($exp, 0, 1) == '/')
                {
                    $result  = $this->callFunction($tag[$level], $text[$level]);
                    if ( is_array($result) )
                    {
                        return $result;
                    }
                    $text[$level-1] .= $result;
                    $level--;
                }
                else
                {
                    $level++;
                    $text[$level] = '';
                    $tag[$level]  = $exp;
                }
            }
        }
        return $text[0];
    }

    /**
     * Call a definide funcion
     *
     * @param $function (String) the name of the function
     * @param $content (String) the content to apply the function
     *
     * @return String the content/string converted/interpreted
     */
    public function callFunction($function, $content)
    {
        $f = $this->explodeFunction($function);

        switch (strtolower($f->name))
        {
            //Funes genricas
            case 'cut':
                if ($f->parameter[1])
                {
                    $content = substr($content, $f->parameter[0], $f->parameter[1]);
                }
                else
                {
                    $content = substr($content, $f->parameter[0]);
                }
                break;

            case 'lower':
                $content = strtolower($content);
                break;

            case 'pad':
                $p1 = $f->parameter[1];
                if (!$p1)
                {
                    $p1 = ' ';
                }
                switch (strtoupper($f->parameter[2]))
                {
                    case 'RIGHT':
                        $p3 = STR_PAD_RIGHT;
                        break;
                    case 'LEFT':
                        $p3 = STR_PAD_LEFT;
                        break;
                    case 'BOTH':
                        $p3 = STR_PAD_BOTH;
                        break;
                    default:
                        $p3 = STR_PAD_RIGHT;
                        break;
                }
                $content = str_pad($content, $f->parameter[0], $p1, $p3 );

                // impede que i texto exeda o tamanho
                if(strlen($content)>$f->parameter[0])
                {
                    $content = substr($content,0, $f->parameter[0]-3)."...";
                }

                break;

            case 'replace':
                $content = str_replace($f->parameter[0], $f->parameter[1], $content);
                break;

            case 'upper':
                $content = strtoupper($content);
                break;

            case 'style':
                if ($f->amountParameters > 0 )
                {
                    $style = "";
                    foreach ( $f->parameter as $line => $info )
                    {
                        switch ($info)
                        {
                            case 'b':
                                $content = '<b>'.$content.'</b>';
                                break;
                            case 'i':
                                $content = '<i>'.$content.'</i>';
                                break;
                            case 'u':
                                $content = '<u>'.$content.'</u>';
                                break;
                            default:

                                $permition = array('color', 'font-size');

                                list($command, $value) =  explode('=', $info);

                                if ( in_array($command, $permition) )
                                {
                                	$style .= $command.':'.$value .'; ';
                                }

                                break;

                        }
                    }
                    if ( strlen($style) > 0 )
                    {
                    	$content = '<font style= "' . $style . '">'.$content.'</font>';
                    }
                    
                }
                break;

            case 'date' :
                $f->parameter[0] = str_replace("y", "Y", $f->parameter[0]);
                $content = date($f->parameter[0]);
                break;
                
            case 'datemarc' :
            	$content = date('ymd'); 
            	break;

            case 'ifexists' :
                if ( $f->parameter[0] )
                {
                    $content = $content;
                }
                else
                {
                    $content = '';
                }
                break;

            case 'ifnotexists' :
                if ( !strlen($f->parameter[0] ))
                {
                    $content = $content;
                }
                else
                {
                    $content = '';
                }
                break;

            case 'compare' :

                if(ereg("\([0-9]{1,20} [0-9]{3}\.[a-z0-9]{1}\)", $f->parameter[0]))
                {
                    $f->parameter[0] = str_replace(array('(', ')'), '', $f->parameter[0]);
                    list($controlNumber, $tag) = explode(" ", $f->parameter[0]);
                    $f->parameter[0] = $this->busMaterial->getContentTag($controlNumber, $tag);
                }
                if(ereg("\([0-9]{1,20} [0-9]{3}\.[a-z0-9]{1}\)", $f->parameter[2]))
                {
                    $f->parameter[2] = str_replace(array('(', ')'), '', $f->parameter[2]);
                    list($controlNumber, $tag) = explode(" ", $f->parameter[2]);
                    $f->parameter[2] = $this->busMaterial->getContentTag($controlNumber, $tag);
                }

                switch ($f->parameter[1])
                {
                    case 'maior' :
                        $content = ($f->parameter[0] > $f->parameter[2]) ? $content : '';
                        break;

                    case 'menor' :
                        $content = ($f->parameter[0] < $f->parameter[2]) ? $content : '';
                        break;

                    case 'menor=' :
                        $content = ($f->parameter[0] >= $f->parameter[2]) ? $content : '';
                        break;

                    case 'maior=' :
                        $content = ($f->parameter[0] <= $f->parameter[2]) ? $content : '';
                        break;

                    case '!=' :
                        $content = ($f->parameter[0] != $f->parameter[2]) ? $content : '';
                        break;

                    case 'like' :
                        $content = ereg($f->parameter[0], $f->parameter[2]) ? $content : '';
                        break;

                    case 'ilike' :
                        $content = eregi($f->parameter[0], $f->parameter[2]) ? $content : '';
                        break;

                    default :
                        $content = ($f->parameter[0] == $f->parameter[2]) ? $content : '';
                        break;
                }

                break;

            case 'href' :
                if ( $f->parameter[0] )
                {
                    $label = strlen($content) ? $content : $f->parameter[0];

                    $content = "<a href=\"{$f->parameter[0]}\" target=\"_new\" >{$label}</a>";
                }
                break;


            case 'pregmatch' :

                if(!strlen($f->parameter[1]))
                {
                    break;
                }

                preg_match("/{$f->parameter[0]}/", $f->parameter[1], $match);
                if (isset($match[0]))
                {
                    $content = $match[0];
                }
                break;


            case 'executephp' :
                if ( $this->executeFunctions )
                {
                    $php = '$content = '. $content.';';
                    $php = str_replace("\'","'", $php);
                    $php = str_replace('\"','"', $php);
                    eval ( $php );
                }
                break;


            case 'dbmaterialcount' :

                $controlNumber  = $f->parameter[0];
                $tag            = $f->parameter[1];
                $operator       = $f->parameter[2];
                $value          = $f->parameter[3];

                if(!$controlNumber || !$tag)
                {
                    $content = '';
                    break;
                }

                list($fieldId, $subFieldId) = explode(".", $tag);
                $sql = "SELECT count(*) FROM gtcMaterial WHERE controlNumber = '$controlNumber' AND fieldId = '$fieldId' AND subFieldId = '$subFieldId'";

                $gnutecaBusiness = new GnutecaBusiness();
                $result = $gnutecaBusiness->executeSelect( $sql );
                $result = $result[0][0];

                if(!$operator || !$value)
                {
                    $content = $result;
                    break;
                }

                switch ($operator)
                {
                    case 'maior' :
                        $content = ($result > $value) ? $content : '';
                        break;

                    case 'menor' :
                        $content = ($result < $value) ? $content : '';
                        break;

                    case 'maior=' :
                        $content = ($result >= $value) ? $content : '';
                        break;

                    case 'menor=' :
                        $content = ($result <= $value) ? $content : '';
                        break;

                    case '=' :
                        $content = ($result == $value) ? $content : '';
                        break;

                    case '!=' :
                        $content = ($result != $value) ? $content : '';
                        break;
                }

                break;

            case 'executesql' :
                if ( $this->executeFunctions )
                {
                    $gnutecaBusiness = new GnutecaBusiness();
                    $sql       = $content;
                    $result    = $gnutecaBusiness->executeSelect( $sql );
                    if ( count( $result ) == 1 && count( $result[0] ) ==1 )
                    {
                        $content = $result[0][0];
                    }
                    else
                    {
                        $content = $result;
                    }
                }
                break;

            case 'executedb' :
                if ( $this->executeFunctions )
                {
                    $temp       = explode('->', $content);
                    $bus        = $temp[0];
                    $function   = $temp[1];
                    $busTemp    = $this->MIOLO->getBusiness( $this->module, $bus);
                    $options    = $busTemp->$function();
                    //return $options;
                    $content = $options;
                    //$content = date($f->parameter[0]);
                };
                break;

                //Funes especficas do gnuteca
            case 'gtcgetmaterialcontent':
            case 'getmaterialcontent':

                $content = $this->busMaterial->getContentTag($f->parameter[0], $f->parameter[1]);

                if($f->parameter[2] == '1')
                {
                    $prefix  = $busMaterial->getMaterialTagPrefixSuffix($f->parameter[0], $f->parameter[1], 1);
                    $content = "{$prefix}{$content}";
                }

                if($f->parameter[3] == '1')
                {
                    $suffix     = $busMaterial->getMaterialTagPrefixSuffix($f->parameter[0], $f->parameter[1], 2);
                    $content   .= "{$suffix}";
                }

                break;

            case 'gettagdescription' :

            	//parameter[0] = controlNumber
            	//parameter[1] = tag
            	//parameter[2] = precisa documentao
           	
            	//TODO documentar o porque desta comparao
                if($f->parameter[2] == 1)
                {
                    $controlNumberGranFather =  $this->busMaterial->getContentTag($f->parameter[0], MARC_ANALITIC_ENTRACE_TAG);
                    
                    if($controlNumberGranFather)
                    {
                        $f->parameter[0] = $controlNumberGranFather;
                    }
                }

                //$this->line pega a linha definida no gnuteca function
                $contentX = $this->busMaterial->getContentTag($f->parameter[0], $f->parameter[1], $this->line);

                if(!$contentX)
                {
                    $content = '';
                    break;
                }

                $contentX = $this->busMaterial->relationOfFieldsWithTable($f->parameter[1], $contentX, false);

                if(!$contentX)
                {
                    $content = '';
                    break;
                }

                $content  = str_replace("_DESCRIPTION_", $contentX, $content);
                break;

            case 'getauthors700aabntformat' :
            	$author100a = $this->busMaterial->getContent($f->parameter[0], '100', 'a', null, true, true, 'line');
                $author700a = $this->busMaterial->getContent($f->parameter[0], '700', 'a', null, true, true, 'line');

                //se tiver 100a
                if ( $author100a )
                {
                    $author             = $author100a[0]->content;
                    $primeiraVirgula    = strpos($author, ',');
                    $sobreNomeUpper     = strtoupper(substr($author,0, $primeiraVirgula));
                    $nomeResto          = trim(substr($author, $primeiraVirgula));
                    $content            = "{$sobreNomeUpper}{$nomeResto}";
                    if ( sizeof($author700a) > 0 )
                    {
                    	$cont1 = 0;
                    	foreach($author700a as $k=>$aut)
                    	{
                    		$e700 = $this->busMaterial->getContent($f->parameter[0], '700', 'e', $aut->line, true, true, 'line');
                    		if ( strlen($e700[0]->content) == 0 ) //no pode ter termo relacionador
                    		{
                    			$author             = $aut->content;
			                    $primeiraVirgula    = strpos($author, ',');
			                    $sobreNomeUpper     = strtoupper(substr($author,0, $primeiraVirgula));
			                    $nomeResto          = trim(substr($author, $primeiraVirgula));
			                    $content           .= "; {$sobreNomeUpper}{$nomeResto}";

                    			if ( $cont == 1 )
                    			{
                    				break;
                    			}
                    			$cont++;
                    		}
                    	}
                    }
                    $content .= substr($content, -1) == '.' ? '' : '.'; //bota '.' no final, se no tiver
                }
                else
                {
                	//pega o 700.4 da primeira linha
                    $author7004 = $this->busMaterial->getContent($f->parameter[0], '700', '4', $author700a[0]->line, true, true, 'line');

                    if ( sizeof($author700a) > 0 )
                    {
                        if ( (!preg_match('/ - /', $author700a[0]->content, $var1)) && (preg_match('/et al/', $author7004[0]->content, $var2)) ) //et al sem org
                        {
                            $author             = $author700a[0]->content;
                            $primeiraVirgula    = strpos($author, ',');
                            $sobreNomeUpper     = strtoupper(substr($author,0, $primeiraVirgula));
                            $nomeResto          = trim(substr($author, $primeiraVirgula));
                            $content            = "{$sobreNomeUpper}{$nomeResto} et al.";

                            break;
                        }
                        elseif (preg_match('/et al/', $author7004[0]->content, $var2)) //et al com org, no primeiro
                        {
                            $author = $author700a[0]->content;
                            $primeiraVirgula    = strpos($author, ',');
                            $sobreNomeUpper     = strtoupper(substr($author,0, $primeiraVirgula));
                            $nomeResto          = trim(substr($author, $primeiraVirgula));

                            $broken = explode(' - ', $nomeResto);
                            $nomeResto = $broken[0];

                            $pChar = strtoupper(substr($broken[1],0, 1));
                            $resto = str_replace('.', '', $pChar . substr($broken[1], 1, strlen($broken[1]) -1)); //substitu '.' por nada
                            $resto = "({$resto}.)";

                            $content = "{$sobreNomeUpper}{$nomeResto} et al. {$resto}.";

                        }
                        else
                        {
                        	//faz o agrupamento por 700.e
                        	$group = '';
                        	$cont = 0;
                        	$content = '';
                        	$sufix = '';

                        	//700.4 -> sufixo que ser utilizado
                        	//700.e -> agrupador
                        	foreach ($author700a as $key=>$lineVal)
                        	{
                        		$e700 = $this->busMaterial->getContent($f->parameter[0], '700', 'e', $lineVal->line, true, true, 'line');

                                if ( $e700[0]->content == $group || $key == 0 )
                                {
                                	$group = $e700[0]->content;

                                	if ($key == 0) //se for na primeira volta, pega o sufixo da string
                                	{
                                		$codRel = $this->busMaterial->getContent($f->parameter[0], '700', '4', $lineVal->line, true, true, 'line');
                                		$sufix = $codRel[0]->content;
                                	}
                                    
                                	$author             = $lineVal->content;
		                            $primeiraVirgula    = strpos($author, ',');
		                            $sobreNomeUpper     = strtoupper(substr($author,0, $primeiraVirgula));
		                            $nomeResto          = trim(substr($author, $primeiraVirgula));
		                            $content           .= "{$sobreNomeUpper}{$nomeResto}; ";

		                        	if ( $cont == 2) //para de agrupar no mximo 3 autores
		                            {
		                                break;
		                            }
                                    
		                            $cont++;
                                }
                        	}

                        	//trata o content
                        	$content = substr($content, 0, strlen($content) -2); //tira a virgula do final

                        	//examplo de sufix -> (Trads, Orgs)
                        	if ( strlen($sufix) > 0 )
                        	{
                                if ( substr($sufix, -1, 1) == '.' )
                                {
                                    $sufix = substr($sufix, 0, strlen($sufix) -1); //se tiver '.' no final, o tira
                                }

                        		$sufix = strtoupper(substr($sufix, 0, 1)) . substr($sufix, 1, strlen($sufix)); //primeriro caractr como maisculo
	                        	$sufix .= $cont > 1 ? 's' : ''; //se tiver mais que 1 autor, concateca 's' no final
	                        	$content .= " ({$sufix}.).";
                        	}

                        	$content .= substr($content, -1) == '.' ? '' : '.'; //se no tiver '.' no final, bota.
                        }
                    }
                }

                break;

            case "gettitleabntfotmated" :

                $controlNumber          = $f->parameter[0];
                list($field, $subField) = explode(".",$f->parameter[1]);

                $titles = $this->busMaterial->getContent($controlNumber, $field, $subField, null, true, true, 'line');
                $title = $titles[0];

                if(!$title)
                {
                    break;
                }

                $cutStart   = strlen($title->indicator2) ? $title->indicator2 : 0;
                $titleX     = substr($title->content, $cutStart);
                $firstSpace = strpos($titleX, " ");
                $firstSpace = !$firstSpace ? strlen($titleX) : $firstSpace;
                $firstWord  = substr($titleX, 0, $firstSpace);

                $content = str_replace($firstWord, strtoupper($firstWord), $title->content);

                break;

            case 'gtcgettagname':
                $busTag = $this->MIOLO->getBusiness($this->module, 'BusTag');
                $content = $busTag->getTagNameByTag($f->parameter[0]);
                break;

            case 'gtcseparator':
                $content = $this->gtcSeparator($f);
                break;

            case 'gtcgetseparator':
                $content = $this->gtcGetSeparator($f);
                break;
        }

        return $content;
    }


    /**
     * Este metodo documenta detalhes sobre as funes;
     *
     * @param string $function
     * @return simple array details
     */
    public function helpFunctions($function=null)
    {
        if (($function != null) && (!is_array($function)))
        {
            $function = array($function);
        }

        $help['cut']->description   = 'Retorna a parte da string especificada pelo parmetro 1 e 2.';
        $help['cut']->parameter[0]  = 'Se parmetro 1 no for negativo, a string retornada iniciar na posio especificada, comeando em zero. Por exemplo, na string \'abcdef\', o caractere na posio 0  \'a\', o caractere na posio 2  \'c\', e assim em diante.';
        $help['cut']->parameter[1]  = 'Se parmetro 2 for dado e for positivo, a string retornada ir conter x caracteres comeando no valor especificado no parmetro 1.';
        $help['cut']->example       = '<cut 0| 11>O retorno ser as 11 primeiras letras</cut>';
        $help['cut']->return        = 'O retorno s';

        $help['upper']->description = 'Converte uma string para maisculas';
        $help['upper']->example     = '<upper>maiscula</upper>';
        $help['upper']->return      = 'MAISCULA';

        $help['pad']->description   = 'Retorna a string preenchida na esquerda, direita ou ambos os lados at o tamanho especificado.';
        $help['pad']->parameter[0]  = 'Tamanho final da string.';
        $help['pad']->parameter[1]  = 'Se no for indicado, a string  preenchida com espaos';
        $help['pad']->parameter[2]  = 'Pode ser RIGHT (preencher a direita), LEFT (preencher a esquerda), ou BOTH (preencher de ambos os lados). Se no for especificado  assumido que seja RIGHT.';
        $help['pad']->example       = '<pad 10| -| LEFT>Gntueca</pad>';
        $help['pad']->return        = '---Gnuteca';

        $help['replace']->description   = 'Substitui todas as ocorrncias do parmetro 1 pelo parmetro 2.';
        $help['replace']->parameter[0]  = 'String de procura.';
        $help['replace']->parameter[1]  = 'String de substituio.';
        $help['replace']->example       = '<replace bom| timo>O gnuteca  um bom software</replace>';
        $help['replace']->return        = 'O gnuteca  um timo software';

        $help['lower']->description = 'Converte uma string para minsculas';
        $help['lower']->example     = '<lower>MINSCULA</lower>';
        $help['lower']->return      = 'minscula';

        $help['date']->description  = 'Retorna datas.';
        $help['date']->parameter[0] = 'Formato requerido.<br>Exemplos:<br>d = Dia;<br>m = Ms;<br>Y = Ano;<br>H = Hora;<br>i = Minutos;<br>s = Segundos;<br> Maiores Informaes: http://php.net/date';
        $help['date']->example      = '<date d/m/Y></date>';
        $help['date']->return       = 'dd/mm/YYYY = 01/01/2009';

        $help['ifexists']->description  = 'Verifica se o parmetro existe, caso no existe, no mostra o contdo.';
        $help['ifexists']->parameter[0] = 'o texto para verificao';
        $help['ifexists']->example      = '<ifexists texto>algum texto</>';
        $help['ifexists']->return       = 'o contedo original caso exista o parmetro';

        $help['ifnotexists']->description  = 'Verifica se o parmetro NO existe, caso no existe, mostra o contdo.';
        $help['ifnotexists']->parameter[0] = 'o texto para verificao';
        $help['ifnotexists']->example      = '<ifnotexists texto>algum texto</ifnotexists>';
        $help['ifnotexists']->return       = 'o contedo original caso NO exista o parmetro';

        $help['compare']->description  = 'Faz comparao entre do valores.';
        $help['compare']->parameter[0] = 'Primeiro valor, para comparar com um valor da base, utilize ($001.a tag marc). Ex: ($001.a 100.a)';
        $help['compare']->parameter[1] = 'condio [ = | != | maior | menor | maior= | menor= | like | ilike ]';
        $help['compare']->parameter[3] = 'Segundo valor';
        $help['compare']->example      = '<compare 1 | menor | 2> RETORNO </compare><compare luiz | ilike | ($001.a 100.a)> RETORNO </compare>';
        $help['compare']->return       = 'Retorna o contedo de dentro das tags.';

        $help['href']->description  = 'Cria um link para um conteudo.';
        $help['href']->parameter[0] = 'o link';
        $help['href']->example      = '<href $link > $description </>';
        $help['href']->return       = '';

        $help['pregmatch']->description  = 'Retona o contedo conforme uma expresso regular.';
        $help['pregmatch']->parameter[0] = 'Expresso regular';
        $help['pregmatch']->example      = 'Contedo onde ser aplicado a expresso regular.';
        $help['pregmatch']->return       = 'Contedo encontrado.';

        $help['gtcgetmaterialcontent']->description   = 'Pega o contedo de um etiqueta marc de um determinado material.';
        $help['gtcgetmaterialcontent']->parameter[0]  = 'Nmero de controle do material.';
        $help['gtcgetmaterialcontent']->parameter[1]  = 'Etiqueta marc.';
        $help['gtcgetmaterialcontent']->parameter[2]  = 'Retorna o Prefixo';
        $help['gtcgetmaterialcontent']->parameter[3]  = 'Retorna o Suffixo';
        $help['gtcgetmaterialcontent']->example       = '<gtcGetMaterialContent 25| 245.a | 1 | 0 ></gtcGetMaterialContent>';
        $help['gtcgetmaterialcontent']->return        = 'Ttulo do material';

        $help['getauthors700aabntformat']->description   = 'Regras para autor 700.a da ABNT';
        $help['getauthors700aabntformat']->example       = '<getauthors700aabntformat></getauthors700aabntformat>';
        $help['getauthors700aabntformat']->return        = 'Autores formatados';

        $help['gettitleabntfotmated']->description   = 'Regras para ttulo da ABNT';
        $help['gettitleabntfotmated']->example       = '<gettitleabntfotmated $001.a | 245.a></gettitleabntfotmated>';
        $help['gettitleabntfotmated']->return        = 'Retrna o ttulo formatado';

        $help['gettagdescription']->description   = 'Retorna a descrio de uma determinado campo que armazena indices';
        $help['gettagdescription']->parameter[0]  = 'Nmero de controle';
        $help['gettagdescription']->parameter[1]  = 'Tag marc';
        $help['gettagdescription']->parameter[2]  = 'Se for 1, tenta buscar o contedo do pai.';
        $help['gettagdescription']->example       = '<gettagdescription $001.a | 901.c | 1 >_DESCRIPTION_</gettagdescription>';
        $help['gettagdescription']->return        = 'Descrio do campo. A Contante _DESCRIPTION_ sera substituida pelo contedo encontrado.';

        $help['gtcgettagname']->description   = 'Pega o nome de uma determinada Tag.';
        $help['gtcgettagname']->parameter[0]  = 'Tag MARC.';
        $help['gtcgettagname']->example       = '<gtcGetTagName 245.a></gtcGetTagName>';
        $help['gtcgettagname']->return        = 'Nome da Tag';

        $help['gtcseparator']->description   = 'Une varias tags separando-as por um caracter determinado. O caracter \'#\' significa que  um separador cadastrado na base.';
        $help['gtcseparator']->parameter[0]  = '$001.a = Parametro Obrigatrio, pois a funo necessita do numero de controle da obra.';
        $help['gtcseparator']->example       = '<gtcSeparator $001.a | 245.a | # | 245.b | x | 246.a></gtcSeparator> <br> Para carregar os prefixos e/ou sufixos da tags, utilize um ponto de interrogao no inicio ou no final da tag. Ex: <gtcSeparator $001.a | ?245.a? | # | ?245.b? | x | ?246.a?></gtcSeparator> ';
        $help['gtcseparator']->return        = 'Conteudo dos campos';

        $help['gtcgetseparator']->description   = 'Retorna o separador da tag especificada. Caso o separador no estiver cadastrado para o material especfico, ser adicionado como padro o caracter definido no quarto parmetro. ';
        $help['gtcgetseparator']->parameter[0]  = '$001.a = Parametro Obrigatrio, pois a funo necessita do numero de controle da obra.';
        $help['gtcgetseparator']->parameter[1]  = 'Parametro Obrigatrio, tag1.';
        $help['gtcgetseparator']->parameter[2]  = 'Parametro Obrigatrio, tag2.';
        $help['gtcgetseparator']->parameter[3]  = 'Utiliza este para definir um separador padro caso o material no possua um cadastrado.';
        $help['gtcgetseparator']->example       = '<gtcGetSeparator $001.a | 245.a | 245.b | : ></gtcGetSeparator> <br> A funo buscar o separador das tags especificadas. Ex: <gtcGetSeparator $001.a | 245.a | 245.b | : ></gtcGetSeparator> ';
        $help['gtcgetseparator']->return        = 'Separador';

        $help['executeDB']->description    = 'Executa funo de um business.';
        $help['executeDB']->example        = '<executedb>BusLibraryUnit->listLibraryUnit</executedb>';
        $help['executeDB']->return         = 'Retorna a lista de bibliotecas que  gerada na funo listLibraryUnit da business LibraryUnit.';

        $help['executePHP']->description   = 'Executa cdigo PHP.';
        $help['executePHP']->example       = '<executephp>array("1" => "mesa", "2" => "cadeira");</executephp>';
        $help['executePHP']->return        = 'Retorna Mesa e Cadeira do array.';

        $help['dbmaterialcount']->description   = 'Retorna o numero de ocorrencias de uma tag na gtcMaterial ou tambem pode compara com ou outro valor.';
        $help['dbmaterialcount']->parameter[0]  = 'Nmero de controle do material.';
        $help['dbmaterialcount']->parameter[1]  = 'Etiqueta marc.';
        $help['dbmaterialcount']->parameter[2]  = '[Opcional] = Condio de comparao [ = | != | maior | menor | maior= | menor= ]';
        $help['dbmaterialcount']->parameter[3]  = '[Opcional] = Valor a ser comparado';
        $help['dbmaterialcount']->example       = '<dbMaterialCount 25 | 700.a | maior | 3></dbMaterialCount>';
        $help['dbmaterialcount']->return        = 'Contedo dentro da tag.';

        $help['executeSQL']->description   = 'Gera select de SQL.';
        $help['executeSQL']->example       = '<executesql>SELECT exemplaryStatusId, description FROM gtcExemplaryStatus</executesql>';
        $help['executeSQL']->return        = 'Lista todos os estados dos exemplares. Retorna uma string ou um array.';

        if ($function)
        {
            foreach ($help as $func => $val)
            {
                if (!in_array($func, $function))
                {
                    unset($help[$func]);
                }
            }
        }

        return $help;
    }

    /**
     *
     *
     * @param $function (String)
     *
     * @return Obj
     */
    public function explodeFunction($function)
    {
        $explode = preg_split('/ ([^\|]*)/', $function, -1, PREG_SPLIT_DELIM_CAPTURE);

        $amountParameters = 0;
        foreach ($explode as $key => $exp)
        {
            $exp = trim($exp);

            if ($key == 0)
            {
                $func->name = strtolower($exp);
            }
            elseif (!is_null($exp) && $exp != "|")
            {
                $func->parameter[] = $exp;
                $amountParameters++;
            }
        }

        /**
         * Adicionado esta verificao para remover o oltimo indice que estava
         * vindo sempre "a mais" ou seja... sobrando...
         * so que antes de remove-lo...  feita uma verificaao para garantir que este indice esta vazio
         * -- Luiz
         */
        if(!strlen(trim($func->parameter[$amountParameters-1])))
        {
            $func->amountParameters = ($amountParameters-1);
            unset($func->parameter[$func->amountParameters]);
        }

        return $func;
    }


    /**
     * Este metodo  uma funo  do gnuteca function, foi criado pois o switch que seleciona a funo esta muito comprido ja.
     *
     * @param object $f
     * @return string content
     */
    private function gtcSeparator($f)
    {
        $content        = '';
        $controlNumber  = null;
        $tagAnterior    = null;
        $separator      = null;
        $busMaterial    = $this->MIOLO->getBusiness($this->module, 'BusMaterial');

        foreach ($f->parameter as $value)
        {
            $value = trim($value);

            if(!strlen($value))
            {
                continue;
            }
            if(is_null($controlNumber))
            {
                $controlNumber = $value;
                continue;
            }
            if(ereg("[0-9]{3}.[0-9A-Za-z]{1}", $value))
            {
                $getPrefix  = ($value[0] == "?");
                $getSuffix  = ($value[strlen($value)-1] == "?");
                $value      = str_replace("?", "", $value);

                $tagContent = $busMaterial->getContentTag($controlNumber, $value);

                if($getPrefix && strlen($tagContent))
                {
                    $prefix = $busMaterial->getMaterialTagPrefixSuffix($controlNumber, $value);
                    $tagContent = "{$prefix}{$tagContent}";
                }
                if($getSuffix && strlen($tagContent))
                {
                    $suffix     = $busMaterial->getMaterialTagPrefixSuffix($controlNumber, $value, 2);
                    $tagContent.= "{$suffix}";
                }

                if(!strlen($tagContent))
                {
                    $tagAnterior = null;
                    return $content;
                }
                if(strlen($tagAnterior) && $separator == '#')
                {
                    $separator  = $busMaterial->getTagSeparator($tagAnterior, $controlNumber);
                    if(!$separator)
                    {
                        return $content;
                    }
                    $content   .= $separator[0][2];
                }
                elseif(strlen($tagAnterior) && $separator != '#')
                {
                    $content.= " $separator ";
                }

                $content    .= $tagContent;
                $tagAnterior = $value;
            }
            elseif(strlen($value) < 5 && strlen($tagAnterior))
            {
                $separator = $value;
            }
        }

        return $content;
    }


     /**
     * Retorna o separadore de duas tags.
     *
     * @param object $f
     * @return string content
     */
    private function gtcGetSeparator($f)
    {
        $controlNumber  = $f->parameter[0];
        $tag1           = $f->parameter[1];
        $tag2           = $f->parameter[2];
        $separator      = $f->parameter[3];
        $busMaterial    = $this->MIOLO->getBusiness($this->module, 'BusMaterial');

        $tagSeparator = $busMaterial->getTagSeparator($tag1, $controlNumber);

        return ( $tagSeparator[0][2] ? $tagSeparator[0][2] : $separator );
    }

}
?>
