<?php
/**
 * <--- Copyright 2005-2011 de Solis - Cooperativa de Solues Livres Ltda. e
 * Univates - Centro Universitrio.
 * 
 * Este arquivo  parte do programa Gnuteca.
 * 
 * O Gnuteca  um software livre; voc pode redistribu-lo e/ou modific-lo
 * dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao
 * do Software Livre (FSF); na verso 2 da Licena.
 * 
 * Este programa  distribudo na esperana que possa ser til, mas SEM
 * NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO
 * ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU/GPL em
 * portugus para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo
 * "LICENCA.txt", junto com este programa, se no, acesse o Portal do Software
 * Pblico Brasileiro no endereo www.softwarepublico.gov.br ou escreva para a
 * Fundao do Software Livre (FSF) Inc., 51 Franklin St, Fifth Floor, Boston,
 * MA 02110-1301, USA --->
 * 
 * GnutecaGrid - Grid functions
 *
 * @author Moises Heberle [moises@solis.coop.br]
 *
 * @version $Id$
 *
 * \b Maintainers: \n
 * Eduardo Bonfandini [eduardo@solis.coop.br]
 * Jamiel Spezia [jamiel@solis.coop.br]
 * Moises Heberle [moises@solis.coop.br]
 *
 * @since
 * Class created on 08/01/2009
 *
 **/
$MIOLO = MIOLO::getInstance();

$MIOLO->getClass('gnuteca3', 'GnutecaGridNavigator');

class GnutecaGrid extends MGrid
{
	public $transaction;
	public $actionUpdate   = false;
	public $actionDelete   = false;
	public $useCSV         = TRUE;
	public $comma          = ',';
    public $categoryAndLevelMenus; //menus de categoria e nivel utilizados para montar link de filhos
    public $materialPermission;
    public $busMaterialControl;
    public $busSpreadSheet;

    const TYPE_BOOK                 = 'isBook';
    const TYPE_COLLECTION           = 'isCollection';
    const TYPE_COLLECTION_FASCICLE  = 'isCollectionFascicle';

    public function __construct($data, $columns, $href, $pageLength = null, $index = null, $name = null, $useSelecteds = null, $useNavigator = null)
    {
    	$this->setTransaction($data->transaction);
    	parent::__construct($data, $columns, $href, $pageLength, $index, $name, $useSelecteds, $useNavigator);
    }


    public function setTransaction($transaction = null)
    {
    	$this->transaction = $transaction;
    }


    public function addActionUpdate($href)
    {
    	if (GnutecaPerms::checkAccess($this->transaction, 'update', false))
        {
        	$this->actionUpdate = true;
            $this->addActionIcon(_M("Editar"), 'edit', $href, 0, 'edit');
        }
    }


    public function addActionDelete($href)
    {
        if (GnutecaPerms::checkAccess($this->transaction, 'delete', false))
        {
        	$this->actionDelete = true;
            parent::addActionDelete($href);
        }
    }

    public function addActionIcon($alt, $icon, $href, $index = 0, $imageName = null )
    {
        $object = new GGridActionIcon($this, $icon, $href, $alt, $imageName );
        $this->actions[] = $object;
        return $object;
    }

    public function setCSV($useCSV = true, $comma =',')
    {
    	$this->useCSV = $useCSV;
    	$this->comma  = $comma;
    }

    public function generateData()
    {
        if (!$this->data)
        {
            return;
        }

        $this->orderby = $this->page->request('orderby');

        if ($this->ordered = isset($this->orderby))
        {
            $this->applyOrder($this->orderby);
            $this->page->setViewState('orderby', $this->orderby, $this->name);
        }

        if ($this->getFiltered())
        {
            $this->applyFilter();
        }
        if ($this->pageLength)
        {
            $this->pn = new GnutecaGridNavigator($this->pageLength, $this->rowCount,
                                          $this->getURL($this->filtered, $this->ordered), $this);
            $this->data = $this->getPage();
        }
        else
        {
            $this->pn = null;
        }
    }

    private function generateNavigationDiv()
    {
        $array[0] = $this->pn->getPageFirst();
        $array[0]->_AddStyle('float', 'left');
        $array[1] = $this->pn->getPagePrev();
        $array[1]->_AddStyle('float', 'left');
        $array[2] = $this->pn->getPageRange();
        $array[2]->_AddStyle('float', 'left');
        $array[3] = $this->pn->getPageNext();
        $array[3]->_AddStyle('float', 'left');
        $array[4] = $this->pn->getPageLast();
        $array[4]->_AddStyle('float', 'left');
        $d = new MDiv('', $array, 'm-pagenavigator');
        $d->addStyle('float', 'right');
        return $d;
    }

    public function generateNavigationHeader()
    {
        $pageCount = 1;

        if ( $this->pn )
        {
            $pageCount = $this->pn->getPageCount();
        }

    	$fields[0] = new MLabel( _M("A busca gerou <b>@1</b> resultado(s) listados em <b>@2</b> pgina(s)", MIOLO::getCurrentModule() , $this->rowCount, $pageCount) );
    	$fields[0]->setClass('m-label m-label-grid-upper');

        //mostra ordenao para usurio
        $orderby = MIOLO::_REQUEST('orderby');

        if ( $orderby )
        {
            $column  = $this->columns[$orderby];

            $fields[1] = new MLabel( _M('Ordenando por: ', $this->module) . $column->title);
            $fields[1]->addStyle('float', 'right');
        }

        $d = new MDiv('', $fields, 'm-grid-navigation');
        return $d;
    }

    public function generateNavigationFooter()
    {
        if (!$this->pn)
        {
            return null;
        }

        $links = $this->pn->getPageLinks();

        foreach ($links as $link)
        {
            $link->float = 'left';
        }

        $d1 = new MDiv('', $links);
        $d1->addStyle('float', 'left');
        $d2 = $this->generateNavigationDiv();
        $d = new MDiv('', array($d1, $d2), 'm-grid-navigation m-grid-navigation-footer');
        return $d;
    }



    public function generateActions(&$tbl)
    {
        $i = $this->currentRow;

        if ($this->hasDetail)
        {
           $i += $this->currentRow;
        }
        $c = 0; // colNumber

        $spanClass = ($this->select != NULL) ? ' tall' : '';
        $control = new MSpan('', '&nbsp;');
        if ($this->actionDefault->href)
        {
            $control->addAttribute('onclick',$this->actionDefault->generate());
        }
        $control->setClass($this->buttonSelectClass,false);
        $tbl->setCell($i, $c, $control);
        $tbl->setCellClass($i, $c, 'btn');

        if ($this->hasDetail)
        {
           $tbl->setCell($i+1,$c,new MDiv('ddetail' . $this->currentRow, NULL));
           $tbl->setCellClass($i+1,$c,'action-default');
        }
        $c++;
        $inicio = $c;
        if ($n = count($this->actions))
        {
            // generate action links
            while ($c < ($n + 1))
            {
                $action = $this->actions[$c-1];
                $tbl->setCell($i,$c,$action->generate(),$action->attributes());
                if ($this->hasDetail)
                {
                    $tbl->setCell($i+1,$c,new MDiv('adetail' . $this->currentRow, NULL));
                }
                $action = $c == ($inicio) ? 'action' : 'action-hide';
                $tbl->setCellClass($i,$c,$action);

                //para funcionar corretamente a grid em outros temas
                if ( $action == 'action-hide')
                {
                    $tbl->setCellAttribute($i,$c,'style','display:none');
                }

                $tmp .= $tbl->cell[$i][$c]->generate();
                $tbl->cell[$i][$c] = null;
                $c++;
            }
        }

        $tbl->cell[$i][$inicio] = new Div(null, $tmp, 'gridActions');

        if ($this->select != NULL)
        {
            $tbl->setRowAttribute($i,'id',"row".$this->name."[{$this->currentRow}]");
            $tbl->setCellClass($i,$c,'data select');
            $select = $this->select->generate();
            $select->checked = (array_search($i, $this->selecteds) !== false);
            $tbl->cell[$i][$c] = $select;
            if ($this->hasDetail)
            {
                $tbl->setCell($i+1,$c,new MDiv('sdetail' . $this->currentRow, NULL));
            }
        }
    }



    public function generateColumnsHeading(&$tbl)
    {
        $spanClass = ''; // adjusted via javascript
        $p = 0;
        $this->page->onLoad("miolo.grid.ajustSelect('linkbtn');");
        $this->page->onLoad("miolo.grid.ajustTHead();");
        $tbl->setColGroup($p);
        $span = new MSpan('','&nbsp;',$this->buttonSelectClass);
        $tbl->setHead($p, $span );
        $tbl->setHeadClass($p++, 'btn');
        if ($n = count($this->actions))
        {
            $tbl->setColGroup($p,"span={$n}");
            $tbl->setHead($p,new MSpan('',_M('Action'),$spanClass));
            //$tbl->setHeadAttribute($p,'colspan',$n); //TODO
            $tbl->setHeadClass($p++,'action');
        }
        if ($this->select != NULL)
        {
            $rowCount = count($this->data);
            $this->page->onLoad("miolo.grid.checkEachRow($rowCount,'".$this->name."');");
            $tbl->setColGroup($p);
            $check = new MCheckBox("chkAll", 'chkAction', '');
            $check->addAttribute('onclick',"javascript:miolo.grid.checkAll(this,$rowCount,'".$this->name."');");
            $check->_addStyle('padding','0px');
            $tbl->setHead($p,new MSpan('',$check,'select'));
            $tbl->setHeadClass($p++,'select');
        }

        // generate column headings
        $tbl->setColGroup($p); $c = 0;
        $last = count($this->columns) - 1;
        foreach ($this->columns as $k => $col)
        {
            if ((!$col->visible) || (!$col->title))
            {
                continue;
            }

            if ($col->order)
            {
                $this->orderby = $k;
                $link = new MLinkButton('', $col->title, $this->getURL($this->filtered, true));
                $link->setClass('order');
                $colTitle = new MSpan('',$link,$spanClass);
                $tbl->setHeadClass($p+$c,'order');
            }
            else
            {
                $colTitle = new MSpan('',$col->title,$spanClass);
                $tbl->setHeadClass($p+$k,'data');
            }

            if (($col->width))
            {
                $attr =  ($k != $last) ? " width=\"$col->width\"" : " width=\"100%\"";
            }
            else
            {
                // scrollable tables need col width
                if( $this->scrollable )
                {
                    $this->manager->logMessage( _M("[WARNING] Using scrollable table, it's necessary to inform column width. ") );
                }
            }

            $tbl->setHead($p+$c++,$colTitle);
        }
    }


    /**
     * Gera a grid e o csv dela , se quiser que nao gere o CSV passe o comma como null
     *
     * @param String $comma
     * @return String
     */
    function generate()
    {
        $MIOLO     = MIOLO::getInstance();
        $module    = MIOLO::getCurrentModule();
    	$csv       = "";
    	$columns   = $this->columns;
        $data      = $this->data;
        $comma     = $this->comma;
        $other     = $comma == ',' ? ';' : ','; //troca ',' por ';' caso contenha vrgula no texto
        $event     = GnutecaForm::getEvent();

        #caso a grid no possua um nome, define um nome padro, em funo dos nomes dos arquivos
        if ( !$this->name )
        {
            $this->name = 'grid';
        }

        if (is_array($data) && ($comma) && $this->useCSV)
        {
            if ($event == 'generateGridCSV')
            {
                //Prepara os dados
                if (is_array($columns))
                {
                   foreach ($columns as $line => $column)
                   {
                        $title = null;
                        if ($column->visible)
                        {
                            $title = str_replace($comma, $other, $column->title );
                            $csv .= $title . $comma;
                        }
                   }
                }
                $csv .= "\n";
                foreach ( $data as $line => $info)
                {
                   if (is_array($info))
                   {
                        foreach ( $info as $l => $i )
                        {
                            $column = $columns[$l];
                        	$tempData = null;
                            if ( $column->visible )
                            {
                            	if ( is_object($i) || is_array($i) )
                            	{
                            		$i = print_r($i,1);
                            	}
                                $tempData = str_replace( $comma, $other, $i );
                                $tempData = str_replace("\n" , $other , $tempData);
                            	$csv .= $tempData . $comma;
                            }
                        }
                   }
    	    	   $csv .= "\n";
                }
                
                //Disponibiliza download
                BusinessGnuteca3BusFile::openDownload('grid',$this->name.'.csv', $csv);
            }
            else if ($event == 'generateGridPdf')
            {
                $colTitles = array();
                $array     = array();
                
                //Prepara os dados
                if (is_array($columns))
                {
                    foreach ($columns as $line => $column)
                    {
                        $title = null;
                        if ($column->visible)
                        {
                            $colTitles[] = $column->title;
                        }
                    }
                }
                foreach ($data as $line => $info)
                {
                    if (!is_array($info))
                    {
                       continue;
                    }
                    foreach ( $info as $l => $i )
                    {
                        $column = $columns[$l];
                        $tempData = null;
                        if ( $column->visible )
                        {
                            if ( is_object($i) || is_array($i) )
                            {
                                $i = print_r($i,1);
                            }
                            $array[$line][] = $i;
                        }
                    }
                }
                
                //Instancia e gera PDF
                $MIOLO->uses('classes/GnutecaPDFTable.class.php', $module);
                $pdf = new GnutecaPDFTable('L', 'pt');
                $pdf->addTable( new MTableRaw(null, $array, $colTitles) );
                
                //Disponibiliza o download
                BusinessGnuteca3BusFile::openDownload('grid',$this->name.'.pdf', $pdf->Output(null, 'S'));
            }
            
            $footer   = is_array($this->footer) ? $this->footer : array($this->footer);

            //Generate CSV button
            $url = GnutecaUtils::getAjax('generateGridCSV');
	    	$footer[] = $btnCsv = new MButton('getCsv', ' ', $url, $MIOLO->getUI()->getImage($module, 'csv-16x16.png'));
            $btnCsv->addAttribute('alt',_M('Get CSV', $module) );
            $btnCsv->addAttribute('title',_M('Get CSV', $module) );
            
            //Generate PDF button
            $url = GnutecaUtils::getAjax('generateGridPdf');
            $footer[] = $btnPdf = new MButton('getPdf', ' ', $url, $MIOLO->getUI()->getImage($module, 'document-16x16.png'));
            $btnPdf->addAttribute('alt',_M('Get PDF', $module) );
            $btnPdf->addAttribute('title',_M('Get PDF', $module) );

            //guarda a pgina que a grid esta atualmente para consulta posterior
            $footer[] = new MHiddenField('gridPage', MIOLO::_REQUEST('pn_page') );
            $footer[] = new MHiddenField('orderby', MIOLO::_REQUEST('orderby') );

            $this->setFooter( $footer );
        }

    	$generate = parent::generate();
    	return $generate;
    }

    
    public function getMaterialPermission()
    {
        if (!isset($this->materialPermission))
        {
            $this->materialPermission = ( GnutecaPerms::checkAccess('gtcMaterial', 'insert', false) && GnutecaOperator::isLogged() );
        }
        return $this->materialPermission;
    }
    
    /**
     * Ajusta a acao de adicionar filho na grid dependendo do tipo de material e permissao
     *
     * @param GGridActionIcon $action
     * @param unknown_type $controlNumber
     */
    public function adjustChildAction($action, $controlNumber)
    {
        $MIOLO  = MIOLO::getInstance();
        $module = MIOLO::getCurrentModule();
        
        if (!$controlNumber || !$action)
        {
            return;
        }

        //tendo permisso de insert na gtcmaterial ele pega todos os menus em um nico sql, guardando em um array
        if ( $this->getMaterialPermission() && !$this->categoryAndLevelMenus )
        {
            if (!$this->busSpreadSheet)
            {
                $this->busSpreadSheet = $MIOLO->getBusiness($module, 'BusSpreadsheet');
            }
            $menus = $this->busSpreadSheet->getMenus( null, null, false ); //select
            if ( is_array( $menus ))
            {
                foreach ( $menus as $line => $menu )
                {
                    $this->categoryAndLevelMenus[$menu->category][$menu->level] = $menu;
                }
            }
        }
        
        if (!$this->busMaterialControl)
        {
            $this->busMaterialControl = $MIOLO->getBusiness($module, 'BusMaterialControl');
        }
        
        //Obtem o tipo de material
        if ( $isBook = $this->busMaterialControl->isBook( $controlNumber ) )
        {
            $type = self::TYPE_BOOK;
        }
        else if ( $isCollection = $this->busMaterialControl->isCollection( $controlNumber ) )
        {
            $type = self::TYPE_COLLECTION;
        }
        else if ( $isCollectionFascicle  = $this->busMaterialControl->isCollectionFascicle( $controlNumber ) )
        {
            $type = self::TYPE_COLLECTION_FASCICLE;
        }

        if ( $type )
        {
            //aqui os selects esto com else, ento s executa 1
            if ( $type == self::TYPE_BOOK )
            {
                $menu = $this->categoryAndLevelMenus['BA']['4'];
            }
            else
            if ( $type == self::TYPE_COLLECTION )
            {
                $menu = $this->categoryAndLevelMenus['SE']['4'];
            }
            else
            if ( $type == self::TYPE_COLLECTION_FASCICLE )
            {
                $menu = $this->categoryAndLevelMenus['SA']['4'];
            }

            $extraFields[]   = new MSeparator('<br/>');

            if ($menu)
            {
                $menuoption = str_replace("#", "*", $menu->menuoption);
                $pos = strpos($action->href, 'leaderString=');
                if ($pos)
                {
                    $action->href = substr($action->href, 0, $pos) . 'leaderString=' . $menuoption;
                }
            }
            $action->enable();
        }
        else
        {
            $action->disable();
        }
    }

    public function getURL($filter = false, $order = false, $item = '')
    {
        $params['__filter'] = ($filter) ? '1' : '0';
        $params['orderby'] = $this->orderby;

        $pn_page = MIOLO::_REQUEST('pn_page');

        if ((is_numeric($pn_page)) || (!$pn_page))
        {
            $params['pn_page'] = $pn_page;
        }
        else
        {
            $dec = GnutecaUtils::decodeJsArgs($pn_page);
            $params['pn_page'] = $dec->pn_page;
        }
        
        $url = GnutecaUtils::getAjax('searchFunction', $params);
        return $url;
    }
}

class GGridActionIcon extends MGridAction
{
    public $path;
    public $visible = true;
    public $imageName = true;

    public function __construct($grid, $value, $href, $alt = null, $imageName = null)
    {
        parent::__construct($grid, 'image', $alt, $value, $href);

        if ( !stripos($value, 'png'))
        {
            $this->path[true]  = $this->manager->getUI()->getImageTheme( $this->manager->theme->id, $this->value.'.png');
            $this->path[false] = $this->manager->getUI()->getImageTheme( $this->manager->theme->id, $this->value.'.png');;
        }
        else
        {
            $this->path[true]  = $value;
            $this->path[false] = $value;
        }

        $this->imageName = $imageName;
    }

    public function setVisible($visible)
    {
        $this->visible = $visible;
    }

    public function show()
    {
        $this->setVisible(true);
    }

    public function hide()
    {
        $this->setVisible(false);
    }

    public function getVisible($visible)
    {
        return $this->visible;
    }

    public function generate()
    {
        $path   = $this->path[$this->enabled];
        $class  = "m-grid-action-icon";

        if ($this->enabled)
        {
            $row = $this->grid->data[$this->grid->currentRow];
            $href = $this->generateLink( $row );
            $control = new MImageButton( '' , $this->alt, $href, $path);
            $control->addAttribute('alt', $this->alt);
            $control->addAttribute('title', $this->alt);
        }
        else
        {
            $control = new MImage( '' , $this->alt, $path);
            $control->addAttribute('title', $this->alt);
            $control->addStyle('opacity', '0.3');
            $control->addStyle('filter', 'Alpha(opacity=30)');
        }

        if ( !$this->visible )
        {
            $control->addStyle('display','none');
        }
        
        return $control;
    }
}
?>
