<?php
define('PN_PAGE', 'pn_page');
/**
 * <--- Copyright 2005-2011 de Solis - Cooperativa de Solues Livres Ltda. e
 * Univates - Centro Universitrio.
 * 
 * Este arquivo  parte do programa Gnuteca.
 * 
 * O Gnuteca  um software livre; voc pode redistribu-lo e/ou modific-lo
 * dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao
 * do Software Livre (FSF); na verso 2 da Licena.
 * 
 * Este programa  distribudo na esperana que possa ser til, mas SEM
 * NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO
 * ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU/GPL em
 * portugus para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo
 * "LICENCA.txt", junto com este programa, se no, acesse o Portal do Software
 * Pblico Brasileiro no endereo www.softwarepublico.gov.br ou escreva para a
 * Fundao do Software Livre (FSF) Inc., 51 Franklin St, Fifth Floor, Boston,
 * MA 02110-1301, USA --->
 * 
 * Uma implementao de controles de navagao de pginas para grids
 */
class GnutecaGridNavigator extends MControl
{
    public $pageLength;
    public $pageNumber;
    public $action;
    public $range;
    public $rowCount;
    public $gridCount;
    public $pageCount;
    public $idxFirst;
    public $idxLast;
    public $showPageNo = false;
    public $linktype; // hyperlink or linkbutton
    public $grid;

    /**
     *
     * @param $length  Number of records per page
     * @param $total   Number total of records
     * @param $action  Action URL
     * @param $grid    The grid which contains this component
     * @return unknown_type
     */
    public function __construct($length = 20, $total = 0, $action = '?', $grid = NULL )
    {
        parent::__construct();
        $this->pageLength = $length;
        $this->gridCount = $length;
        $this->setRowCount($total);
        $this->grid = $grid;
        $this->setPageNumber( MIOLO::_REQUEST('pn_page') );
        $this->action = $action;
        $this->linktype = 'hyperlink';
    }


    public function setAction($url)
    {
        $this->action = $url;
    }

    public function setLinkType($linktype)
    {
        $this->linktype = $linktype;
    }

    public function setRowCount($rowCount)
    {
        $this->rowCount = $rowCount;
        $this->pageCount = ($this->pageLength > 0) ? (int)(($this->rowCount + $this->pageLength - 1) / $this->pageLength) : 1;
    }

    public function setGridCount($gridCount)
    {
        $this->gridCount = $gridCount;
    }

    public function setPageNumber($num)
    {
        $this->pageNumber = (int)($num ? $num : 1);
        $this->range = new MRange($this->pageNumber, $this->pageLength, $this->rowCount);
        $this->setIndexes();
    }

    public function setCurrentPage( $pageNumber )
    {
        $this->setPageNumber( $pageNumber );
    }

    public function setIndexes()
    {
      $this->range-> __construct($this->pageNumber, $this->pageLength, $this->rowCount);
      $this->idxFirst = $this->range->offset;
      $this->idxLast = $this->range->offset + $this->range->rows - 1;
      $this->setGridCount($this->range->rows);
    }

    public function setGridParameters($pageLength, $rowCount, $action, $grid)
    {
      $this->pageLength = $pageLength;
      $this->setRowCount($rowCount);
      $this->action = $action;
      $this->grid = $grid;
      $this->setIndexes();
    }

    public function getRowCount()
    {
        return $this->rowCount;
    }

    public function getGridCount()
    {
        return $this->gridCount;
    }

    public function getPageNumber()
    {
        return $this->pageNumber;
    }

    public function getPageCount()
    {
        return $this->pageCount;
    }

    public function getPagePosition($showPage = true)
    {
        $position = '[' . ($showPage ? _M('Page') : '') . ' ' . $this->getPageNumber() . ' ' . _M('of') . ' ' . $this->getPageCount() . "]";
        return $position;
    }

    public function getPageLinks($showPage = true, $limit = 10)
    {
    	return '';
    }

    public function getPageLink($showPage = true, $limit = 10)
    {
        $form       = $this->page->getFormId();
        $pageCount  = $this->getPageCount(); //quantidade de pginas
        $pageNumber = $this->getPageNumber(); //pgina atual
        $pageLinks  = array(); //array de links
        $div        = $this->grid->name;

        $p = 0;

        //aqui ele entra caso no tenha retornado dados na pesquisa
        if ( !$this->getRowCount() ) //quantidade de linhas
        {
            $pageLinks[$p] = new MLabel('&nbsp;&nbsp;&nbsp;');
            $pageLinks[$p++]->setClass('m-pagenavigator-text');
        }
        else
        {
            if ($showPage)
            {
                $pageLinks[$p] = new MText('', '&nbsp;Pgina:&nbsp;');
                $pageLinks[$p++]->setClass('m-pagenavigator-text');
            }

            if ($pageNumber <= $limit)
            {
                $o = 1;
            }
            else
            {
                $o = ceil( ( ($pageNumber - 1) / $limit)  * $limit );
                $other = $o-1;
                $pageLinks[$p] = new MLinkButton('', '...', "miolo.doAjax('searchFunction',new Array('$other', '$div'), '$form');" );
                $pageLinks[$p++]->setClass('m-pagenavigator-link');
            }

            //passa por todas pginas montando o array de links
            for ( $i = 0; ($i < $limit) && ($o <= $pageCount); $i++, $o++ )
            {
                $pg = $o;

                //se NO for a pgina selecionada
                if ( $o != $pageNumber )
                {
                    $pageLinks[$p] = new MLinkButton('', $pg, "miolo.doAjax('searchFunction',new Array('$o', '$div'), '$form');" );
                    $pageLinks[$p]->setClass('m-pagenavigator-link');
                    $pageLinks[$p++]->setAttribute('onMouseOver', "top.status='Pgina $pg'");
                }
                else
                {
                	//pgina atual
                    $pageLinks[$p] = new Label($pg );
                    $pageLinks[$p++]->setClass('m-pagenavigator-selected');
                }
            }

            if ( $o < $pageCount )
            {
                $pageLinks[$p++] = new MLabel('');
                $pageLinks[$p] = new MLinkButton('', '...', "miolo.doAjax('searchFunction',new Array('$o', '$div'), '$form');" );
                $pageLinks[$p++]->setClass('m-pagenavigator-link');
            }
        }

        return $pageLinks;
    }

    public function getPageRange($subject = '')
    {
        if (!$this->getRowCount())
        {
            $range = 'Nenhum dado';
        }
        else
        {
            $first = $this->idxFirst + 1;
            $last = $this->idxLast + 1;
            //$range = '[' . $first . '..' . $last . '] de ' . $this->getRowCount() . $subject;
            $range = $this->getPageLink(false) ;
        }

        return new MSpan('', $range, 'm-pagenavigator-range');
    }

    public function getPageRows($subject = '')
    {
        $rows = $this->getGridCount() . '&nbsp;' . $subject;
        return $rows;
    }

    protected function getSpanForPage( $id, $label, $pageNumber, $imageUrl )
    {
        //desabilita os links conforme a situao
        if ( $pageNumber == $this->getPageNumber() || $pageNumber > $this->getPageCount() || $pageNumber < 1 )
        {
            $btn = new MImage( $id, $label, str_replace('_x','', $imageUrl) ,array('border' => '0') );
        }
        else
        {
        	$div = $this->grid->name;
        	$btn  = new MImageButton($id, $label, "miolo.doAjax('searchFunction',new Array('$pageNumber', '$div'),'{$this->page->getFormId()}');" ,$imageUrl);
        }

        return new MSpan('', $btn, 'm-pagenavigator-image');
    }

    public function getPageFirst()
    {
        return $this->getSpanForPage( '_gnFirst', _M('Primeira'), 1, "images/but_pg_primeira_x.gif" );
    }

    public function getPagePrev()
    {
        return $this->getSpanForPage( '_gnPrev', _M('Anterior'), $this->getPageNumber() -1, "images/but_pg_anterior_x.gif" );
    }

    public function getPageNext()
    {
        return $this->getSpanForPage( '_gnNext', _M('Prxima'), $this->getPageNumber() +1, "images/but_pg_proxima_x.gif" );
    }

    public function getPageLast()
    {
        return $this->getSpanForPage( '_gnLast', _M('ltima'), $this->getPageCount(), "images/but_pg_ultima_x.gif" );
    }
}
?>
