<?php

/**
 * <--- Copyright 2005-2011 de Solis - Cooperativa de Solues Livres Ltda. e
 * Univates - Centro Universitrio.
 * 
 * Este arquivo  parte do programa Gnuteca.
 * 
 * O Gnuteca  um software livre; voc pode redistribu-lo e/ou modific-lo
 * dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao
 * do Software Livre (FSF); na verso 2 da Licena.
 * 
 * Este programa  distribudo na esperana que possa ser til, mas SEM
 * NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO
 * ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU/GPL em
 * portugus para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo
 * "LICENCA.txt", junto com este programa, se no, acesse o Portal do Software
 * Pblico Brasileiro no endereo www.softwarepublico.gov.br ou escreva para a
 * Fundao do Software Livre (FSF) Inc., 51 Franklin St, Fifth Floor, Boston,
 * MA 02110-1301, USA --->
 * 
 * Class
 *
 * @author Luiz Gilberto Gregory Filho [luz@solis.coop.br]
 *
 * @version $Id$
 *
 * \b Maintainers: \n
 * Eduardo Bonfandini [eduardo@solis.coop.br]
 * Jamiel Spezia [jamiel@solis.coop.br]
 * Luiz Gregory Filho [luiz@solis.coop.br]
 * Moises Heberle [moises@solis.coop.br]
 *
 * @since
 * Class created on 04/05/2009
 *
 **/

class GnutecaISO2709 extends MBusiness
{
    /**
     * Attributes
     */

    private $errorCode,
            $fieldDelimiter,
            $subFieldDelimiter,
            $recordDelimiter,
            $fileName;

    private $regSize            = 12,
            $record             = null,
            $defaultSubField    = 'a',
            $range              = null;

    /**
     * Constructor Method
     */
    function __construct($fileName = null, $fieldDelimiter = '#', $subFieldDelimiter = '$', $recordDelimiter = '#@')
    {
        parent::__construct();
        $this->setFileName          ($fileName);
        $this->setFieldDelimiter    ($fieldDelimiter);
        $this->setSubFieldDelimiter ($subFieldDelimiter);
        $this->setRecordDelimiter   ($recordDelimiter);
        $this->setRange();
    }




    /**
     * Seta o arquivo
     *
     * @param string $fileName
     */
    public function setFileName($fileName)
    {
        $this->fileName = $fileName;
    }




    /**
     * Seta o delimitador de Campos
     *
     * @param char $delimiterChar
     */
    public function setFieldDelimiter($delimiterChar = '#')
    {
        $this->fieldDelimiter = $delimiterChar;
    }




    /**
     * Seta o delimitador de subCampos
     *
     * @param char $delimiterChar
     */
    public function setSubFieldDelimiter($delimiterChar = '$')
    {
        $this->subFieldDelimiter = $delimiterChar;
    }




    /**
     * Seta o delimitador de subCampos
     *
     * @param char $delimiterChar
     */
    public function setRecordDelimiter($delimiterChar = '#@')
    {
        $this->recordDelimiter = $delimiterChar;
    }



    /**
     * Seta o range de importao.
     *
     * @param int $start
     * @param int $length
     */
    public function setRange($start = 0, $length = 10)
    {
        $this->range->start     = $start;
        $this->range->length    = $length;
    }



    /**
     * retorna o arquivo
     *
     */
    public function getFileName()
    {
        return $this->fileName;
    }




    /**
     * retorna delimitador de Campos
     *
     */
    public function getFieldDelimiter()
    {
        return $this->fieldDelimiter;
    }




    /**
     * retorna o delimitador de subCampos
     *
     */
    public function getSubFieldDelimiter()
    {
        return $this->subFieldDelimiter;
    }




    /**
     * retorna o delimitador de subCampos
     *
     */
    public function getRecordDelimiter()
    {
        return $this->recordDelimiter;
    }



    /**
     * retorna o range de importao.
     *
     */
    public function getRange()
    {
        return $this->range;
    }



    /**
     * importa o conteudo do arquivo para um array
     *
     */
    public function import()
    {
        if(!$this->checkFile())
        {
            return false;
        }

        $fd = @fopen ($this->getFileName(), "r");

        if (!$fd)
        {
            $this->errorCode = 3;
            return false;
        }

        $nRec = 0;
        $this->record = null;
        $registro = '';
        $range = $this->getRange();

        while (!feof ($fd))
        {
            $buffer     = '';
            $buffer     = fgets($fd, 80);
            $buffer     = ereg_replace("\n", '', $buffer);
            $buffer     = ereg_replace(chr(13), '', $buffer);

            $registro  .= $buffer;

            if (ereg($this->getRecordDelimiter(), $buffer))
            {
                if($nRec < $range->start)
                {
                    $nRec++;
                    $registro = '';
                    continue;
                }

                $registro = substr($registro, 0, strpos($registro, $this->getRecordDelimiter()));
                $registro = str_replace($this->getRecordDelimiter(), '', $registro);
                $this->record[$nRec++] = $this->makeRecord($registro);
                $registro = '';
            }

            if($nRec >= ($range->start + $range->length))
            {
                break;
            }
        }

        fclose ($fd);

        return true;
    }




    /**
     * retorna os registros
     *
     * @return array
     */
    public function getRecord()
    {
        return $this->record;
    }



    /**
     * Trabalha os registros e transforma em array
     *
     * @param text $registro
     */
    protected  function makeRecord($registro)
    {
        // fonte pesquisa : http://www.aix.com.br/wiki/index.php?title=Importar/Remover_Arquivo_ISO-2709

        $returnRec      = null;
        $record         = explode($this->getFieldDelimiter(), $registro);

        $control        = $record[0];

        $controlIndex   = 0;
        $recordSize     = substr($control, ($this->regSize * $controlIndex++), $this->regSize);
        $codeListSize   = substr($control, ($this->regSize * $controlIndex++), $this->regSize);
        $nFields        = (strlen($control) / $this->regSize);

        // remove o primeiro indice do array record que corrresponde as controladores.
        array_shift(&$record);

        $x = 0;
        for( ; $controlIndex < $nFields; $controlIndex++)
        {
            $fieldControl = substr($control, ($this->regSize * $controlIndex), $this->regSize);
            $field        = substr($fieldControl, 0, 3);
            $contetSize   = substr($fieldControl, 3, 4);

            if(!strlen($field))
            {
                continue;
            }

            $returnRec[$field]          = null;
            $returnRec[$field]->ind1    = '';
            $returnRec[$field]->ind2    = '';
            $line = 0;

            $subFexplode        = explode($this->getSubFieldDelimiter(), $record[$x]);

            if(strlen($subFexplode[0]))
            {
                $returnRec[$field]->subfields[$this->defaultSubField]->content[$line]  = $subFexplode[0];
            }
            elseif(!strlen($subFexplode[0]))
            {
                for ($y = 1; $y < count($subFexplode); $y++ )
                {
                    $subF = substr($subFexplode[$y],0,1);
                    $subC = substr($subFexplode[$y],1, strlen($subFexplode[$y]));

                    $returnRec[$field]->subfields[$subF]->content[$line]   = $subC;
                }
            }

            $x++;
        }

        return $returnRec;
    }


    /**
     * verifica se o arquivo existe
     *
     */
    protected function checkFile()
    {
        if(!strlen($this->getFileName()))
        {
            $this->errorCode = 1;
            return false;
        }

        if(!file_exists($this->getFileName()))
        {
            $this->errorCode = 2;
            return false;
        }

        return true;
    }



    /**
     * Retorna a string do erro
     *
     * @return string
     */
    public function getErrorStr()
    {
        if(is_null($this->getErrorCode()))
        {
            return null;
        }

        switch ($this->getErrorCode())
        {
            case 1: return "Arquivo Invlido! File: '". $this->getFileName() ."'";
            case 2: return "Arquivo Inesistente! File: '". $this->getFileName() ."'";
            case 3: return "No foi possvel ler o arquivo: '". $this->getFileName() ."'";
        }
    }


    /**
     * retorna o codigo do erro;
     *
     * @return integer
     */
    public function getErrorCode()
    {
        return $this->errorCode;
    }

} // final da classe

?>
