<?php

/**
 * <--- Copyright 2005-2011 de Solis - Cooperativa de Solues Livres Ltda. e
 * Univates - Centro Universitrio.
 * 
 * Este arquivo  parte do programa Gnuteca.
 * 
 * O Gnuteca  um software livre; voc pode redistribu-lo e/ou modific-lo
 * dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao
 * do Software Livre (FSF); na verso 2 da Licena.
 * 
 * Este programa  distribudo na esperana que possa ser til, mas SEM
 * NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO
 * ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU/GPL em
 * portugus para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo
 * "LICENCA.txt", junto com este programa, se no, acesse o Portal do Software
 * Pblico Brasileiro no endereo www.softwarepublico.gov.br ou escreva para a
 * Fundao do Software Livre (FSF) Inc., 51 Franklin St, Fifth Floor, Boston,
 * MA 02110-1301, USA --->
 * 
 * Class
 *
 * @author Luiz Gilberto Gregory Filho [luz@solis.coop.br]
 *
 * @version $Id$
 *
 * \b Maintainers: \n
 * Eduardo Bonfandini [eduardo@solis.coop.br]
 * Jamiel Spezia [jamiel@solis.coop.br]
 * Luiz Gregory Filho [luiz@solis.coop.br]
 * Moises Heberle [moises@solis.coop.br]
 *
 * @since
 * Class created on 06/ago/2008
 *
 **/

class GnutecaMail
{
    private $address;                           // array
    private $attachment;                        // array
    private $content;                           // string
    private $subject;                           // string
    private $from;                              // string
    private $fromName;                          // string
    private $user;                              // string
    private $password;                          // string
    private $host;                              // string
    private $port;                              // int
    private $authenticate   = false;            // boolean
    private $isHTML         = true;             // boolean

    private $mailer;                            // Object PHP Mailer

    private $log = false,
            $logFile;

    function __construct()
    {
        /**
         * Include phpmailer class
         * dirname(__FILE__) = full path this file
         */
        include_once( dirname(__FILE__) . "/phpmailer/class.phpmailer.php");

        //Defaults
        $this->setHost(EMAIL_SMTP);
        $this->setPort(EMAIL_PORT);
        $this->setFrom(EMAIL_FROM);
        $this->setFromName(EMAIL_FROM_NAME);
        $this->setAuthenticate(EMAIL_AUTHENTICATE);
        $this->setUser(EMAIL_USER);
        $this->setPassword(EMAIL_PASSWORD);
        $this->setContentType(EMAIL_CONTENT_TYPE);

        $this->mailer = new PHPMailer ();
        $this->mailer->SetLanguage('br');
    }


    /**
     * Retorna o arquivo de log
     *
     */
    private function getLogFilePath()
    {
        if(strlen($this->logFile))
        {
            return;
        }

        // VERIFICA CONFIGURAO DO LOG
        if(!defined('MAIL_LOG_GENERATE') || MAIL_LOG_GENERATE != 't')
        {
            return;
        }

        $MIOLO          = MIOLO::getInstance();
        $this->logFile  = $MIOLO->getConf('home.logs')."/";
        $this->logFile .= defined('MAIL_LOG_FILE_NAME') ? MAIL_LOG_FILE_NAME : "gnuteca3-mail.log";
    }



    /**
     * Este mtodo incrementa o array de destinatrios
     *
     * @param (String) $address
     * @return void
     */
    public function addAddress($address)
    {
        if(!ereg("@", $address))
        {
            return;
        }

        if(ereg(",", $address))
        {
            $a = explode(",", $address);
            foreach($a as $v)
            {
                $this->addAddress($v);
            }
        }
        else
        {
            $this->address[] = trim($address);
        }
    }



    /**
     * Define um destinatrio
     *
     * @param $address (String)
     */
    public function setAddress($address)
    {
        unset($this->address);
        $this->addAddress($address);
    }



    /**
     * Adiciona um anexo
     *
     * @param String - full file path
     */
    public function addAttachment($pathFile)
    {
        $pathFile = trim($pathFile);

        if(ereg(",", $pathFile))
        {
            $a = explode(",", $pathFile);
            foreach($a as $v)
            {
                $this->addAttachment($v);
            }
        }
        else
        {
            if(file_exists($pathFile))
            {
                $this->attachment[] = $pathFile;
            }
        }
    }



    /**
     * seta o conteudo que sera encaminhado pelo email
     *
     * @param String
     */
    public function setContent($content)
    {
        $this->content = $content;
    }




    /**
     * seta o assunto do email
     *
     * @param String
     */
    public function setSubject($subject)
    {
        $this->subject = EMAIL_SUBJECT_PREFIX . ' ' . $subject;
        $this->subject = str_replace('  ', ' ', $this->subject);
    }




    /**
     * Seta o Usuario
     *
     * O usurio  necessario caso o metodo de envio seja atutenticado
     *
     * @param String
     */
    public function setUser($user)
    {
        $user       = str_replace(array("\n", "\t", "\r"), "", $user);
        $user       = trim($user);
        $this->user = $user;
    }




    /**
     * Seta o password
     *
     * este parametro  necessario caso o metodo de envio seja autenticado
     *
     * @param String
     */
    public function setPassword($password)
    {
        $this->password = $password;
    }




    /**
     * Seta o servidor
     *
     * @param String
     */
    public function setHost($host)
    {
        $this->host = $host;
    }





    /**
     * Seta a porta de conexao
     *
     * @param Int
     */
    public function setPort($port)
    {
        $this->port = $port;
    }



    /**
     * Seta o remetente
     *
     * @param String
     */
    public function setFrom($from)
    {
        $from       = str_replace(array("\n", "\t", "\r"), "", $from);
        $from       = trim($from);
        $this->from = $from;
    }




    /**
     * Seta o nome do rementente
     *
     * @param String
     */
    public function setFromName($fromName)
    {
        $this->fromName = $fromName;
    }




    /**
     *
     * Seta se o metodo de conexao com o server  autenticado ou no
     *
     * @param (Boolean)
     */
    public function setAuthenticate($authenticate)
    {
        if(!is_bool($authenticate))
        {
            $authenticate = (array_search($authenticate, array('1', 't', 'true','yes')) !== false);
        }
        $this->authenticate = $authenticate;
    }




    /**
     * Seta o tipo do conteudo que ser enviado (HTML|TEXT)
     *
     * @param Boolean
     */
    public function setIsHtml($isHtml)
    {
        $this->isHTML = $isHtml;
    }



    /**
     * Seta a liguagem do conteudo
     *
     * @param varchar $type
     */
    public function setContentType($type = 'html')
    {
        switch ($type)
        {
            default:
                $this->setIsHtml(true);
        }
    }


    /**
     *
     * Retorna os destinatrios
     *
     * @return (Array)
     */
    public function getAddress()
    {
        return $this->address;
    }



    /**
     * Retorna os anexos
     *
     * @return (Array)
     */
    public function getAttachment()
    {
        return $this->attachment;
    }



    /**
     * retorna o remetente
     *
     * @return String
     */
    public function getFrom()
    {
        return $this->from;
    }



    /**
     * retorna o nome do remetente
     *
     * @return String
     */
    public function getFromName()
    {
        return $this->fromName;
    }




    /**
     * retorna o Assunto do email
     *
     * @return String
     */
    public function getSubject()
    {
        return $this->subject;
    }




    /**
     * retorna o Conteudo do email
     *
     * @return String
     */
    public function getContent()
    {
        if ($this->getIsHTML())
        {
            $this->content = str_replace("\n", "<br>", $this->content);
        }
        return $this->content;
    }




    /**
     * retorna o usuario
     *
     * @return String
     */
    public function getUser()
    {
        return $this->user;
    }




    /**
     * retorna o password
     *
     * @return String
     */
    public function getPassword()
    {
        return $this->password;
    }




    /**
     * retorna o host
     *
     * @return String
     */
    public function getHost()
    {
        return $this->host;
    }




    /**
     * retorna a porta
     *
     * @return Integer
     */
    public function getPort()
    {
        return $this->port;
    }




    /**
     * retorna se o metodo de conexao  autenticado (authenticate true|false)
     *
     * @return (Boolean)
     */
    public function getAuthenticate()
    {
        return $this->authenticate;
    }




    /**
     * retorna se o tipo de texto  html (isHTML true|false)
     *
     * @return (Boolean)
     */

    public function getIsHtml()
    {
        return $this->isHTML;
    }




    /**
     * envia o email
     *
     * @return Boolean
     */
    public function send()
    {
        $this->mailer->ClearAddresses();

        $this->mailer->From         = $this->getFrom        ();
        $this->mailer->FromName     = $this->getFromName    ();
        $this->mailer->Subject      = $this->getSubject     ();
        $this->mailer->Body         = $this->getContent     ();
        $this->mailer->Host         = $this->getHost        ();
        $this->mailer->Port         = $this->getPort        ();
        $this->mailer->SMTPAuth     = $this->getAuthenticate();
        $this->mailer->Username     = $this->getUser        ();
        $this->mailer->Password     = $this->getPassword    ();

        $this->mailer->IsHTML       ($this->getIsHtml());
        $this->mailer->IsSMTP();

        // VERIFICA SE  TESTES E SUBTITUI OS ENDEREOS PELO EMAIL DE TESTE
        if(EMAIL_TESTING == 't' && strlen(EMAIL_TEST_RECEIVE))
        {
            $this->setAddress(EMAIL_TEST_RECEIVE);
        }

        if(!is_null($this->getAddress()))
        {
            foreach($this->getAddress() as $email)
            {
                $this->mailer->AddAddress(trim($email));
            }
        }

        if(!is_null($this->getAttachment()))
        {
            foreach($this->getAttachment() as $anexo)
            {
                $this->mailer->AddAttachment($anexo);
            }
        }

        $send = $this->mailer->Send();
        $this->recordLog($send);
        return $send;
    }


    /**
     * Escreve o arquivo de log
     *
     * @param resultado do envio (Boolean) $result
     */
    private function recordLog($result)
    {
        $this->getLogFilePath();

        if(!strlen($this->logFile))
        {
            return;
        }

        $recordSeparator    = "+----------------------------------------------------------------+";

        $content = "\n{$recordSeparator}\n";
        $content.= ($result) ? "E-mail foi enviado com sucesso!\n" : "No foi possvel enviar o e-mail.\n";
        $content.= "Destino: '". (!is_null($this->getAddress()) ? implode(",\n\t", $this->getAddress()) : "null") ."'\n";
        $content.= "Data/Hora: '". date("d/m/Y H:i:s") ."'\n";
        $content.= "ContentType: '". $this->mailer->ContentType ."'\n";
        $content.= "Authenticate: '". $this->getAuthenticate() ."'\n";
        $content.= "Host: '". $this->getHost() .":". $this->getPort() ."'\n";
        $content.= "User: '". $this->getUser() ."'\n";
        $content.= "Password: '". $this->getPassword() ."'\n";
        $content.= "From: '". $this->getFrom() ."'\n";
        $content.= "From Name: '". $this->getFromName() ."'\n";
        $content.= "Subject: '". $this->getSubject() ."'\n";
        $content.= "Content: '". $this->getContent() ."'\n";
        if(!$result)
        {
            $content.=  "\n". $this->mailer->ErrorInfo ."\n";
        }

        file_put_contents($this->logFile, $content, FILE_APPEND);
    }


    public function getErrorInfo()
    {
    	return $this->mailer->ErrorInfo;
    }

    public function getError()
    {
        return $this->getErrorInfo();
    }
}
?>