<?php

/**
 * <--- Copyright 2005-2011 de Solis - Cooperativa de Solues Livres Ltda. e
 * Univates - Centro Universitrio.
 * 
 * Este arquivo  parte do programa Gnuteca.
 * 
 * O Gnuteca  um software livre; voc pode redistribu-lo e/ou modific-lo
 * dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao
 * do Software Livre (FSF); na verso 2 da Licena.
 * 
 * Este programa  distribudo na esperana que possa ser til, mas SEM
 * NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO
 * ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU/GPL em
 * portugus para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo
 * "LICENCA.txt", junto com este programa, se no, acesse o Portal do Software
 * Pblico Brasileiro no endereo www.softwarepublico.gov.br ou escreva para a
 * Fundao do Software Livre (FSF) Inc., 51 Franklin St, Fifth Floor, Boston,
 * MA 02110-1301, USA --->
 * 
 * Class GnutecaPDF
 *
 * @author Luiz Gilberto Gregory Filho [luz@solis.coop.br]
 *
 * @version $Id$
 *
 * \b Maintainers: \n
 * Eduardo Bonfandini [eduardo@solis.coop.br]
 * Jamiel Spezia [jamiel@solis.coop.br]
 * Luiz Gregory Filho [luiz@solis.coop.br]
 * Moises Heberle [moises@solis.coop.br]
 *
 * @since
 * Class created on 16/02/2009
 *
 **/
$MIOLO  = MIOLO::getInstance();
$module = MIOLO::getCurrentModule();
$MIOLO->Uses('classes/fpdf/fpdf.php', $module);

class GnutecaPDF extends FPDF
{
    private $filePath; //caminho absoluto, evite defini-lo direramente
    public $filename; //o ome do arquivo exemplo file.pdf
    public $folder = 'pdf'; //a pasta onde o arquivo deve ser salvo
    public $absolutePath;
    public $textContent;
    public $lineHeight = 4;
    public $linePage   = 70;
    public $pdf;
    public $MIOLO;
    public $module;
    public $busFile;

    /**
     * Construtor
     *
     * @param string $filePath evite definir o filePath
     */
    function __construct($filePath = null, $orientation = 'P', $unit = 'mm', $format = 'A4')
    {
        $this->setFilePath($filePath);

        $this->MIOLO    = MIOLO::getInstance();
        $this->module   = MIOLO::getCurrentModule();
        $this->busFile = new BusinessGnuteca3BusFile();

        parent::FPDF($orientation, $unit, $format);
        $this->setAuthor('Gnuteca');
        $this->setCreator('Gnuteca');
        $this->setFont('Courier', '', 11);
        $this->setMargins(10, 30, 0);
    }

    /**
     * Enter description here...
     *
     * @param unknown_type $filePath
     */
    public function setFilePath($filePath)
    {
        $this->filePath = $filePath;
    }

    /**
     * retorna o nome do arquivo gerado
     *
     * @return string
     */
    public function getFilePath()
    {
        return $this->filePath;
    }

    public function setFilename($filename)
    {
        $this->filename = $filename;
    }


    public function getFilename()
    {
        return $this->filename;
    }

    /**
     * seta o height entre as linhas
     *
     * @param int $height
     */
    public function setLineHeight($height)
    {
        $this->lineHeight = $height;
    }

    /**
     * seta o conteudo do arquivo
     *
     * @param text $textContent
     */
    public function setTextContent($textContent)
    {
        $this->textContent = $textContent;
    }

    /**
     * Seta numero de linhas de uma pagina.
     *
     * @param int $linePage
     */
    public function setMaxLinePage($linePage)
    {
        $this->linePage = $linePage;
    }

    /**
     * gera o conteudo do pdf
     *
     * @return boolean
     */
    public function generate($processContent = true)
    {
        if ( !$this->filePath )
        {
            $this->filePath = $this->busFile->getAbsoluteFilePath($this->folder, $this->filename);
        }

    	//testa se o diretrio a ser gravado possui permisso de escrita
    	if ( !is_writable($this->busFile->getAbsoluteFilePath($folder) ) )
        {
        	return false;
        }
        else
        {
	    	if ($processContent)
	    	{
                $this->addPage();

		        $lines  = explode("\n", $this->textContent);
		        $lh     = $this->lineHeight;

		        foreach($lines as $n => $l)
		        {
		            $this->Text(30, $lh, $l);

		            $lh += $this->lineHeight;

		            if($lh >= ($this->linePage * $this->lineHeight))
		            {
		                $this->addPage();
		                $lh = $this->lineHeight;
		            }
		        }
	    	}

	        $this->output($this->filePath, 'F');
	        return true;
        }
    }

    /**
     * Seta a fonte como negrito ou no
     *
     * @param $bold
     * @return unknown_type
     */
    public function setBold( $bold = true)
    {
        $this->SetFont( null, $bold ? 'B' : '' );
    }

    /**
     * Remove o arquivo...
     */
    public function delete()
    {
        @unlink($this->filePath);
    }

    /**
     * Get download link of the report file
     *
     * @return String
     */
    public function getDownloadURL()
    {
        $file = $this->busFile->getFile( $this->folder.'/'.$this->filename );
        return $file->mioloLink;
    }

    public function showDownloadInfo()
    {
        $MIOLO = MIOLO::getInstance();
        $MIOLO->getClass('gnuteca3', 'controls/FileUploader');
        $file = $this->busFile->getFile($this->folder.'/'.$this->filename);
        FileUploader::downloadFile($file);
    }
}
?>
