<?php

/**
 * <--- Copyright 2005-2011 de Solis - Cooperativa de Solues Livres Ltda. e
 * Univates - Centro Universitrio.
 * 
 * Este arquivo  parte do programa Gnuteca.
 * 
 * O Gnuteca  um software livre; voc pode redistribu-lo e/ou modific-lo
 * dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao
 * do Software Livre (FSF); na verso 2 da Licena.
 * 
 * Este programa  distribudo na esperana que possa ser til, mas SEM
 * NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO
 * ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU/GPL em
 * portugus para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo
 * "LICENCA.txt", junto com este programa, se no, acesse o Portal do Software
 * Pblico Brasileiro no endereo www.softwarepublico.gov.br ou escreva para a
 * Fundao do Software Livre (FSF) Inc., 51 Franklin St, Fifth Floor, Boston,
 * MA 02110-1301, USA --->
 * 
 * Class
 *
 * @author Luiz Gilberto Gregory Filho [luz@solis.coop.br]
 *
 * @version $Id$
 *
 * \b Maintainers: \n
 * Eduardo Bonfandini [eduardo@solis.coop.br]
 * Jamiel Spezia [jamiel@solis.coop.br]
 * Luiz Gregory Filho [luiz@solis.coop.br]
 * Moises Heberle [moises@solis.coop.br]
 * Jader Osvino Fiegenbaum [jader@solis.coop]
 *
 * @since
 * Class created on 08/09/2009
 *
 **/

class GnutecaSendMail
{
	public  $MIOLO, $module;
	public  $busLibraryUnitConfig;
	public  $busExemplaryControl;
	public  $busLibraryUnit;
	public  $busLoanBetweenLibrary;
	public  $mailTo;
	private $mail;

	function __construct()
	{
		$this->MIOLO    = MIOLO::getInstance();
		$this->module   = MIOLO::getCurrentModule();

		$this->busLibraryUnitConfig = $this->MIOLO->getBusiness($this->module, 'BusLibraryUnitConfig');
		$this->busExemplaryControl  = $this->MIOLO->getBusiness($this->module, 'BusExemplaryControl');
		$this->busLibraryUnit       = $this->MIOLO->getBusiness($this->module, 'BusLibraryUnit');
		$this->busLoanBetweenLibrary= $this->MIOLO->getBusiness($this->module, 'BusLoanBetweenLibrary');
		$this->MIOLO->getClass($this->module, 'GnutecaMail');
		$this->MIOLO->getClass($this->module, 'GnutecaFunction');
	}



	/**
	 * Este mtodo encaminha um email para o administrador informando que uma requisio foi cancelada.
	 */
	public function sendMailToAdminCancelRequestChangeExemplaryStatus( $requestChangeExemplaryStatusId )
	{
		$busRequestChangeExemplaryStatus    = $this->MIOLO->getBusiness($this->module, 'BusRequestChangeExemplaryStatus' );
		$libraryUnit                        = $busRequestChangeExemplaryStatus->getLibraryUnit($requestChangeExemplaryStatusId);

		$gf = new GnutecaFunction();
		$gf->setVariable('$LN', "\n");
		$gf->setVariable('$SP', " ");
		$gf->setVariable('$REQUEST_ID', $requestChangeExemplaryStatusId);

		$to         = $this->getAdminEmailTo($libraryUnit->libraryUnitId, "EMAIL_ADMIN_REQUEST_CHANGE_EXEMPLARY_STATUS");
		$subject    = $this->busLibraryUnitConfig->getValueLibraryUnitConfig($libraryUnit->libraryUnitId, 'EMAIL_CANCEL_SUBJECT_REQUEST_CHANGE_EXEMPLARY_STATUS');
		$content    = $this->busLibraryUnitConfig->getValueLibraryUnitConfig($libraryUnit->libraryUnitId, 'EMAIL_CANCEL_CONTENT_REQUEST_CHANGE_EXEMPLARY_STATUS');

		$mail = new GnutecaMail();
		$mail->setAddress($to);
		$mail->setSubject($subject);
		$mail->setContent($gf->interpret($content));
		$mail->send();
	}




	/**
	 * Email que comunica o administrado da biblioteca sobre uma nova requisio de emprestimo entre biblioteca
	 *
	 * @param integer $loanBetweenLibraryId
	 * @return boolean
	 */
	public function sendMailLoanBetweenLibraryRequest($loanBetweenLibraryId)
	{
		return $this->sendMailLoanBetweenLibrary($loanBetweenLibraryId, "request");
	}


	/**
	 * Email que comunica o administrado da biblioteca sobre uma requisio de emprestimo entre biblioteca que foi cancelada
	 *
	 * @param integer $loanBetweenLibraryId
	 * @return boolean
	 */
	public function sendMailLoanBetweenLibraryCancel($loanBetweenLibraryId)
	{
		return $this->sendMailLoanBetweenLibrary($loanBetweenLibraryId, "cancel", null);
	}


	/**
	 * Email que comunica o administrado da biblioteca sobre uma requisio de emprestimo entre biblioteca que foi cancelada
	 *
	 * @param integer $loanBetweenLibraryId
	 * @return boolean
	 */
	public function sendMailLoanBetweenLibraryReturnMaterial($loanBetweenLibraryId, $exemplaries)
	{
		return $this->sendMailLoanBetweenLibrary($loanBetweenLibraryId, "return", $exemplaries);
	}


	/**
	 * Email que comunica o administrado da biblioteca sobre uma requisio de emprestimo entre biblioteca que foi cancelada
	 *
	 * @param integer $loanBetweenLibraryId
	 * @return boolean
	 */
	public function sendMailLoanBetweenLibraryApproveMaterial($loanBetweenLibraryId, $exemplaries)
	{
		return $this->sendMailLoanBetweenLibrary($loanBetweenLibraryId, "approve", $exemplaries);
	}



	/**
	 * Email que comunica o administrado da biblioteca sobre uma requisio de emprestimo entre biblioteca que foi cancelada
	 *
	 * @param integer $loanBetweenLibraryId
	 * @return boolean
	 */
	public function sendMailLoanBetweenLibraryDisapproveMaterial($loanBetweenLibraryId)
	{
		return $this->sendMailLoanBetweenLibrary($loanBetweenLibraryId, "disapprove");
	}



	/**
	 * Envia um email para o administrador da biblioteca sobre emprestimos entre bibliotecas
	 *
	 * @param integer $loanBetweenLibraryId
	 * @return boolean
	 */
	public function sendMailLoanBetweenLibrary($loanBetweenLibraryId, $type = "request", $auxArgs = null)
	{
		$busLoanBetweenLibrary  = $this->MIOLO->getBusiness($this->module, 'BusLoanBetweenLibrary');
		$busExemplaryControl    = $this->MIOLO->getBusiness($this->module, 'BusExemplaryControl');
		$busLibraryUnit         = $this->MIOLO->getBusiness($this->module, 'BusLibraryUnit');

		$loanBetweenLibrary     = $busLoanBetweenLibrary->getLoanBetweenLibrary($loanBetweenLibraryId);

		if(!$loanBetweenLibrary || !$loanBetweenLibrary->libraryComposition)
		{
			return false;
		}

		$composition = $loanBetweenLibrary->libraryComposition;

		//Group by library
		$libraryItems = array();
		foreach ($composition as $ex)
		{
			$add = false;

			switch ($type)
			{
				case "return"   :
				case "approve"  :
					if(in_array($ex->itemNumber, $auxArgs))
					{
						$add = true;
					}
					break;

				default:
					$add = true;
					break;
			}

			if($add)
			{
				$libraryUnitId = $this->busExemplaryControl->getExemplaryControl($ex->itemNumber);
                //Se reprovar solicitao, enviar s um aviso
				if ($type == "disapprove")
                {
                    $libraryItems[1][] = $ex;
                }
                else
                {
                    $libraryItems[$libraryUnitId->originalLibraryUnitId][] = $ex;
                }
			}
		}

		foreach ($libraryItems as $libraryUnitId => $exemplaryes)
		{
			$materials = array();
			foreach ($exemplaryes as $ex)
			{
				$info = $busExemplaryControl->getExemplaryControl($ex->itemNumber, TRUE);
				$materials[$ex->itemNumber] = _M('Item Number', $this->module)       . ": $ex->itemNumber\n";
				$materials[$ex->itemNumber].= _M('Title', $this->module)             . ": $info->title\n";
				$materials[$ex->itemNumber].= _M('Author', $this->module)            . ": $info->author\n";
				$materials[$ex->itemNumber].= _M('Classification', $this->module)    . ": $info->classification $info->cutter\n";
			}

			//$to      = $this->getAdminEmailTo($libraryUnitid, 'EMAIL_ADMIN_LOAN_BETWEEN_LIBRARY');
			//Pega a biblioteca de acordo com a funo executada
			$libraryUnitIdRequest = $this->busLoanBetweenLibrary->getLibraryId($loanBetweenLibraryId);
		    if ( ($type == 'approve') || ($type == 'disapprove') )
            {
                $libraryUnitIdRequest = $libraryUnitId;
            	$libraryUnitId = $this->busLoanBetweenLibrary->getLibraryId($loanBetweenLibraryId);
            }

			$to      = $this->getEmailTo($loanBetweenLibraryId, $libraryUnitId, $type, $exemplaryes);

			$subject = $this->busLibraryUnitConfig->getValueLibraryUnitConfig($libraryUnitIdRequest, 'EMAIL_LOANBETWEENLIBRARY_SUBJECT');

			$action = "";

			switch ($type)
			{
				case 'cancel'       :
					$content    = $this->busLibraryUnitConfig->getValueLibraryUnitConfig($libraryUnitIdRequest, 'EMAIL_LOANBETWEENLIBRARY_CANCEL_CONTENT');
					$status     = _M('Canceled', $this->module);
					break;

				case 'request'      :
					$content    = $this->busLibraryUnitConfig->getValueLibraryUnitConfig($libraryUnitIdRequest, 'EMAIL_LOANBETWEENLIBRARY_REQUEST_CONTENT');
					$status     = _M('Requested', $this->module);
					break;

				case 'return'       :
					$content    = $this->busLibraryUnitConfig->getValueLibraryUnitConfig($libraryUnitIdRequest, 'EMAIL_LOANBETWEENLIBRARY_RETURNMATERIAL_CONTENT');
					$status     = _M('Return material', $this->module);
					break;

				case 'approve'      :
					$content    = $this->busLibraryUnitConfig->getValueLibraryUnitConfig($libraryUnitIdRequest, 'EMAIL_LOANBETWEENLIBRARY_CONFIRMLOAN_CONTENT');
					$status     = _M('Approve material',    $this->module);
					$action     = _M('approved',            $this->module);
					break;

				case 'disapprove'   :
					$content    = $this->busLibraryUnitConfig->getValueLibraryUnitConfig($libraryUnitIdRequest, 'EMAIL_LOANBETWEENLIBRARY_CONFIRMLOAN_CONTENT');
					$status     = _M('Disapprove material', $this->module);
					$action     = _M('not approved',        $this->module);
					break;
			}


			$library = $busLibraryUnit->getLibraryUnit($libraryUnitIdRequest);

			$gf = new GnutecaFunction();
			$gf->setVariable('$LN', "\n");
			$gf->setVariable('$STATUS',                     $status);
			$gf->setVariable('$ACTION',                     $action);
			$gf->setVariable('$LIBRARY_UNIT_DESCRIPTION',   $library->libraryName);
			$gf->setVariable('$MATERIALS',                  implode("\n", $materials));
			$gf->setVariable('$LOAN_DATE',                  GnutecaDate::toUser($loanBetweenLibrary->loanDate, 'DATE'));
			$gf->setVariable('$RETURN_FORECAST_DATE',       GnutecaDate::toUser($loanBetweenLibrary->returnForecastDate, 'DATE'));

			$mail = new GnutecaMail();
			$mail->setSubject($gf->interpret($subject));
			$mail->setContent($gf->interpret($content));
			$mail->setAddress($to);
			$mail->send();
		}
	}


	/**
	 * Pega e-mail do emprstimo entre bibliotecas de acordo com as aes
	 */
	public function getEmailTo($loanBetweenLibraryId, $libraryUnitId, $type)
	{
		if ( ($type == 'request') || ($type == 'return') || ($type == 'cancel') )
		{
			$emailLibrary  = $this->busLibraryUnit->getLibraryUnit1($libraryUnitId);
			return $emailLibrary->email;
		}
		elseif ( ($type == 'approve') || ($type == 'disapprove'))
		{
			$libraryUnitId = $this->busLoanBetweenLibrary->getLibraryId($loanBetweenLibraryId);
			$emailLibrary  = $this->busLibraryUnit->getLibraryUnit1($libraryUnitId);
			return $emailLibrary->email;
		}
		else
		{
			return EMAIL_ADMIN_LOAN_BETWEEN_LIBRARY;
		}
	}



	/**
	 * Envia um email comunicando o usuario que sua reserva foi atendida
	 *
	 * @param integer $reserveId
	 */
	public function sendMailToUserReserveAnswered($reserveObj, $personName, $personMail)
	{
		$busReserve             = $this->MIOLO->getBusiness($this->module, 'BusReserve' );
		$busReserveComposition  = $this->MIOLO->getBusiness($this->module, 'BusReserveComposition' );
		$busExemplaryControl    = $this->MIOLO->getBusiness($this->module, 'BusExemplaryControl');
		$busMaterial            = $this->MIOLO->getBusiness($this->module, 'BusMaterial');
		$busLibraryUnit         = $this->MIOLO->getBusiness($this->module, 'BusLibraryUnit');

		//Executa o processo de envio do aviso
		$reserveId = $reserveObj->reserveId;
		$busReserveComposition->reserveId = $reserveObj->reserveId;
		$rc = $busReserveComposition->getReserveComposition();

		$itemNumber = $rc[0]->itemNumber;
		$title      = $busMaterial->getContentByItemNumber($itemNumber, MARC_TITLE_TAG);

		$subject = $this->busLibraryUnitConfig->getValueLibraryUnitConfig($libraryUnitId, 'EMAIL_RESERVE_ANSWERED_SUBJECT');
		$content = $this->busLibraryUnitConfig->getValueLibraryUnitConfig($libraryUnitId, 'EMAIL_RESERVE_ANSWERED_CONTENT');

		$limitDate  = new GnutecaDate2($reserveObj->limitDate);
		$gf         = new GnutecaFunction();
		$gf->setVariable('$USER_NAME',                  $personName);
		$gf->setVariable('$CODE',                       $reserveId);
		$gf->setVariable('$MATERIAL_TITLE',             $title);
		$gf->setVariable('$RESERVE_CODE',               $reserveId);
		$gf->setVariable('$LIBRARY_UNIT_DESCRIPTION',   $busLibraryUnit->getLibraryUnit($reserveObj->libraryUnitId)->libraryName);
		$gf->setVariable('$RESERVE_WITHDRAWAL_DATE',    $limitDate->getUserDate('DATE'));

		$mail = new GnutecaMail();
		$mail->setSubject($gf->interpret( $subject ));
		$mail->setContent($gf->interpret( $content ));
		$mail->setAddress($personMail);

		return $mail->send();
	}


    /**
     * Envia um email para o administrador com o resultado dos comunicados enviados para os alunos.
     *
     * @param array $result
     */
    public function sendMailToAdminResultOfReserveAnswered($result, $libraryUnitId)
    {
        $busMaterial = $this->MIOLO->getBusiness($this->module, 'BusMaterial');

        $mountNewResult = array();
        
        if ( count($result) > 0 )
        {
            foreach ($result as $reserveId => $content)
            {
                $boo = $content[2];
                $content[2] = GnutecaUtils::getYesNo($content[2]);
                $mountNewResult[$boo][$reserveId] = $content;
            }
    
            $columns = array
            (
                _M("Reserve id",        $this->module),
                _M("Person",            $this->module),
                _M("Has communicated",  $this->module),
                _M("Return message",    $this->module),
                _M("Item number",       $this->module),
                _M("Person mail",       $this->module),
                _M("Title",             $this->module),
            );
    
            $mailContent = $this->mountTable(_M("Mails failed"),            $columns, $mountNewResult[DB_FALSE], _M("All emails were sent successfully!", $this->module), true);
            $mailContent.= $this->mountTable(_M("Mail sent successfully"),  $columns, $mountNewResult[DB_TRUE],  _M("All emails failed!", $this->module), true);
        }
        else 
        {
            $mailContent = _M('There is no communication of reserve', $this->module);           
        }
        
        $to      = $this->getAdminEmailTo($libraryUnitid, 'EMAIL_ADMIN_RESERVE');
        
        $subject = $this->busLibraryUnitConfig->getValueLibraryUnitConfig($libraryUnitId, 'EMAIL_RESERVE_ANSWERED_ADMIN_RESULT_SUBJECT');
        $content = $this->busLibraryUnitConfig->getValueLibraryUnitConfig($libraryUnitId, 'EMAIL_RESERVE_ANSWERED_ADMIN_RESULT_CONTENT');

        $gf   = new GnutecaFunction();
        $gf->setVariable('$LN',         "\n");
        $content = $gf->interpret($content);
        $content = str_replace('$CONTENT', $mailContent, $content);

        $mail = new GnutecaMail();
        $mail->setSubject($subject);
        $mail->setContent($content);
        $mail->setAddress($to);
        return $mail->send();

    }


	/**
	 * Envia email para o fornecedor agradecendo e/ou solicitando materiais.
	 *
	 * @param integer $interchangeId
	 * @param text $formatedContent
	 */
	public function sendMailToSupplierInterchengeReceipt($interchangeId, $materials, $formatedContent = null)
	{
		$busInterchange             = $this->MIOLO->getBusiness($this->module, 'BusInterchange');
		$busSupplierTypeAndLocation = $this->MIOLO->getBusiness($this->module, 'BusSupplierTypeAndLocation');
		$interchange                = $busInterchange->getInterchange($interchangeId, "object");

		$supplier = $busSupplierTypeAndLocation->getSupplierTypeAndLocationValueForm( $interchange->supplierId, $interchange->type );
		$mailTo   = $supplier->email;
		if (!$mailTo)
		{
			return 1;
		}
		$this->mailTo = $mailTo;

		// AS LINHAS ABAIXO ESTAO COMENTADOS POR OS ITENS GRAVAM O CONTROL NUMBER, COM ESTE FICA RUIM BUSCAR A LIBRARY_UNIT DO MATERIAL.. QUE PODE TER VARIAS... O IDEAS SERIA GRAVAR O ITEMNUMBER
		//$item = $busInterchange->getInterchangeItem($interchangeId);
		//$content = !is_null($formatedContent) ? $formatedContent: $this->busLibraryUnitConfig->getValueLibraryUnitConfig($libraryUnitId, 'INTERCHANGE_MAIL_RECEIPT_CONTENT');
		//$subject = $this->busLibraryUnitConfig->getValueLibraryUnitConfig($libraryUnitId, 'INTERCHANGE_MAIL_RECEIPT_SUBJECT');

		$content = !is_null($formatedContent) ? $formatedContent: INTERCHANGE_MAIL_RECEIPT_CONTENT;
		$subject = INTERCHANGE_MAIL_RECEIPT_SUBJECT;

		//Obtem o conteudo do email
		$gf = new GnutecaFunction();
		$gf->setVariable('$CONTACT_NAME',   $supplier->contact);
		$gf->setVariable('$MATERIALS',      $materials);

		//Envia e-mail
		$mail = new GnutecaMail();
		$mail->setAddress($mailTo);
		$mail->setSubject($subject);
		$mail->setContent($gf->interpret($content));

		if (!$mail->send())
		{
			return 2;
		}

		// sucess
		return 0;
	}


	/**
	 * Encaminha um email para o solicitante informando sobre o trmino da requisio
	 *
	 * @param object $requestObject
	 */
    public function informaSolicitanteTerminoRequisicao($person, $subject, $content)
    {
        //Envia e-mail
        $mail = new GnutecaMail();
        $mail->setAddress($person->email);
        $mail->setSubject($subject);
        $mail->setContent($content);
        return $mail->send();
    }


	/**
	 * Este metodo informa o usuario que sua reserva foi cancelada.
	 *
	 * @param integer $reserveId
	 * @return boolean
	 */
	public function sendMailToUserInformingReserveCancel($reserveId)
	{
		$busPerson  = $this->MIOLO->getBusiness($this->module, 'BusPerson');
		$busReserve = $this->MIOLO->getBusiness($this->module, 'BusReserve');

		$reserve = $busReserve->getReserves(null, null, $reserveId);

		if(!$reserve)
		{
			return false;
		}

		$reserve    = $reserve[0];
		$person     = $busPerson->getBasicPersonInformations($reserve->personId);

		if(!strlen($person->email))
		{
			return false;
		}

		$busMaterial    = $this->MIOLO->getBusiness($this->module, 'BusMaterial');
		$busLibraryUnit = $this->MIOLO->getBusiness($this->module, 'BusLibraryUnit');
		$subject        = $this->busLibraryUnitConfig->getValueLibraryUnitConfig($reserve->libraryUnitId, 'EMAIL_CANCEL_RESERVE_COMUNICA_SOLICITANTE_SUBJECT');
		$content        = $this->busLibraryUnitConfig->getValueLibraryUnitConfig($reserve->libraryUnitId, 'EMAIL_CANCEL_RESERVE_COMUNICA_SOLICITANTE_CONTENT');

		//Obtem o conteudo do email
		$gf = new GnutecaFunction();
		$gf->setVariable('$USER_NAME',                  $person->name);
		$gf->setVariable('$MATERIAL_TITLE',             $busMaterial->getMaterialTitleByItemNumber($reserve->itemNumber));
		$gf->setVariable('$ITEM_NUMBER',                $reserve->itemNumber);
		$gf->setVariable('$LIBRARY_UNIT_DESCRIPTION',   $busLibraryUnit->getLibraryUnit($reserve->libraryUnitId)->libraryName);
		$content = $gf->interpret($content);

		//Envia e-mail
		$mail = new GnutecaMail();
		$mail->setAddress($person->email);
		$mail->setSubject($subject);
		$mail->setContent($content);
		return $mail->send();
	}




	/**
	 * Este metodo cria uma tabela simples para enviar por email.
	 *
	 * @param unknown_type $firstLineText
	 * @param unknown_type $columns
	 * @param unknown_type $content
	 * @param unknown_type $emptyMsg
	 * @return unknown
	 */
    public function mountTable($firstLineText, $columns, $content, $emptyMsg = "", $displayTotalLines = false)
    {
        $line = $colummn = 0;

        $tableRaw = new MSimpleTable(null, "cellspacing=0 cellpadding=2 border=1");

        if(!is_null($firstLineText))
        {
            $tableRaw->setCell($line++, 0, $firstLineText, 'colspan="100" align="center"');
        }

        if(!is_array($content) || !count($content))
        {
            $tableRaw->setCell($line++, 0, $emptyMsg, 'colspan="100" align="center"');
        }
        else
        {
            foreach ($columns as $text)
            {
                $tableRaw->setCell($line, $colummn++, $text, 'align="center"');
            }

            foreach ($content as $celContent)
            {
                $line++;
                foreach ($celContent as $index => $txt)
                {
                    $txt = strlen($txt) ? $txt : ' - ';
                    $tableRaw->setCell($line, $index, $txt);
                }
            }

            if($displayTotalLines)
            {
                $tableRaw->setCell(++$line, 0, _M("Total") . ": ". count($content) ."  ", "colspan=100 align=right");
            }
        }

        return str_replace(array("\n", "\t", "\r"), "", $tableRaw->generate());
    }


	/**
	 * Esta funo retorna o email do administrador para cada tipo de operao.
	 *
	 * @param id da biblioteca $libraryUnitid
	 * @param operation $type
	 * @return email string
	 */
	private function getAdminEmailTo($libraryUnitId, $type = null)
	{
		if(!is_null($type))
		{
			// BUSCA EMAIL ESPECIFICO DA UNIDADE
			$to = $this->busLibraryUnitConfig->getValueLibraryUnitConfig($libraryUnitId, $type);

			if(strlen($to))
			{
				return $to;
			}
		}

		// RETORNA EMAIL PADRAO
		return $this->busLibraryUnitConfig->getValueLibraryUnitConfig($libraryUnitId, 'EMAIL_ADMIN');
	}
	
	
    /**
     * Envia e-mail para usurio informando sobre devoluo
     * @param (array) $loan
     * @param varchar $subject
     * @param varchar $content
     */
    public function sendMailToUserCommunicateReturn($loan, $subject, $content)
    {
        $busLibraryUnit = $this->MIOLO->getBusiness($this->module, 'BusLibraryUnit');
        $libraryUnitDescription = $busLibraryUnit->getLibraryName($loan->libraryUnitId);

        $tags = array('$USER_NAME'                => $loan->personName,
                      '$MATERIAL_TITLE'           => $loan->title,
                      '$ITEM_NUMBER'              => $loan->itemNumber,
                      '$RETURN_FORECAST_DATE'     => GnutecaDate::toUser($loan->returnForecastDate, 'DATE'),
                      '$LIBRARY_UNIT_DESCRIPTION' => $libraryUnitDescription,
                      '$LN'                       => EMAIL_LINE_BREAK);

        $mail = new GnutecaMail();
        $mail->setSubject($subject);
        $mail->setContent(strtr($content, $tags));
        $mail->setAddress($loan->personEmail);
        return $mail->send();
    }

    
    /**
     * Envia um email para o administrador com o resultado dos comunicados de devoluo enviados para os alunos.
     *
     * @param array $result
     * @param code of library
     */
    public function sendMailToAdminResultOfCommunicateReturn($result, $libraryUnitId)
    {
        $mountNewResult = array();
        if ( count($result) )
        {
            foreach ($result as $reserveId => $content)
            {
                $boo = $content[2];
                $content[2] = GnutecaUtils::getYesNo($content[2]);
                $mountNewResult[$boo][$reserveId] = $content;
            }
    
            $columns = array
            (
                _M("Loan code",        $this->module),
                _M("Person",            $this->module),
                _M("Has communicated",  $this->module),
                _M("Return message",    $this->module),
                _M("Item number",       $this->module),
                _M("Person mail",       $this->module),
                _M("Title",             $this->module),
            );
            
            $mailContent = $this->mountTable(_M("Mails failed"),            $columns, $mountNewResult[DB_FALSE], _M("All emails were sent successfully!", $this->module), true);
            $mailContent.= $this->mountTable(_M("Mail sent successfully"),  $columns, $mountNewResult[DB_TRUE],  _M("All emails failed!", $this->module), true);
        }
        else 
        {
            $mailContent = _M('There are no reports of return', $this->module);
        }
        
        $to      = $this->getAdminEmailTo($libraryUnitId, 'EMAIL_ADMIN_DEVOLUTION');
        $subject = $this->busLibraryUnitConfig->getValueLibraryUnitConfig($libraryUnitId, 'EMAIL_ADMIN_DEVOLUTION_RESULT_SUBJECT');
        $content = $this->busLibraryUnitConfig->getValueLibraryUnitConfig($libraryUnitId, 'EMAIL_ADMIN_DEVOLUTION_RESULT_CONTENT');

        $gf   = new GnutecaFunction();
        $gf->setVariable('$LN',         "\n");
        $content = $gf->interpret($content);
        $content = str_replace('$CONTENT', $mailContent, $content);

        $mail = new GnutecaMail();
        $mail->setSubject($subject);
        $mail->setContent($content);
        $mail->setAddress($to);

        return $mail->send();
    }
    
    
    /**
     * Envia e-mail para usurio informando sobre emprstimo atrasado
     * @param (array) $loan
     * @param varchar $subject
     * @param varchar $content
     */
    public function sendMailToUserCommunicateDelayedLoan($loan, $subject, $content)
    {
        $busLibraryUnit = $this->MIOLO->getBusiness($this->module, 'BusLibraryUnit');
        $libraryUnitDescription = $busLibraryUnit->getLibraryName($loan->libraryUnitId);
        
        $tags = array('$USER_NAME'                => $loan->personName,
                      '$ITEM_NUMBER'              => $loan->itemNumber,
                      '$MATERIAL_TITLE'           => $loan->title,
                      '$RETURN_DATE'              => GnutecaDate::toUser($loan->returnForecastDate, 'DATE'),
                      '$LIBRARY_UNIT_DESCRIPTION' => $libraryUnitDescription,
                      '$LN'                       => EMAIL_LINE_BREAK);
        
        $mail = new GnutecaMail();  
        $mail->setSubject($subject);
        $mail->setContent(strtr($content, $tags));
        $mail->setAddress($loan->personEmail);
        
        return $mail->send();
    }
    
    
    /**
     * Envia um email para o administrador com o resultado dos comunicados de emprstimos atrasados enviados para os alunos.
     *
     * @param array $result
     * @param code of library
     */
    public function sendMailToAdminResultOfCommunicateDelayedLoan($result, $libraryUnitId)
    {
        $mountNewResult = array();
        if ( count($result) > 0 )
        {
            foreach ($result as $reserveId => $content)
            {
                $boo = $content[2];
                $content[2] = GnutecaUtils::getYesNo($content[2]);
                $mountNewResult[$boo][$reserveId] = $content;
            }
    
            $columns = array
            (
                _M("Loan code",        $this->module),
                _M("Person",            $this->module),
                _M("Has communicated",  $this->module),
                _M("Return message",    $this->module),
                _M("Item number",       $this->module),
                _M("Person mail",       $this->module),
                _M("Title",             $this->module),
            );
            
            $mailContent = $this->mountTable(_M("Mails failed"),            $columns, $mountNewResult[DB_FALSE], _M("All emails were sent successfully!", $this->module), true);
            $mailContent.= $this->mountTable(_M("Mail sent successfully"),  $columns, $mountNewResult[DB_TRUE],  _M("All emails failed!", $this->module), true);
        }
        else 
        {
            $mailContent = _M('There are no notification of delayed loans', $this->module); 
        }

        $to      = $this->getAdminEmailTo($libraryUnitId, 'EMAIL_ADMIN_DELAYED_LOAN');
        $subject = $this->busLibraryUnitConfig->getValueLibraryUnitConfig($libraryUnitId, 'EMAIL_ADMIN_DELAYED_LOAN_RESULT_SUBJECT');
        $content = $this->busLibraryUnitConfig->getValueLibraryUnitConfig($libraryUnitId, 'EMAIL_ADMIN_DELAYED_LOAN_RESULT_CONTENT');

        $gf   = new GnutecaFunction();
        $gf->setVariable('$LN',         "\n");
        $content = $gf->interpret($content);
        $content = str_replace('$CONTENT', $mailContent, $content);

        $mail = new GnutecaMail();
        $mail->setSubject($subject);
        $mail->setContent($content);
        $mail->setAddress($to);

        return $mail->send();
    }
    
    
    /**
     * Envia um e-mail para o usurio informando as novas aquisies
     * @param (array) $personInformations
     * @param varchar $subject
     * @param varchar $content
     */
    public function sendMailToUserNotifyAcquisition($personInformations, $subject, $content)
    {
         $mail = new GnutecaMail();
         $mail->setSubject($subject);
         $mail->setContent($content);
         $mail->setAddress($personInformations->email);
         return $mail->send();
    }
    
    
    /**
     * Envia um email para o administrador com o resultado das notificaes de aquisies
     *
     * @param array $result
     * @param code of library
     */
    public function sendMailToAdminResultOfNotifyAcquisition($result, $libraryUnitId)
    {
        $busPreference = $this->MIOLO->getBusiness($this->module, 'BusPreference');
        $mountNewResult = array();

        if ( count($result) > 0 )
        {
            foreach ($result as $reserveId => $content)
            {
                $boo = $content[2];
                $content[2] = GnutecaUtils::getYesNo($content[2]);
                $mountNewResult[$boo][$reserveId] = $content;
            }
    
            $columns = array
            (
                _M("Person code",       $this->module),
                _M("Name",              $this->module),
                _M("Has communicated",  $this->module),
                _M("Return message",    $this->module),
                _M("Person mail",       $this->module),
            );
            
            $mailContent = $this->mountTable(_M("Mails failed"),            $columns, $mountNewResult[DB_FALSE], _M("All emails were sent successfully!", $this->module), true);
            $mailContent.= $this->mountTable(_M("Mail sent successfully"),  $columns, $mountNewResult[DB_TRUE],  _M("All emails failed!", $this->module), true);
        }
        else 
        {
            $mailContent = _M('There are no notification of acquisitions', $this->module);
        }
        $to = $this->getAdminEmailTo($libraryUnitId, 'EMAIL_ADMIN_NOTIFY_ACQUISITION');
        $content = $this->busLibraryUnitConfig->getValueLibraryUnitConfig($libraryUnitId, 'EMAIL_ADMIN_NOTIFY_ACQUISITION_RESULT_CONTENT');
        $subject = $this->busLibraryUnitConfig->getValueLibraryUnitConfig($libraryUnitId, 'EMAIL_ADMIN_NOTIFY_ACQUISITION_RESULT_SUBJECT');

        $gf   = new GnutecaFunction();
        $gf->setVariable('$LN',         "\n");
        $content = $gf->interpret($content);
        $content = str_replace('$CONTENT', $mailContent, $content);

        $mail = new GnutecaMail();
        $mail->setSubject($subject);
        $mail->setContent($content);
        $mail->setAddress($to);

        return $mail->send();
    }
    
    
    /**
     * Envia um email para o administrador com o resultado das notificaes de trmino de requisio
     *
     * @param array $result
     * @param code of library
     */
    public function sendMailToAdminResultOfNotifyEndRequest($result, $libraryUnitId)
    {
        $busPreference = $this->MIOLO->getBusiness($this->module, 'BusPreference');
        $mountNewResult = array();

        if ( count($result) > 0 )
        {
            foreach ($result as $reserveId => $content)
            {
                $boo = $content[2];
                $content[2] = GnutecaUtils::getYesNo($content[2]);
                $mountNewResult[$boo][$reserveId] = $content;
            }
    
            $columns = array
            (
                _M("Request code",     $this->module),
                _M("Person",           $this->module),
                _M("Has communicated", $this->module),
                _M("Return message",   $this->module),
                _M("Person mail",      $this->module),
                _M("Closure",          $this->module)
            );
            
            $mailContent = $this->mountTable(_M("Mails failed"),            $columns, $mountNewResult[DB_FALSE], _M("All emails were sent successfully!", $this->module), true);
            $mailContent.= $this->mountTable(_M("Mail sent successfully"),  $columns, $mountNewResult[DB_TRUE],  _M("All emails failed!", $this->module), true);
        }
        else 
        {
            $mailContent = _M('There are no notification of acquisitions', $this->module);
        }
        $to = $this->getAdminEmailTo($libraryUnitId, 'EMAIL_ADMIN_NOTIFY_END_REQUEST');
        $content = $this->busLibraryUnitConfig->getValueLibraryUnitConfig($libraryUnitId, 'EMAIL_ADMIN_NOTIFY_END_REQUEST_RESULT_CONTENT');
        $subject = $this->busLibraryUnitConfig->getValueLibraryUnitConfig($libraryUnitId, 'EMAIL_ADMIN_NOTIFY_END_REQUEST_RESULT_SUBJECT');

        $gf   = new GnutecaFunction();
        $gf->setVariable('$LN',         "\n");
        $content = $gf->interpret($content);
        $content = str_replace('$CONTENT', $mailContent, $content);

        $mail = new GnutecaMail();
        $mail->setSubject($subject);
        $mail->setContent($content);
        $mail->setAddress($to);

        return $mail->send();
    }


}

?>
