<?php
/**
 * <--- Copyright 2005-2011 de Solis - Cooperativa de Solues Livres Ltda. e
 * Univates - Centro Universitrio.
 * 
 * Este arquivo  parte do programa Gnuteca.
 * 
 * O Gnuteca  um software livre; voc pode redistribu-lo e/ou modific-lo
 * dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao
 * do Software Livre (FSF); na verso 2 da Licena.
 * 
 * Este programa  distribudo na esperana que possa ser til, mas SEM
 * NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO
 * ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU/GPL em
 * portugus para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo
 * "LICENCA.txt", junto com este programa, se no, acesse o Portal do Software
 * Pblico Brasileiro no endereo www.softwarepublico.gov.br ou escreva para a
 * Fundao do Software Livre (FSF) Inc., 51 Franklin St, Fifth Floor, Boston,
 * MA 02110-1301, USA --->
 * 
 * Gnuteca Task
 *
 * @author Luiz Gilberto Gregory Filho [luz@solis.coop.br]
 *
 * @version $Id$
 *
 * \b Maintainers: \n
 * Eduardo Bonfandini [eduardo@solis.coop.br]
 * Jamiel Spezia [jamiel@solis.coop.br]
 * Luiz Gregory Filho [luiz@solis.coop.br]
 * Moises Heberle [moises@solis.coop.br]
 *
 * @since
 * Class created on 06/08/2009
 *
 **/


abstract class GnutecaTask
{

    /**
     * ATTRIBUTES
     */

    protected   $MIOLO, $module;
    private     $myTas;
    protected   $busScheduleTaskLog;
    protected   $parameters;
    private     $startTime;


    /**
     * CONSTANTES
     */

    const STARTING_LOG_MSG  = "Iniciando agendamento de tarefa";
    const SUCCESS_LOG_MSG   = "Sucesso no agendamento da tarefa";
    const UNSUCCESS_LOG_MSG = "Falha no agendamento da tarefa";

    /**
     *
     * @param OBJECT $MIOLO
     */
    function __construct($MIOLO, $myTask)
    {
        $this->MIOLO        = $MIOLO;
        $this->myTask       = $myTask;
        $this->module       = $this->MIOLO->getCurrentModule();

        $this->busScheduleTaskLog = $this->MIOLO->getBusiness($this->module, 'BusScheduleTaskLog');
        $this->MIOLO->getClass($this->module, 'GnutecaDate2');
    }


    /**
     * retorna a id da tarefa que pode ser setada apenas no construct
     *
     * @return integer
     */
    protected function getMyTaskId()
    {
    	if ( $this->myTask->ScheduleTaskId )
    	{
    		$taskId = $this->myTask->ScheduleTaskId;
    	}
    	else
    	{
    		$taskId = $this->myTask->scheduleTaskId;
    	} 
        return $taskId;
    }


    /**
     * Registra um conteudo no log
     *
     * @param string $msg
     */
    protected function registerLog($msg)
    {
        $this->busScheduleTaskLog->insertScheduleTaskLog($this->getMyTaskId(), $msg);
    }


    /**
     * Registra log de inicializao
     */
    public function startLog()
    {
        $this->registerLog(self::STARTING_LOG_MSG);
    }


    /**
     * Registra log de finalizao
     *
     * @param boolean $ok
     */
    public function concludeLog($ok = true)
    {
        $this->registerLog($ok ? self::SUCCESS_LOG_MSG : self::UNSUCCESS_LOG_MSG);
    }


    /**
     * Set o tempo que iniciou a execuo do script principal.
     *
     * @param unknown_type $startTime
     */
    public function setStartTime($startTime)
    {
        $this->startTime = $startTime;
    }


    /**
     * Este mtodo verifica se  preciso executar o script
     *
     * @return boolean
     */
    public function needExecute()
    {
        $time = str_replace(array("Y", "m", "d", "H", "w"),
                            array($this->startTime->Y, $this->startTime->m, $this->startTime->d, $this->startTime->H, $this->startTime->w),
                            $this->myTask->ScheduleCycleValueType);
        
        $find = false;  
        // Check para ver se esta no dia e hora correta para executar                  
        if ( $cycles = explode(';', $this->myTask->ScheduleTaskCycleValue) )
        {
        	foreach( $cycles as $cycle )
        	{
        	   	if ( $time == $cycle )
        	   	{
        	   		$find = true;
        	   		break;
        	   	}
        	}
        }
        elseif($time == $this->myTask->ScheduleTaskCycleValue)
        {
                $find = true;
        }
        
        if (!$find)
        {
        	return false;
        }

        // REGISTRA LOG DE INICIO DA TAREFA
        $this->startLog();
        return true;
    }





    /**
     * Este metodo verifica se  preciso ser executado
     *
     * @return booelan
     */
    public function execute()
    {
        return $this->needExecute();
    }


    /**
     * Seta os parametros para a classe
     *
     * @param string $parameters
     */
    public function setParameters($parameters)
    {
        $this->parameters = explode('|', $parameters);
    }

}

?>
