<?php
/**
 * <--- Copyright 2005-2011 de Solis - Cooperativa de Solues Livres Ltda. e
 * Univates - Centro Universitrio.
 * 
 * Este arquivo  parte do programa Gnuteca.
 * 
 * O Gnuteca  um software livre; voc pode redistribu-lo e/ou modific-lo
 * dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao
 * do Software Livre (FSF); na verso 2 da Licena.
 * 
 * Este programa  distribudo na esperana que possa ser til, mas SEM
 * NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO
 * ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU/GPL em
 * portugus para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo
 * "LICENCA.txt", junto com este programa, se no, acesse o Portal do Software
 * Pblico Brasileiro no endereo www.softwarepublico.gov.br ou escreva para a
 * Fundao do Software Livre (FSF) Inc., 51 Franklin St, Fifth Floor, Boston,
 * MA 02110-1301, USA --->
 * 
 * GnutecaUtils - Usefull miscelaneous functions
 *
 * @author Moises Heberle [moises@solis.coop.br]
 *
 * @version $Id$
 *
 * \b Maintainers: \n
 * Eduardo Bonfandini [eduardo@solis.coop.br]
 * Jamiel Spezia [jamiel@solis.coop.br]
 * Luiz Gregory Filho [luiz@solis.coop.br]
 * Moises Heberle [moises@solis.coop.br]
 *
 * @since
 * Class created on 29/07/2008
 *
 **/
class GnutecaUtils
{
    /**
     * Get array of "Yes" or "No" to use on MIOLO form's (MSelection/combolist/etc)
     *
     * @param $type (int): Format (0 or 1)
     *
     * @return $data (array): Array of options
     */
    public function listYesNo($type = 0)
    {
        $MIOLO  = MIOLO::getInstance();
        $module = $MIOLO->getCurrentModule();

        if ($type == 0)
        {
            $data = array(
                DB_TRUE  => 'Sim',
                DB_FALSE => 'No'
            );
        }
        elseif ($type == 1)
        {
            $data = array(
                array('Sim', DB_TRUE),
                array('No', DB_FALSE)
            );

        }
        
        return $data;
    }

    /**
     * Get array list of "Yes" and "No"
     *
     * @param $key (String): If defined, returns specified value
     *
     * @return $data (array): Array of options
     */
    public function getYesNo($key=null)
    {
        $MIOLO  = MIOLO::getInstance();
        $module = $MIOLO->getCurrentModule();

        $data = array(
            DB_TRUE  => 'Sim',
            DB_FALSE => 'No'
        );
        if ($key)
        {
            return $data[$key];
        }
        return $data;
    }

    function getDbActionList()
    {
        $MIOLO  = MIOLO::getInstance();
        $module = $MIOLO->getCurrentModule();

        $array = array
        (
            "I" => _M("Insert", $module),
            "U" => _M("Update", $module),
            "D" => _M("Delete", $module),
        );

        return $array;
    }

    /**
     * Compara dois array verificando se so ou no iguais
     * 
     * Retorna true se so iguais
     * Retorna false se diferentes
     * 
     * @param $array1
     * @param $array2
     * @return boolean
     */
    function compareArray($array1, $array2)
    {
    	//filtra o array tirando, dados em branco 
    	if ( is_array( $array1 ) )
    	{
            $array1 = array_filter( $array1 );
            
            if ( count( $array1) == 0 ) 
            {
            	$array1 = array(); 
            	$array1 = null;
            }
    	}
    	else
    	{
    		$array1 = null;
    	}
    	
        if ( is_array( $array2 ) )
        {
            $array2 = array_filter( $array2 );
            
            if ( count( $array2) == 0 ) 
            {
            	$array2 = null;

            }
        }
        else
        {
        	$array2 = null;
        }
        
    	// se os dois arrays forem nulos retorna true, dizendo que so iguais
        if(is_null($array1) && is_null($array2))
        {
            return true;
        }

        //se a quantidade for diferente,  diferente
        if(count($array1) != count($array2))
        {
            return false;
        }

        //faz a comparao linha a linha, comparando recursivamente
        foreach ($array1 as $index => $content)
        {
            if(is_array($content) && is_array($array2[$index]))
            {
                $ok = GnutecaUtils::compareArray($content, $array2[$index]);

                if(!$ok)
                {
                    return false;
                }

                continue;
            }

            if($content != $array2[$index])
            {
                return false;
            }
        }

        return true;
    }

    public function extractMarcVariables($text)
    {
        preg_match_all('/\$[0-9]{3}\.[0-9a-zA-Z]/', $text, $matches);
        return $matches[0];
    }

    /**
     * Return formatted money value
     *
     * @param unknown_type $value
     * @return unknown
     */
    public static function moneyFormat($value)
    {
    	return number_format($value, 2, ',', '.');
    }    
    
    /**
     * Retorna o calculo somado de um array de valores ou string
     *
     * @param (Array) $values ou (String) com operacao
     * 
     * @return (float)
     */
    public function moneySum($values)
    {
        if (is_array($values))
        {
            foreach ($values as $i => $value)
            {
                $values[$i] = GnutecaUtils::moneyToFloat($value);
            }
            $str = implode('+', $values);
        }
        else
        {
            $str = $values;
        }
        
        $gnutecaBusiness = new GnutecaBusiness();
        $value = $gnutecaBusiness->executeSelect( "SELECT ({$str})" );
        $value = $value[0][0];
        
        return GnutecaUtils::floatToMoney($value);
    }
    
    function moneyToFloat($valor)
    {
        $nao_tem_virgula    = strpos($valor, ",") === false;
        $tem_ponto          = strpos($valor, ".") !== false;
        
        if($nao_tem_virgula)
        {
	        if($tem_ponto)
	        {
		        $pos    = strrpos($valor, ".");
		        $valor  = substr($valor, 0, $pos) . "," . substr($valor, $pos+1, strlen($valor)-$pos+1);
	        }
        }
        
        $valor = str_replace(".", "", $valor);
        $valor = str_replace(",", ".", $valor);
        //$valor = (float)$valor;
        return $valor;
    }

	function floatToMoney($valor)
	{
		return number_format($valor, 2, ",", ".");
	}
    
    public function checkConstantBoolean($constant)
    {
        $trueArray  = array("t", "v", "true");    
        $constant = strtolower($constant);
        
        if(in_array($constant, $trueArray))
        {
            return TRUE;
        }
        
        return FALSE;
    }
    
    /**
     * Converts $content to ASCII
     *
     * @param unknown_type $content
     * @return unknown
     */
    public static function toASCII($content)
    {
        $content = trim($content);
        $content = eregi_replace("[]",    "A", $content);
        $content = eregi_replace("[]",      "E", $content);
        $content = eregi_replace("[]",      "I", $content);
        $content = eregi_replace("[]",    "O", $content);
        $content = eregi_replace("[]",      "U", $content);
        $content = eregi_replace("[]",            "N", $content);
        $content = eregi_replace("[]",            "C", $content);
        $content = eregi_replace("\+",              "",  $content);
        return strtoupper( $content );
    }
    
   /**
     * Obtm a verso do gnuteca
     *
     * @return: string com a verso
     */
    public static function getVersion()
    {
        $MIOLO   = MIOLO::getInstance();
        //tem de ser hardcoded pois a acessando da statusBar o mdulo no  reconhecido
        $module  = 'gnuteca3';
        $version = file($MIOLO->getModulePath($module, "VERSION"));
        $v = explode('.', $version[0]);

        return trim($v[0] . '.' . $v[1]);
    }

     /**
     * Obtm a sub verso do gnuteca
     *
     * @return: string com a verso
     */
    public static function getSubVersion()
    {
        $MIOLO   = MIOLO::getInstance();
        //tem de ser hardcoded pois a acessando da statusBar o mdulo no  reconhecido
        $module  = 'gnuteca3';
        $version = file($MIOLO->getModulePath($module, "VERSION"));
        $v = explode('.', $version[0]);

        return trim($v[2]);
    }

    public static function getChangeLog()
    {
        $MIOLO   = MIOLO::getInstance();
        //tem de ser hardcoded pois a acessando da statusBar o mdulo no  reconhecido
        $module  = 'gnuteca3';
        return file_get_contents($MIOLO->getModulePath($module, "ChangeLog"));;
    }

    /**
     * Retorna a string de chamada para algum evento AJAX do formulario
     *
     * @param String $function
     * @param String $args
     * @param String|Array $formName
     * @param boolean $literalJsArgs
     * @return String
     */
    public static function getAjax($function, $args = null, $formName = null, $literalJsArgs = false)
    {
        $MIOLO = MIOLO::getInstance();
        
        if (!$formName)
        {
            $formName = $MIOLO->page->getFormId();
        }

        if (is_array($args))
        {
            $args = GnutecaUtils::encodeJsArgs($args);
        }

        if (!$literalJsArgs)
        {
            $args = "'{$args}'";
        }
        return "miolo.doAjax('{$function}', {$args}, '{$formName}');";
    }

    /*
     * Codifica um array para utilizao dentro de funo ajax.
     *
     */
    public static function encodeJsArgs(array $args)
    {
        foreach ($args as $key => $val)
        {
            $str[] = "{$key}|~|{$val}";
        }
        return implode('|#|', $str);
    }

    /*
     * Decodifica os argumentos passados atravs do encodeJsArgs
     *
     */
    public static function decodeJsArgs($value)
    {
        $values = explode('|#|', $value);
        $result = array();
        if ($values)
        {
            foreach ($values as $v)
            {
                list($key, $val) = explode('|~|', $v);
                $result[$key] = $val;
            }
        }
        return (object)$result;
    }

    /*
     * Retorna os argumentos passados na funo ajax.
     * Note que ainda precisa ser utilizada o decodeJsArgs, caso necessrio
     *
     */
    public static function getAjaxEventArgs()
    {
        $MIOLO = MIOLO::getInstance();
        return MIOLO::_REQUEST("{$MIOLO->page->getFormId()}__EVENTARGUMENT");
    }

    /**
     * Retorna qual  a funo/chamada ajax executada neste momento
     * @return <string> funo/chamada ajax executada neste momento
     */
    public static function getAjaxFunction()
    {
        $MIOLO = MIOLO::getInstance();
        return MIOLO::_REQUEST("{$MIOLO->page->getFormId()}__EVENTTARGETVALUE");
    }

    /**
     * Retorna string da aao javascript de fechar uma janela de dilogo
     *
     * @return <string> aao javascript de fechar uma janela de dilogo
     */
    public static function getCloseAction($javascript = false)
    {
        $result = 'gnuteca.closeAction();';

        if ( $javascript )
        {
            $result = 'javascript:'. $result;
        }

        return $result;
    }

    /**
     * Funcao utilizada para alinhar campos que nao estao sendo renderizados corretamente nos formularios
     *
     * @param <array> $controls
     * @return MContainer
     */
    public static function alinhaForm($controls)
    {
        $isControlsArrayInitial = is_array($controls);

        if (!is_array($controls))
        {
            $controls = array($controls);
        }

        $container = new MContainer('hctAlinhaForm'.rand(), $controls, null, MControl::FORM_MODE_SHOW_SIDE);
        $container->setClass('alinhaForm');

        if ($isControlsArrayInitial) //Se recebeu array, retorna array, senao apenas o MContainer
        {
            return array($container);
        }
        
        return $container;
    }

    /**
     * Retorna url completa da imagem do tema
     *
     * @param <string> $imageName exemplo report-16x16.png
     * @return <string> url completa
     */
    public static function getImageTheme($imageName)
    {
        $MIOLO = MIOLO::getInstance();
        return $MIOLO->getUI()->getImageTheme( $MIOLO->getTheme()->getId(), $imageName);
    }


    /**
     * Execute um teste unitrio dentro do gnuteca
     *
     * @param <string> $testFile TestLibraryUnit.class.php';
     * @return <string> shell return string
     */
    public static function executeUnitTest($testFile, $format = true )
    {
        ob_start();
        $command = "cd ../modules/gnuteca3/unittest/; phpunit $testFile";
        system($command);
        $message = ob_get_contents();
        ob_clean();

        $tempMessage = explode("\n", $message);

        //nulifica linha s com espaos
        if ( is_array($tempMessage) )
        {
            foreach ( $tempMessage as $line => $info )
            {
                $tempMessage[$line] = trim($info);

                if ( $tempMessage[$line][0] == '.' )
                {
                    $tempMessage[$line][0] = '';
                    $tempMessage[$line] = trim($tempMessage[$line]);
                }
            }
        }

        $tempMessage = array_filter($tempMessage); //remove linhas vazias
        $tempMessage = array_values($tempMessage); //reajeita os arrays

        $lastLine = $tempMessage[ count($tempMessage)-1 ];

        $result = stripos($lastLine, 'OK') === 0 ? true : false;

        if ( $format )
        {
            $message = implode("\n",$tempMessage);
            $message = str_replace("OK","<b><font color ='green'>OK</font></b>", $message);
            $message = str_replace("FAILURES","<b><font color ='red'>FAILURES</font></b>", $message);
            $message = str_replace("FAIL","<b><font color ='red'>FAIL</font></b>", $message);
        }

        return array($result, $message);
    }
}
?>