<?php
/**
 * <--- Copyright 2005-2011 de Solis - Cooperativa de Solues Livres Ltda. e
 * Univates - Centro Universitrio.
 * 
 * Este arquivo  parte do programa Gnuteca.
 * 
 * O Gnuteca  um software livre; voc pode redistribu-lo e/ou modific-lo
 * dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao
 * do Software Livre (FSF); na verso 2 da Licena.
 * 
 * Este programa  distribudo na esperana que possa ser til, mas SEM
 * NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO
 * ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU/GPL em
 * portugus para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo
 * "LICENCA.txt", junto com este programa, se no, acesse o Portal do Software
 * Pblico Brasileiro no endereo www.softwarepublico.gov.br ou escreva para a
 * Fundao do Software Livre (FSF) Inc., 51 Franklin St, Fifth Floor, Boston,
 * MA 02110-1301, USA --->
 * 
 * Replace of original MIOLO validator functions
 *
 * @author Moises Heberle [moises@solis.coop.br]
 *
 * @version $Id$
 *
 * \b Maintainers: \n
 * Eduardo Bonfandini [eduardo@solis.coop.br]
 * Jamiel Spezia [jamiel@solis.coop.br]
 * Luiz Gregory Filho [luiz@solis.coop.br]
 * Moises Heberle [moises@solis.coop.br]
 *
 * @since
 * Class created on 21/01/2009
 *
 **/
class GnutecaValidators extends MGrid
{
	public $errors;
	public $validators = array();
	public /*GnutecaForm*/ $form;


    public function __construct()
    {
    }


    public function setValidators($validators)
    {
        if(is_null($validators))
        {
            $this->validators = array();
            return false;
        }

        if (!is_array($validators) )
        {
            $validators = array($validators);
        }


        /*MRequiredValidator Object (
         [field] =>
         weekDayId [min] => 0
         [max] => 3
         [type] => required
         [chars] => ALL
         [mask] =>
         [checker] =>
         [msgerr] => algum erro obscuro
         [html] =>
         [label] => Erro
         [value] =>
         [hint] =>
         [form] =>
         [formName] =>
         [showLabel] => 1
         [autoPostBack] =>
         [validator] =>
         [_numberId:private] => 46
         [id] => required
         [uniqueId] => m46
         [cssClass] =>
         [enabled] => 1 */

        //filtra em funo do $this->miolo
        foreach ($validators as $line => $validator)
        {
            $valid[$line]->id     = $validator->id;
            $valid[$line]->field  = $validator->field;
            $valid[$line]->min    = $validator->min;
            $valid[$line]->max    = $validator->max;
            $valid[$line]->type   = $validator->type;
            $valid[$line]->chars  = $validator->chars;
            $valid[$line]->mask   = $validator->mask;
            $valid[$line]->checker= $validator->checker;
            $valid[$line]->msgerr = $validator->msgerr;
            $valid[$line]->html   = $validator->html;
            $valid[$line]->label  = $validator->label;
            $valid[$line]->value  = $validator->value;
            $valid[$line]->hint   = $validator->hint;
            $valid[$line]->regexp = $validator->regexp;
        }

        $this->validators = $valid;
    }


    public function getValidators()
    {
        return $this->validators;
    }


    public function setForm(GnutecaForm $form)
    {
        $this->form = $form;
    }


	/**
	 * Return the message to show for validator
	 *
	 * @param string $message to show
	 * @param object $valid validator do detect the message
	 * @return string the message
	 */
	public function getValidatorMessage($message, $valid)
	{
	    $module = MIOLO::getCurrentModule();
	    $msg = _M($message , $module, $valid->field);

	    if ( $valid->msgerr )
	    {
            $msg = $valid->msgerr;
	    }
	    else if ( $valid->label )
	    {
            $msg = _M( $message, $module, $valid->label);
	    }
	    else if ( $this->form )
	    {
            //Get label name of form field
	    	$field = $this->form->getField($valid->field);
            //procura label no campo
            $value = '';
            if ( $field->label ) //pega label do campo normal (MTextField)
	    	{
                $value = $field->label;
	    	}
            elseif ( $field->caption ) //pega label da GnutecaRepetitiveField
            {
                $value = $field->caption;
            }
            else//procura label do lado do campo
            {
                $controls = $this->form->getControls();
                if ( is_array($controls) )
                {
                    foreach ($controls as $i=>$control)
                    {
                        if ( strlen($value) == 0 ) //s procura se ainda no tiver achado a label
                        {
                            //pega containers do formulrio
                            if ( $control instanceof MContainer ) //se for da instncia mcontainer (mhcontainer, mvcontainer e gcontainer)
                            {
                                $internalControls = $control->getControls();
                                if (is_array($internalControls) )
                                {
                                    $tmpValue = '';
                                    foreach ($internalControls as $k=>$internalControl)
                                    {
                                        if ($internalControl instanceof MLabel)
                                        {
                                            $tmpValue = $internalControl->value;
                                        }

                                        if ( $internalControl->name == $field->name) //achar o campo
                                        {
                                            $value = $tmpValue;
                                            break;
                                        }
                                    }
                                }
                            }
                        }
                        else
                        {
                            //retira o ":" se estiver no final da string
                            if (substr($value, -1, 1) == ':')
                            {
                                $value = substr($value, 0, strlen($value)-1);
                            }
                            
                            break;
                        }
                    }
                }
            }
            $msg = _M( $message, $module, $value);
	    }
        
	    return $msg;
	}


	public function validate( $data = NULL, $validators = NULL )
	{
		$errors = $this->errors;

        //tenta encontrar validadores, caso no sejam passados
		if (!$validators)
		{
            $validators = $this->getValidators();
		}

        //tenta encontrar os dados caso, no sejam passados
		if (!$data)
		{
            foreach ($validators as $v)
            {
                $data[] = MIOLO::_REQUEST($v->field);
            }
		}

		if (!$validators ||  !is_array($validators)  )
	    {
	        return null;
	    }

        foreach ($validators as $line => $valid)
        {
            $field = $valid->field;

            if ( $this->form )
            {
                $fieldObj =  $this->form->getField($field);
            }
            
            /*[field] => weekDayId
             [min] => 0
             [max] => 3
             [type] => required
             [chars] => ALL
             [mask] =>
             [checker] =>
             [msgerr] => algum erro obscuro
             [html] =>
             [label] => Erro
             [value] =>
             [hint] =>*/

            if ( ($valid->type == 'required') )
            {
                //valor requerido na GnutecaRepetiveField
                if ( $fieldObj instanceof GnutecaRepetitiveField )
                {
                    $tmpData = GnutecaRepetitiveField::getData($field, true);
                    $quant = 0;

                    if ( is_array($tmpData) )
                    {
                        foreach ($tmpData as $i=>$tmpD)
                        {
                            if ( !$tmpD->removeData )
                            {
                                $quant++;
                            }
                        }
                    }
                    
                    if ( $quant == 0 )
                    {
                        $errors[] = $this->getValidatorMessage( _M('The field "@1" is required.', $module), $valid);
                    }
                }
                elseif ( $data->$field == '' && $_REQUEST[$field] !== '0') //tem que ser pego do request para reconhecer o 0, o miolo esta filtrando este valor
                {
                    $errors[] = $this->getValidatorMessage( _M('The field "@1" is required.', $module), $valid);
                }
            }
            else if ($data->$field)
            {
            	$msg_invalid = $this->getValidatorMessage( _M('The field "@1" is invalid.', $module), $valid);
	            if ( ($valid->checker == 'REGEXP') && ($valid->regexp) )
	            {
	                $ok = ereg($valid->regexp, $data->$field);
	                if (!$ok)
	                {
	                    $errors[] = $msg_invalid;
	                }
	            }
                else if ( ($valid->id == 'date') )
                {
                    $mask = ereg("/", $data->$field) ? "dd/mm/yyyy" : null;
                    $mask = ereg("-", $data->$field) ? "yyyy-mm-dd" : $mask;

                    if(strlen($data->$field) && is_null(GnutecaDate::identifyDateMask($data->$field, null, false, $mask )))
                    {
                        $errors[] = $msg_invalid;
        			}

                }
                else if ( ($valid->checker == 'TIME') && (!ereg('^[0-9]{2}:[0-5][0-9]$', $data->$field)) )
                {
                	$errors[] = $msg_invalid;
                }
                else if ( ($valid->checker == 'EMAIL') && (!is_valid_email($data->$field)) )
                {
                    $errors[] = $msg_invalid;
                }
                else if ( ($valid->id == 'cep') && (!ereg("^[0-9]{5}-[0-9]{3}$", $data->$field)) )
                {
                    $errors[] = $msg_invalid;
                }
                //TODO CPF, CEP, CNPJ, PHONE, RANGE validator
                //TODO Fazer com que o primeiro campo no validado fique com focus no form
            }

			if ($valid->checker == 'unique')
			{
			    $tempData = GnutecaRepetitiveField::getData($data->GnutecaRepetitiveField, true);
			    if ( is_array($tempData) && $tempData )
			    {
			        foreach ($tempData as $l => $i)
			        {
			            if ( !is_numeric($data->arrayItemTemp) && !$i->removeData)
			            {
			                if ($i->$field == $data->$field)
			                {
			                    $errors[] = $this->getValidatorMessage( _M('The field "@1" must be unique.', $module ), $valid);
			                }
			            }
			        }
			    }
			}
        }

	    return $errors;
	}

} // Final da classe  Gnuteca Validators


/**
 * Enter description here...
 *
 */
class GnutecaUniqueValidator extends MValidator
{
    /**
     * Construct a GnutecaUniqueValidator
     *
     * @param string $field the id of the field to validate
     * @param string $label the label of validator
     * @param string $type the type of validator
     * @param string $msgerr the error message to show
     */
    function __construct( $field, $label=null, $type='optional', $msgerr=null )
    {
        parent::__construct();
        $this->id       = 'unique';
        $this->field    = $field;
        $this->label    = $label;
        $this->mask     = '';
        $this->type     = $type;
        $this->checker  = 'unique';
        //$this->min      = 0;
        //$this->max      = $max;
        $this->chars    = 'ALL';
        $this->msgerr   = $msgerr;
    }
}

/**
 * Enter description here...
 *
 */
class GnutecaDateValidator extends MValidator
{

	function __construct( $field, $label = null, $msgerr=null, $mask = "dd/mm/yyyy" )
	{
		parent::__construct();
		$this->id       = 'date';
		$this->field    = $field;
		$this->label    = $label;
		$this->mask     = $mask;
		$this->type     = 'date';
		$this->chars    = 'ALL';
		$this->msgerr   = $msgerr;
	}
}



function is_valid_email($email)
{
	if (!eregi("^[_a-z0-9-]+(\.[_a-z0-9-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)*(\.[a-z]{2,4})$", $email))
	{
        return FALSE;
	}
	return TRUE;
}


//-----------------------------------------------------
//Funcao: validaCNPJ($cnpj)
//Sinopse: Verifica se o valor passado  um CNPJ vlido
// Retorno: Booleano
// Autor: Gabriel Fres - www.codigofonte.com.br
//-----------------------------------------------------
function validaCNPJ($cnpj)
{
    if (strlen($cnpj) <> 18) return 0;
    $soma1 = ($cnpj[0] * 5) +

    ($cnpj[1] * 4) +
    ($cnpj[3] * 3) +
    ($cnpj[4] * 2) +
    ($cnpj[5] * 9) +
    ($cnpj[7] * 8) +
    ($cnpj[8] * 7) +
    ($cnpj[9] * 6) +
    ($cnpj[11] * 5) +
    ($cnpj[12] * 4) +
    ($cnpj[13] * 3) +
    ($cnpj[14] * 2);
    $resto = $soma1 % 11;
    $digito1 = $resto < 2 ? 0 : 11 - $resto;
    $soma2 = ($cnpj[0] * 6) +

    ($cnpj[1] * 5) +
    ($cnpj[3] * 4) +
    ($cnpj[4] * 3) +
    ($cnpj[5] * 2) +
    ($cnpj[7] * 9) +
    ($cnpj[8] * 8) +
    ($cnpj[9] * 7) +
    ($cnpj[11] * 6) +
    ($cnpj[12] * 5) +
    ($cnpj[13] * 4) +
    ($cnpj[14] * 3) +
    ($cnpj[16] * 2);
    $resto = $soma2 % 11;
    $digito2 = $resto < 2 ? 0 : 11 - $resto;
    return (($cnpj[16] == $digito1) && ($cnpj[17] == $digito2));
}
?>