<?php
/**
 * <--- Copyright 2005-2011 de Solis - Cooperativa de Solues Livres Ltda. e
 * Univates - Centro Universitrio.
 * 
 * Este arquivo  parte do programa Gnuteca.
 * 
 * O Gnuteca  um software livre; voc pode redistribu-lo e/ou modific-lo
 * dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao
 * do Software Livre (FSF); na verso 2 da Licena.
 * 
 * Este programa  distribudo na esperana que possa ser til, mas SEM
 * NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO
 * ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU/GPL em
 * portugus para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo
 * "LICENCA.txt", junto com este programa, se no, acesse o Portal do Software
 * Pblico Brasileiro no endereo www.softwarepublico.gov.br ou escreva para a
 * Fundao do Software Livre (FSF) Inc., 51 Franklin St, Fifth Floor, Boston,
 * MA 02110-1301, USA --->
 * 
 * Class
 *
 * @author Luiz Gilberto Gregory Filho [luz@solis.coop.br]
 *
 * @version $Id$
 *
 * \b Maintainers: \n
 * Eduardo Bonfandini [eduardo@solis.coop.br]
 * Jamiel Spezia [jamiel@solis.coop.br]
 * Luiz Gregory Filho [luiz@solis.coop.br]
 * Moises Heberle [moises@solis.coop.br]
 *
 * @since
 * Class created on 04/05/2009
 *
 **/
class GnutecaZ3950
{
    private $user;                              // string
    private $password;                          // string
    private $host;                              // string
    private $port;                              // int
    private $authenticate   = false;            // boolean
    private $options        = array();
    private $yazConnection  = null;

    private $errorCode      = null,
            $erroStr        = null;

    private $range          = null;
    private $tagSearch      = null;


    /**
     * Constructor Method
     */
    function __construct($host = null, $user = null, $password = null, $autoConnect = false)
    {
        $this->setHost($host);
        $this->setUser($user);
        $this->setPassword($password);

        $this->range->start     = 0;
        $this->range->length    = 20;

        if ( strlen($this->getUser()) && strlen($this->getPassword()) )
        {
            $this->setAuthenticate( true );
        }

        if ( $autoConnect )
        {
            $this->connect();
        }
    }

    /**
     * Verifica se o yaz (utilizado pelo Z3950) esta instalado;
     *
     *
     * @return <boolean> se yaz esta instalado
     *
     */
    public function isInstalled()
    {
        return function_exists('yaz_connect');
    }


    /**
     * Adiciona um clausula de condio para pesquisa
     *
     * @param char 5 $tag
     * @param string $content
     */
    public function addTagSearch($tag, $content, $operator = '@and')
    {
        $this->tagSearch[$tag]->content     = $content;
        $this->tagSearch[$tag]->operator    = $operator;
    }

    /**
     * Enter description here...
     *
     * @param unknown_type $tags
     */
    public function setTagSearch($tags = null)
    {
        $this->tagSearch = $tags;
    }

    /**
     * Seta o Usuario
     *
     * O usurio  necessario caso o metodo de envio seja atutenticado
     *
     * @param String
     */
    public function setUser($user)
    {
        $this->setConnectionOption('user', $user );
    }

    /**
     * Seta o password
     *
     * este parametro  necessario caso o metodo de envio seja autenticado
     *
     * @param String
     */
    public function setPassword($password)
    {
        $this->setConnectionOption('password', $password);
    }

    /**
     * Seta o servidor
     *
     * @param String
     */
    public function setHost($host)
    {
        $this->host = $host;
    }

    /**
     * Seta a porta de conexao
     *
     * @param Int
     */
    public function setPort($port)
    {
        $this->port = $port;
    }

    /**
     *
     * Seta se o metodo de conexao com o server  autenticado ou no
     *
     * @param (Boolean)
     */
    public function setAuthenticate($authenticate)
    {
        $this->authenticate = ( $authenticate && ( $this->getUser() && $this->getPassword() ) );
    }

    /**
     * retorna o usuario
     *
     * @return String
     */
    public function getUser()
    {
        return $this->getConnectionOption('user');
    }

    /**
     * retorna o password
     *
     * @return String
     */
    public function getPassword()
    {
        return $this->getConnectionOption('password');
    }

    /**
     * retorna o host
     *
     * @return String
     */
    public function getHost()
    {
        return $this->host;
    }

    /**
     * retorna a porta
     *
     * @return Integer
     */
    public function getPort()
    {
        return $this->port;
    }

    /**
     * retorna se o metodo de conexao  autenticado (authenticate true|false)
     *
     * @return (Boolean)
     */
    public function getAuthenticate()
    {
        return $this->authenticate;
    }

    /**
     * retorna a connexao com a o server
     *
     * @return resource
     */
    public function connect($host = null)
    {
        $host = is_null($host) ? $this->getHost() : $host;
        $this->setHost($host);

        if( ! strlen($this->getHost()) )
        {
            $this->errorCode = 1;
            return false;
        }

        $this->yazConnection = yaz_connect($this->getHost(),$this->options);

        if(!$this->yazConnection)
        {
            $this->errorCode = 2;
            return false;
        }

        return $this->yazConnection;
    }

    /**
     * seta a sintaxe da consulta
     *
     * @param string $sintax
     * @return boolean
     */
    public function setSintax($sintax = "usmarc")
    {
        if(!$this->yazConnection)
        {
            $this->errorCode = 3;
            return false;
        }

        if(!strlen($sintax))
        {
            $sintax = "usmarc";
        }

        return yaz_syntax($this->yazConnection, $sintax);
    }

    /**
     * Seta o numero de registros que sera buscaddoo
     *
     * @param int $start
     * @param  int $length
     * @return boolean
     */
    public function setRange($start = null, $length = null)
    {
        if(!$this->yazConnection)
        {
            $this->errorCode = 3;
            return false;
        }

        $this->range->start     = is_null($start)   ? $this->range->start   : $start;
        $this->range->length    = is_null($length)  ? $this->range->length  : $length;

        return yaz_range($this->yazConnection, $this->range->start, $this->range->length);
    }

    /**
     * Faz a procura dos materiais
     * Exemplo: @attr 1=1003 @attr 4=1 'machado'
     *
     * @param string $query
     * @return unknown
     */
    public function search($query = null)
    {
        if(!$this->yazConnection)
        {
            $this->errorCode = 3;
            return false;
        }

        if(!strlen($query))
        {
            if(!$query = $this->mountQuery())
            {
                return false;
            }
        }

        return yaz_search($this->yazConnection, "rpn", $query);
    }

    /**
     * monta a query de pesquisa com os termos setados
     *
     * @return string
     */
    public function mountQuery()
    {
        if(is_null($this->tagSearch))
        {
            $this->errorCode = 4;
            return false;
        }

        $query  = null;
        $fieldS = null;
        $conteS = null;

        foreach ($this->tagSearch as $id => $content)
        {
            if(is_null($fieldS))
            {
                $fieldS = "@attr 1=$id";
            }
            else
            {
                $fieldS = "@attr 1=1016"; // all Fields
            }

            $conteS = "@attr 4=1 ";
            $opera  = $content->operator;
            $value  = explode(" ", $content->content);

            if(count($value) > 1)
            {
                for ($x = 1; $x < count($value); $x++)
                {
                    $conteS.= " $opera ";
                }
            }

            $value = implode("' '", $value);
            $conteS.= " '$value'";

            break;
        }

        $query = str_replace("  ", " ", "$fieldS $conteS");

        //$query = "@attr 1=1004 @attr 4=1 @and 'machado' 'assis'";
        //$query = '@attr 1=1003 @attr 4=1 "knuth donald"';
        //$query = '@attrset gils @and @attr 1=4 art @attr 1=2000 company';

        return $query;
    }

    /**
     * Seta o tempo maximo de espera
     *
     * @param int $time
     * @return boolena
     */
    public function setTimeOut($time = 30)
    {
        $wait_options = array("timeout" => $time);
        return yaz_wait($wait_options);
    }

    /**
     * retorna o conteudo encontrado
     *
     * @param integer $start
     * @param integer $length
     * @return boolean
     */
    public function getRecord($start = null, $length = null, $returnType = "string")
    {
        if(!$this->yazConnection)
        {
            $this->errorCode = 3;
            return false;
        }

        $start  = is_null($start)   ? $this->range->start : $start;
        $length = is_null($length)  ? $this->range->length: $length;

        $return = array();
        for($x = $start; $x <= ($start + $length); $x++)
        {
            if($abc = yaz_record($this->yazConnection, $x, $returnType))
            {
                $return[$x] = $abc;
            }
        }

        return $return;
    }

    /**
     * Retorna o total de registros encontrados.
     *
     * @return int
     */
    public function getHits()
    {
        if(!$this->yazConnection)
        {
            $this->errorCode = 3;
            return false;
        }

        return yaz_hits($this->yazConnection);
    }

    /**
     * Transforma o conteudo resultante da pesquisa para um formato aceitavel pelo gnuteca
     *
     * @param array $result
     * @param string $currentType
     */
    public function makeResult($result, $currentType = "xml")
    {
        if(ereg("xml",$currentType))
        {
            return $this->makeXmlResult($result);
        }
    }

    /**
     * Trabalha o conteudo XML e converte para Gnuteca Catalogue
     *
     * @param z3950 result $result
     * @return array
     */
    public function makeXmlResult($result)
    {
        require_once(dirname(__FILE__) . "/GnutecaXML.class.php");

        $returnList = array();
        if(!count($result))
        {
            return $returnList;
        }

        $r = array();

        foreach ($result as $index => $xmlResult)
        {
            $aux        = new GnutecaXML($xmlResult);
            $array      = $aux->getResult();
            $array      = $array['record'][0];
            $returnList = array();

            foreach ($array as $tagX => $elements)
            {
                foreach ($elements as $content)
                {
                    $tag    = isset($content['_attributes_']['tag']) && strlen($content['_attributes_']['tag']) ? $content['_attributes_']['tag'] : false;

                    if($tagX == 'controlfield' && $tag)
                    {
                        $line   = isset($returnList[$tag]->subfields['a']) ? count($returnList[$tag]->subfields['a']) : 0;
                        $returnList[$tag]->subfields['a'][$line]->content = $content['_content_'];
                    }
                    elseif($tagX == 'datafield' && $tag)
                    {
                        $subfields = isset($content['subfield']) ? $content['subfield'] : false;
                        if(!$subfields)
                        {
                            continue;
                        }

                        foreach ($subfields as $subfieldContent)
                        {
                            $line   = isset($returnList[$tag]->subfields[$subfieldContent['_attributes_']['code']]) ? count($returnList[$tag]->subfields[$subfieldContent['_attributes_']['code']]) : 0;
                            $returnList[$tag]->subfields[$subfieldContent['_attributes_']['code']][$line]->content = $subfieldContent['_content_'];
                        }

                        $returnList[$tag]->ind1 = isset($content['_attributes_']['ind1']) && strlen($content['_attributes_']['ind1']) ? $content['_attributes_']['ind1'] : false;
                        $returnList[$tag]->ind2 = isset($content['_attributes_']['ind2']) && strlen($content['_attributes_']['ind2']) ? $content['_attributes_']['ind2'] : false;
                    }
                }
            }

            $r[$index] = $returnList;
        }

        return $r;
    }

    /**
     * Este mtodo faz uma relao entre campos do marc com campos do z3950
     *
     * @param unknown_type $tag
     */
    public function getRelationOfMarc21AndZ3950($id = null)
    {
        //FONTE : http://www.loc.gov/z3950/agency/defns/bib1.html
        // DOCUMENTO LOCAL gnuteca3/misc/docs/z3950/semantic.txt

        $opts = Z3950_SEARCH_OPTIONS;
        $opts = explode(";", $opts);

        foreach ($opts as $line)
        {
            if(ereg("//", $line) || ereg("#", $line))
            {
                continue;
            }

            list($id, $label) = explode("=", $line);

            if(!strlen(trim($id)) || !strlen(trim($label)))
            {
                continue;
            }
            $z3950Marc21[trim($id)] = trim($label);
        }


        return $z3950Marc21;
    }

    /**
     * Retorna a string do erro
     *
     * @return string
     */
    public function getErrorStr()
    {
        if(is_null($this->getErrorCode()))
        {
            return null;
        }

        switch ($this->getErrorCode())
        {
            case 1 : return "O atributo \$host  invlido";
            case 2 : return "No foi possvel connectar ao host: ". $this->getHost() .";\n Verifique se o host  vlido ou se a bibliotca 'yaz' esta instalada corretamente.";
            case 3 : return "No foi possvel setar a sintaxe;\n Verifique se a conexo com o server  vlida.";
            case 4 : return "No foi possvel montar a query de pesquisa, no ha termos setados.";
            case 5 : return "No  possvel realizar a pesquisa pois a query esta vazia.";
        }
    }

    /**
     * retorna o codigo do erro;
     *
     * @return integer
     */
    public function getErrorCode()
    {
        return $this->errorCode;
    }


    /**
     * Seta opes de conexo na varivel $options.
     *
     * @return void
     */
    public function setConnectionOption($attribute,$value)
    {
        if ( $value )
        {
            $this->options[$attribute] = $value;
        }
        else
        {
            unset($this->options[$attribute]);
        }
    }


    /**
     * Retorna opes de conexo na varivel $options.
     *
     * @return string
     */
    public function getConnectionOption($attribute)
    {
        if ( isset($this->options['attribute']) )
        {
            $value = $this->options['attribute'];
        }
        else
        {
            $value = null;
        }

        return $value;
    }


}
?>