<?php

/**
 * <--- Copyright 2005-2011 de Solis - Cooperativa de Solues Livres Ltda. e
 * Univates - Centro Universitrio.
 * 
 * Este arquivo  parte do programa Gnuteca.
 * 
 * O Gnuteca  um software livre; voc pode redistribu-lo e/ou modific-lo
 * dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao
 * do Software Livre (FSF); na verso 2 da Licena.
 * 
 * Este programa  distribudo na esperana que possa ser til, mas SEM
 * NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO
 * ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU/GPL em
 * portugus para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo
 * "LICENCA.txt", junto com este programa, se no, acesse o Portal do Software
 * Pblico Brasileiro no endereo www.softwarepublico.gov.br ou escreva para a
 * Fundao do Software Livre (FSF) Inc., 51 Franklin St, Fifth Floor, Boston,
 * MA 02110-1301, USA --->
 * 
 * Class Printer
 *
 * @author Luiz Gilberto Gregory Filho [luiz@solis.coop.br]
 *
 * @version $Id$
 *
 * \b Maintainers: \n
 * Eduardo Bonfandini [eduardo@solis.coop.br]
 * Jamiel Spezia [jamiel@solis.coop.br]
 * Luiz Gregory Filho [luiz@solis.coop.br]
 * Moises Heberle [moises@solis.coop.br]
 *
 * @since
 * Class created on 14/07/2009
 *
 **/

require_once(str_replace("classes", "etc", str_replace("\\", "/", dirname(__FILE__))) . "/gnutecaPrinterServer.conf.php");

class Printer
{

    /**
     * attributes
     */
    private $printer            = null;
    private $errorCode          = null;
    private $printersClassPath  = null;


    /**
     * Constructor method
     */
    function __construct($printer = null, $model =  null)
    {
        $this->printersClassPath = str_replace("\\", "/", dirname(__FILE__)) . "/printers";

        $this->setPrinterModel($printer, $model);
    }



    /**
     * Seta a impressora que esta instalada e cria o objeto da impressora
     *
     * @param string $printer
     * @param string $model
     */
    public function setPrinterModel($printer = null, $model =  null)
    {
        if(is_null($printer))
        {
            $printer    = PRINTER_MARK;
        }

        if(is_null($model))
        {
            $model      = PRINTER_MODEL;
        }

        // SELECIONA A IMPRESSORA QUE SERA UTILIZADA PARA IMPRESSO
        switch (strtolower($printer))
        {
            case "daruma":
                require_once("{$this->printersClassPath}/DarumaPrinter.class.php");
                $this->printer = new DarumaPrinter("", $model);
                break;

            default:
                require_once("{$this->printersClassPath}/DefaultPrinter.class.php");
                $this->printer = new DefaultPrinter();
                break;
        }

        if(is_null($this->printer))
        {
            $this->errorCode = 1;
            return false;
        }

        return true;
    }



    /**
     * seta o conteudo a ser impresso
     *
     * @param string $content
     */
    public function setPrintContent($content)
    {
        if(is_null($this->printer))
        {
            $this->errorCode = 1;
            return false;
        }

        $this->printer->setPrintContent($content);
    }



    /**
     * Imprime o conteudo
     *
     * Este metodo abre o destino e vai escrevendo diretamente nele.
     *
     */
    public function __print()
    {
        if(is_null($this->printer))
        {
            $this->errorCode = 1;
            return false;
        }

        $handler = fopen(PRINTER_DESTINATION, "w");
        if(!$handler)
        {
            $this->errorCode = 2;
            return false;
        }

        $content = $this->printer->getPrintContent();
        $content = $this->makeContent($content);

        if(!fwrite($handler, $content))
        {
            $this->errorCode = 3;
            return false;
        }

        fclose($handler);
        return true;
    }


    /**
     * Este metodo aplica ajustes no conteudo independentemente do modelo da impressora
     *
     * @param string $content
     * @return string
     */
    private function makeContent($content)
    {
        if(REPLACE_ACENTUACAO)
        {
            // SUBSTITUI ACENTUACAO LETRA A
            $content = ereg_replace("[]",   'a', $content);
            $content = ereg_replace("[]",   'A', $content);

            // SUBSTITUI ACENTUACAO LETRA E
            $content = ereg_replace("[]",    'e', $content);
            $content = ereg_replace("[]",    'E', $content);

            // SUBSTITUI ACENTUACAO LETRA I
            $content = ereg_replace("[]",    'i', $content);
            $content = ereg_replace("[]",    'I', $content);

            // SUBSTITUI ACENTUACAO LETRA O
            $content = ereg_replace("[]",   'o', $content);
            $content = ereg_replace("[]",   'O', $content);

            // SUBSTITUI ACENTUACAO LETRA U
            $content = ereg_replace("[]",    'u', $content);
            $content = ereg_replace("[]",    'U', $content);

            // SUBSTITUI ACENTUACAO LETRA 
            $content = ereg_replace("[]",      'c', $content);
            $content = ereg_replace("[]",      'C', $content);
        }

        return $content;
    }

}

?>