<?php

/**
 * <--- Copyright 2005-2011 de Solis - Cooperativa de Solues Livres Ltda. e
 * Univates - Centro Universitrio.
 * 
 * Este arquivo  parte do programa Gnuteca.
 * 
 * O Gnuteca  um software livre; voc pode redistribu-lo e/ou modific-lo
 * dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao
 * do Software Livre (FSF); na verso 2 da Licena.
 * 
 * Este programa  distribudo na esperana que possa ser til, mas SEM
 * NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO
 * ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU/GPL em
 * portugus para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo
 * "LICENCA.txt", junto com este programa, se no, acesse o Portal do Software
 * Pblico Brasileiro no endereo www.softwarepublico.gov.br ou escreva para a
 * Fundao do Software Livre (FSF) Inc., 51 Franklin St, Fifth Floor, Boston,
 * MA 02110-1301, USA --->
 * 
 * Class Generic Gnuteca Receipt;
 *
 * @author Eduardo Bonfandini [eduardo@solis.coop.br]
 *
 * @version $Id$
 *
 * \b Maintainers: \n
 * Eduardo Bonfandini [eduardo@solis.coop.br]
 * Jamiel Spezia [jamiel@solis.coop.br]
 * Moises Heberle [moises@solis.coop.br]
 *
 * @since
 * Class created on 28/06/2010
 *
 **/

$MIOLO = MIOLO::getInstance();
$MIOLO->uses('/classes/receipt/ReceiptWork.class.php', 'gnuteca3');

class Receipt extends GnutecaFunction
{
    //the resultant content
    public $content;
    // if is to post mail
    public $isPostable;
    // if is to print
    public $isPrintable;
    //relation of varios receipts
    public $itens;
    //the id of person of this receipt
    public $personId; //$USER_CODE
    //the name of person of this receipt
    public $personName; //$USER_NAME
    //the email of person of this receipt
    public $email;
    //the id of LibraryUnit
    public $libraryUnitId;
    //the name of libraryUnit
    public $libraryName; //$LIBRARY_UNIT_DESCRIPTION
    //bus material used to get author and title
    protected $busMaterial;
    //path to store the pdf
    public $storagePath;
    //GnutecaPdf object
    public $pdf;
    //the receipt generate date
    public $date; //$DATE
    //the receipt generate time
    public $time; //$TIME
    //the system operator
    public $operator; //$OPERATOR
    //operao (e tipo do recibo)
    public $operation; //$OPERATION
    //model to generate the receipt
    public $model;
    //the model for detail (itens)
    public $modelDetail;
    //modelo de assunto de email
    public $modelMailSubject;
    //modelo de contedo de email
    public $modelMailContent;
    //mensagem a ser passada para a operao geral
    public $message;

    public function __construct($data, $isPostable = false, $isPrintable = false )
    {
        parent::__construct();
        $MIOLO  = MIOLO::getInstance();
        $this->setVariable('$LN', "\r\n");
        $this->setVariable('$SP', " ");
        $this->setData($data);
        $this->storagePath = $MIOLO->getAbsolutePath( $MIOLO->getConf('options.receiptPdfFilePath'), MIOLO::getCurrentModule() );
        $this->mail = new GnutecaMail();
        //get the default business
        $this->isPrintable = $isPrintable;
        $this->isPostable  = $isPostable;
        $this->businessMaterial = $MIOLO->getBusiness('gnuteca3', 'BusMaterial');
    }

    /**
     * Define the data of receipt, it is diferent for each receipt
     *
     * @param <stdclass> $data Define the data of receipt
     */
    public function setData($data)
    {
        $array = (Array) $data;
        
        foreach ( $array as $line => $info )
        {
            $this->$line = $info;
        }
    }

    /**
     * Return the data of receipt
     *
     * @return <stdClasss> the data of receipt
     *
     */
    public function getData()
    {
        return (Object) get_object_vars($this);
    }

    /**
     * Add and item to item list
     * @param <type> $item Add and item to item list
     */
    public function addItem($item)
    {
        $this->itens[] = $item;
    }

    /**
     * Define all itens
     *
     * @param array $itens Define all itens
     *
     */
    public function setItens(Array $itens)
    {
        $this->itens = $itens;
    }

    /**
     * Return all itens
     *
     * @return <array> Return all itens
     */
    public function getItens()
    {
        return $this->itens;
    }

    /**
     * Pass each item set it model, calling it parseData and making it generate.
     *
     * It set the generated string to $WORKS variable
     *
     * @return <string> the generate string;
     */
    public function parseItemData()
    {
        $itens = $this->getItens();

        if ( is_array( $itens ) )
        {
            foreach ( $itens as $line => $item )
            {
                $item->parseData();

                //set the model if needed
                if ( !$item->getModel() )
                {
                    $item->setModel( $this->getModelDetail() );
                }

                $generate .= $item->generate();
            }
        }

        $this->setVariable('$WORKS', $generate);

        return $generate;
    }

    /**
     * Parse the data of receipt, finding needed information in database
     * and set the variables
     */
    public function parseData()
    {
        $MIOLO     = MIOLO::getInstance();

        /*
         * these are security functions, this data has to come with object
         * if don't come, the class search to this.
         * Do the default variables too (operator, date, time)
         */

        $this->setVariable('$USER_CODE', $this->personId);

        if ( !$this->personName )
        {
            $busPerson = $MIOLO->getBusiness('gnuteca3', 'BusPerson');
            $person    = $busPerson->getBasicPersonInformations( $this->personId );

            $this->personName = $person->name;
            $this->email      = $person->email;
        }

        $this->setVariable('$USER_NAME', $this->personName);

        if ( !$this->operator )
        {
            $this->operator = GnutecaOperator::getOperatorId();
        }

        $this->setVariable('$OPERATOR',  $this->operator);

        if ( !$this->libraryName )
        {
            $busLibrary = $MIOLO->getBusiness('gnuteca3', 'BusLibraryUnit');
            $this->libraryName     = $busLibrary->getLibraryName( $this->libraryUnitId );
        }

        $this->setVariable('$LIBRARY_UNIT_DESCRIPTION', $this->libraryName);
        $this->setVariable('$DATE',  date('d/m/Y'));
        $this->setVariable('$TIME',  date('H:i'));
        $this->setVariable('$RECEIPT_FOOTER', $this->getReceiptHash());
        $this->setVariable('$OPERATION', $this->operation);

        $itens = $this->getItens();

        if ( $itens )
        {
            $this->parseItemData();
        }
    }

    /**
     * Return the generate content
     *
     * @return <string> the generate content
     *
     */
    public function getContent()
    {
        return $this->content;
    }


    /**
     * Define the content of receipt
     *
     * @param <string> $content the content of receipt
     */
    public function setContent($content)
    {
        $this->content = $content;
    }

    /**
     * Define if receipt if printable
     * @param <boolean> $isPrintable
     */
    public function setIsPrintable($isPrintable)
    {
        $this->isPrintable = $isPrintable;
    }

    /**
     * Return if receipt is printable
     *
     * @return <boolean> if receipt is printable
     *
     */
    public function getIsPrintable()
    {
        return $this->isPrintable;
    }

    /**
     * Define if receipt can be postable (mailed)
     *
     * @param <boolean> $isPostable if receipt can be postable (mailed)
     *
     */
    public function setIsPostable($isPostable)
    {
        $this->isPostable = $isPostable;
    }

    /**
     * Return if receipt can be postable (mailed)
     *
     * @return <boolean> if receipt can be postable (mailed)
     *
     */
    public function getIsPostable()
    {
        return $this->isPostable;
    }

    /**
     * Define the receipt model, it is a static definition
     *
     * @param <type> $model
     * @param <type> $detailModel
     */
    public function setModel( $model, $detailModel , $modelMailSubject, $modelMailContent )
    {
        $this->model            = $model;
        $this->modelDetail      = $detailModel;
        $this->modelMailSubject = $modelMailSubject;
        $this->modelMailContent = $modelMailContent;
    }

    /**
     * Return the static model of this type of receipt
     *
     * @return <string> the static model of this type of receipt
     *
     */
    public function getModel()
    {
        return $this->model;
    }

    /**
     * Return the static model of detailof this type of receipt
     *
     * @return <string>  the static model of this type of receipt
     */
    public function getModelDetail()
    {
        return $this->modelDetail;
    }

    /**
     * Gera o hash para o recibo
     *
     * @param integer $personId
     * @param object $itensObject
     * @return hash md5
     */
    public function getReceiptHash()
    {
        $stringHash = md5(HASH_KEY.$this->personId.$this->libraryUnitId);

        foreach ($this->itens as $line => $obj)
        {
            $stringHash.= $obj->itemNumber.$obj->loanDate ? $obj->loanDate : $obj->returnForecastDate;
        }

        return md5($stringHash);
    }

    /**
     * Generate the string of receipt
     *
     * @return <string> the generated string of receipt
     */
    public function generate()
    {
        //caso precise gerar o recibo de alguma outra forma que no seja pelo gnuteca receipt pode pegar os modelos por aqui
        $this->parseData();
        $result = $this->interpret( $this->model );
        $this->content = $result;
        
        if ( $this->isPostable )
        {
            $this->send();
        }

        if ( $this->isPrintable )
        {
            return $result;
        }
        else
        {
            return null;
        }
    }

    /**
     * Return the generated string of receipt
     *
     * @return <type> Return the generated string of receipt
     */
    public function __toString()
    {
        return $this->getContent();
    }

    /**
     * Definr a message to receipt
     *
     * @param <string> $message the message of receipt, will be colected in operation
     */
    public function addMessage( $message )
    {
        $this->message = $message;
    }

    /**
     * Return the message defined in receipt
     */
    public function getMessage()
    {
        return $this->message;
    }

    /**
     *
     * Create a pdf of the receipt
     *
     */
    public function createPDF()
    {
        $personNameToFile = str_replace(' ', '', ucwords( strtolower( iconv('UTF-8', 'ASCII//TRANSLIT', $this->personName) ) ) ) ;
        $this->pdf = new GnutecaPDF( $this->storagePath . "/gnutecaReceipt_".$this->personId.'_'.$personNameToFile.'_'. md5(time()+ microtime()) .".pdf"  );
        $this->pdf->setTextContent( $this->getContent() );
        $generate = $this->pdf->generate();

        if ( !$generate )
        {
            $this->addMessage( _M('No foi possvel gerar recibo para @1', MIOLO::getCurrentModule(), "{$this->personId} - {$this->personName}"));
            return false;
        }

        return true;
    }

    /**
     * Return the file path (filename) of generated PDF
     *
     * @return <string> the file path (filename) of generated PDF
     */
    public function getPdfFileName()
    {
        if ( $this->pdf )
        {
            return $this->pdf->getFilePath();
        }
    }

    /**
     * Send the email if is postable.
     *
     * @param fake if you need a fake send (that not realy send, but create objet and put in session, use fake = true
     *
     */
    public function send( $fake = false )
    {
        //if not to post, don't send email
        if ( !$this->isPostable && !$fake )
        {
            return false;
        }

        if ( !strlen( $this->email ) )
        {
            $this->addMessage( _M('Email no encontrado para pessoa <b>@1</b>', MIOLO::getCurrentModule(), "{$this->personId} - {$this->personName}"  ));
            return false;
        }

        $this->createPDF();
        $file = $this->pdf->getFilePath();

        if ( !$file )
        {
            $this->addMessage( _M('Impossvel gerar recibo <b>@1</b>', MIOLO::getCurrentModule(), "{$this->personId} - {$this->personName}"  ));
            return false;
        }

        $receiptMail = new ReceiptMail($this->email, $this->modelMailSubject, $this->interpret( $this->modelMailContent), $file);
        $send        = $receiptMail->send(true, $fake);

        if ( $send == 1 )
        {
            $this->addMessage( _M('Enviado email de <b>@1</b> para @2', $this->module, $this->operation, "{$this->personId} - {$this->personName} - {$this->email}"  ));
            return true;
        }
        else
        {
            $this->addMessage( _M('Falha ao enviar email de <b>@1</b> para @2. Mensagem do servidor: <b>@3</b>.',$this->module, $this->operation, "{$this->personId} - {$this->personName} - {$this->email}",utf8_decode( $receiptMail->getError()) ) );
            return false;
        }      
    }
}
?>