<?php

/**
 * <--- Copyright 2005-2011 de Solis - Cooperativa de Solues Livres Ltda. e
 * Univates - Centro Universitrio.
 * 
 * Este arquivo  parte do programa Gnuteca.
 * 
 * O Gnuteca  um software livre; voc pode redistribu-lo e/ou modific-lo
 * dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao
 * do Software Livre (FSF); na verso 2 da Licena.
 * 
 * Este programa  distribudo na esperana que possa ser til, mas SEM
 * NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO
 * ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU/GPL em
 * portugus para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo
 * "LICENCA.txt", junto com este programa, se no, acesse o Portal do Software
 * Pblico Brasileiro no endereo www.softwarepublico.gov.br ou escreva para a
 * Fundao do Software Livre (FSF) Inc., 51 Franklin St, Fifth Floor, Boston,
 * MA 02110-1301, USA --->
 * 
 * Class used to send receipt mail. It is stored in session, to resend
 *
 * @author Eduardo Bonfandini [eduardo@solis.coop.br]
 *
 * @version $Id$
 *
 * \b Maintainers: \n
 * Eduardo Bonfandini [eduardo@solis.coop.br]
 * Jamiel Spezia [jamiel@solis.coop.br]
 * Moises Heberle [moises@solis.coop.br]
 *
 * @since
 * Class created on 28/06/2010
 *
 **/

class ReceiptMail
{
    protected $email;
    protected $content;
    protected $subject;
    protected $attachment;
    protected $error;

    public function __construct($email, $subject, $content , $attachment)
    {
        $this->email        = $email;
        $this->content      = $content;
        $this->subject      = $subject;
        $this->attachment   = $attachment;
    }

    public function setEmail($email)
    {
        $this->email = $email;
    }

    public function getEmail($email)
    {
        return $this->email;
    }

    public function setContent($content)
    {
        $this->content = $content;
    }

    public function getContent($content)
    {
        return $this->content;
    }

    public function setSubject($subject)
    {
        $this->subject = $subject;
    }

    public function getSubject($subject)
    {
        return $this->subject;
    }

    public function setAttachment($attachment)
    {
        $this->attachment = $attachment;
    }

    public function getAttachment()
    {
        return $this->attachment;
    }

    public function setError($error)
    {
        $this->error = $error;
    }

    public function getError()
    {
        return $this->error;
    }

    public function resend()
    {
        $this->send(false);
    }

    public function send( $storeInSession=true , $fake = false )
    {
        $MIOLO   = MIOLO::getInstance();
        $session = $MIOLO->getSession();

        //stores email in session
        if ( $storeInSession )
        {
            $receipitsInSession = $session->getValue('receiptMailStorage');

            //essa converso  necessria em funo de problemas de sesso na circulao de material
            $receiptMailStdClass = new stdClass();
            $receiptMailStdClass->email         = $this->email;
            $receiptMailStdClass->content       = $this->content;
            $receiptMailStdClass->subject       = $this->subject;
            $receiptMailStdClass->attachment    = $this->attachment;

            $receipitsInSession[] = $receiptMailStdClass;
            $session->setValue('receiptMailStorage',$receipitsInSession);
        }

        $mail = new GnutecaMail();
        $mail->setAddress($this->email);
        $mail->setSubject($this->subject);
        $mail->setContent($this->content);
        $mail->addAttachment($this->attachment);

        if ( !$fake )
        {
            $send =  $mail->send();

            if ( !$send )
            {
                $this->error = $mail->getError();
            }
        
            return $send;
        }
        else
        {
            return true;
        }
    }

    /**
     * Return the list of Receipts stored in session
     * @return <array of ReceiptMail> return the list of Receipts stored in session
     */
    public static function getStoredEmails()
    {
        $MIOLO   = MIOLO::getInstance();
        $session = $MIOLO->getSession();
        $emails  = $session->getValue('receiptMailStorage');

        //essa converso  necessria em funo de problemas de sesso na circulao de material
        if ( is_array( $emails ) )
        {
            foreach ( $emails as $line => $email )
            {
                $emails[$line] = new ReceiptMail($email->email, $email->subject, $email->content, $email->attachment);
            }
        }

        return $emails;
    }

    public static function resendStoredEmails()
    {
        $emails  = ReceiptMail::getStoredEmails();

        if ( is_array($emails) )
        {
            foreach ($emails as $line => $email )
            {
                $email->resend();
            }
        }

        return $emails;
    }

    /**
     * Clear the list of ReceiptMail
     */
    public static function clearStoredEmails()
    {
        $MIOLO   = MIOLO::getInstance();
        $session = $MIOLO->getSession();
        $session->setValue('receiptMailStorage', null);
    }
}

?>
