<?php
/**
 * <--- Copyright 2005-2011 de Solis - Cooperativa de Solues Livres Ltda. e
 * Univates - Centro Universitrio.
 * 
 * Este arquivo  parte do programa Gnuteca.
 * 
 * O Gnuteca  um software livre; voc pode redistribu-lo e/ou modific-lo
 * dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao
 * do Software Livre (FSF); na verso 2 da Licena.
 * 
 * Este programa  distribudo na esperana que possa ser til, mas SEM
 * NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO
 * ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU/GPL em
 * portugus para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo
 * "LICENCA.txt", junto com este programa, se no, acesse o Portal do Software
 * Pblico Brasileiro no endereo www.softwarepublico.gov.br ou escreva para a
 * Fundao do Software Livre (FSF) Inc., 51 Franklin St, Fifth Floor, Boston,
 * MA 02110-1301, USA --->
 * 
 * This file handles the connection and actions for gtcDomain table
 *
 * @author Moises Heberle [moises@solis.coop.br]
 *
 * $version: $Id$
 *
 * \b Maintainers \n
 * Eduardo Bonfandini [eduardo@solis.coop.br]
 * Jamiel Spezia [jamiel@solis.coop.br]
 * Moises Heberle [moises@solis.coop.br]
 *
 * @since
 * Class created on 11/07/2010
 *
 **/

/**
 * Class to manipulate the basConfig table
 **/

class BusinessGnuteca3BusDomain extends GnutecaBusiness
{
    public $domainId;
    public $sequence;
    public $key;
    public $abbreviated;
    public $label;

    public $domainIdS;
    public $sequenceS;
    public $keyS;
    public $abbreviatedS;
    public $labelS;

    public function __construct()
    {
        $this->MIOLO = MIOLO::getInstance();
        $this->module = 'gnuteca3';
        parent::__construct();
        $this->columns = 'domainId,
                      sequence,
                      key,
                      abbreviated,
                      label';
        $this->tables  = 'gtcDomain';
    }

    public function listDomain($domainId, $returnAsObject = false, $returnAssociative = false)
    {
    	$this->clear();
        $this->setColumns('key, label');
        $this->setTables($this->tables);
        $this->setWhere('domainId = ?');
        $this->setOrderBy('domainId, sequence');
        $sql = $this->select(array($domainId));
        $query = $this->query( $sql, $returnAsObject );
        
        if ($returnAssociative && $query)
        {
            $result = array();
            foreach  ($query as $v)
            {
                $result[ $v[0] ] = $v[1];
            }
            return $result;
        }
        else
        {
            return $query;
        }
    }

    public static function listForSelect($domainId,$returnAsObject = false,  $returnAssociative = false)
    {
        $MIOLO  = MIOLO::getInstance();
        $module = 'gnuteca3';

        $busDomain = $MIOLO->getBusiness($module, 'BusDomain');
        return $result = $busDomain->listDomain($domainId, $returnAsObject, $returnAssociative);
    }

    public function getDomain($domainId, $sequence)
    {
        $this->clear();
        $this->setColumns($this->columns);
        $this->setTables($this->tables);
        $this->setWhere('domainId = ? AND sequence = ?');
        $sql = $this->select(array($domainId, $sequence));
        $rs  = $this->query($sql, true);
        $this->setData($rs[0]);
        return $this;
    }


    /**
     * Do a search on the database table handled by the class
     *
     * @param $filters (object): Search filters
     *
     * @return (array): An array containing the search results
     **/
    public function searchDomain()
    {
        $this->clear();

        if ( !empty($this->domainIdS) )
        {
            $this->setWhere('lower(domainId) LIKE lower(?)');
            $data[] = $this->domainIdS . '%';
        }
        if ( !empty($this->sequenceS) )
        {
            $this->setWhere('sequence = ?');
            $data[] = $this->sequenceS;
        }
        if ( !empty($this->keyS) )
        {
            $this->setWhere('lower(key) LIKE lower(?)');
            $data[] = $this->keyS;
        }
        if ( !empty($this->abbreviatedS) )
        {
            $this->setWhere('lower(abbreviated) LIKE lower(?)');
            $data[] = $this->abbreviatedS . '%';
        }
        if ( !empty($this->labelS) )
        {
            $this->setWhere('lower(label) LIKE lower(?)');
            $data[] = $this->labelS . '%';
        }

        $this->setColumns($this->columns);
        $this->setTables($this->tables);
        $this->setOrderBy('domainId, sequence');
        $sql = $this->select($data);
        $rs  = $this->query($sql);

        return $rs;
    }

    public function insertDomain()
    {
        $data = array(
            $this->domainId,
            $this->sequence,
            $this->key,
            $this->abbreviated,
            $this->label
        );

        $this->clear();
        $this->setTables($this->tables);
        $this->setColumns($this->columns);
        $sql = $this->insert($data);
        $rs  = $this->execute($sql);
        
        return $rs;
    }

    public function updateDomain()
    {
        $data = array(
            $this->key,
            $this->abbreviated,
            $this->label,
            $this->domainId,
            $this->sequence
        );

        $this->clear();
        $this->setTables($this->tables);
        $this->setColumns('
            key,
            abbreviated,
            label
        ');
        $this->setWhere('domainId = ? AND sequence = ?');
        $sql = $this->update($data);
        $rs  = $this->execute($sql);

        return $rs;
    }

    public function deleteDomain($domainId, $sequence)
    {
        $data = array(
            $domainId,
            $sequence
        );

        $this->clear();
        $this->setTables($this->tables);
        $this->setWhere('domainId = ? AND sequence = ?');
        $sql = $this->delete($data);
        $rs  = $this->execute($sql);

        return $rs;
    }
}
?>
