<?php
/**
 * <--- Copyright 2005-2011 de Solis - Cooperativa de Solues Livres Ltda. e
 * Univates - Centro Universitrio.
 * 
 * Este arquivo  parte do programa Gnuteca.
 * 
 * O Gnuteca  um software livre; voc pode redistribu-lo e/ou modific-lo
 * dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao
 * do Software Livre (FSF); na verso 2 da Licena.
 * 
 * Este programa  distribudo na esperana que possa ser til, mas SEM
 * NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO
 * ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU/GPL em
 * portugus para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo
 * "LICENCA.txt", junto com este programa, se no, acesse o Portal do Software
 * Pblico Brasileiro no endereo www.softwarepublico.gov.br ou escreva para a
 * Fundao do Software Livre (FSF) Inc., 51 Franklin St, Fifth Floor, Boston,
 * MA 02110-1301, USA --->
 * 
 * This file handles the connection and actions for gtcExemplaryStatusHistory table
 *
 * @author Moises Heberle [moises@solis.coop.br]
 *
 * @version $Id$
 *
 * \b Maintainers \n
 * Eduardo Bonfandini [eduardo@solis.coop.br]
 * Jamiel Spezia [jamiel@solis.coop.br]
 * Luiz Gregory Filho [luiz@solis.coop.br]
 * Moises Heberle [moises@solis.coop.br]
 * Sandro Roberto Weisheimer [sandrow@solis.coop.br]
 *
 * @since
 * Class created on 02/08/2008
 *
 **/


/**
 * Class to manipulate the basConfig table
 **/
class BusinessGnuteca3BusExemplaryStatusHistory extends GnutecaBusiness
{
    public $cols, $table;

    public $itemNumber;
    public $exemplaryStatusId;
    public $libraryUnitId;
    public $date;
    public $operator;

    public $itemNumberS;
    public $exemplaryStatusIdS;
    public $libraryUnitIdS;
    public $dateS;
    public $beginDateS;
    public $endDateS;
    public $operatorS;


    /**
     * Class constructor
     **/
    function __construct()
    {
        parent::__construct();

        $this->table  = 'gtcExemplaryStatusHistory';

        $this->cols = 'itemNumber,
                      exemplaryStatusId,
                      libraryUnitId,
                      date,
                      operator';
    }


    /**
     * Insert a new record
     *
     * @return TRUE if succed, otherwise FALSE
     **/
    public function insertExemplaryStatusHistory()
    {
        if (!$this->operator)
        {
        	$this->operator = GnutecaOperator::getOperatorId();
        }

        $this->clear();
        $this->setColumns($this->cols);
        $this->setTables($this->table);
        $sql = $this->insert( $this->associateData( $this->cols ) );

        $rs  = $this->execute($sql);
        return $rs;
    }



    public function updateExemplaryStatusHistory()
    {

    }


    /**
     * Do a search on the database table handled by the class
     *
     * @return (Array): An array containing the search results
     **/
    public function searchExemplaryStatusHistory()
    {
        $this->clear();

        if ($this->itemNumberS)
        {
            $this->setWhere('A.itemNumber = ?');
            $data[] = $this->itemNumberS;
        }

        if ($this->exemplaryStatusIdS)
        {
            $this->setWhere('A.exemplaryStatusId = ?');
            $data[] = $this->exemplaryStatusIdS;
        }

        if ($this->libraryUnitIdS)
        {
            $this->setWhere('A.libraryUnitId in (' . $this->libraryUnitIdS . ')' );
        }

        if ($this->dateS)
        {
            $this->setWhere('date(A.date) = ?');
            $data[] = $this->dateS;
        }
        if ($this->beginDateS)
        {
            $this->setWhere('date(A.date) >= ?');
            $data[] = $this->beginDateS;
        }
        if ($this->endDateS)
        {
            $this->setWhere('date(A.date) <= ?');
            $data[] = $this->endDateS;
        }

        if ($this->operatorS)
        {
            $this->setWhere('lower(A.operator) LIKE lower(?)');
            $data[] = '%' . $this->operatorS . '%';
        }

        $tables  = '    gtcExemplaryStatusHistory   A
            INNER JOIN  gtcExemplaryStatus          B
                    ON  (A.exemplaryStatusId = B.exemplaryStatusId)
            INNER JOIN  gtcLibraryUnit              C
                    ON  (A.libraryUnitId = C.libraryUnitId)
            INNER JOIN  gtcExemplaryControl         D
                    ON  (A.itemNumber = D.itemNumber)';
        $columns = 'D.controlNumber,
                    A.itemNumber,
                    null AS title,
                    null AS author,
                    B.description,
                    C.libraryName,
                    A.date,
                    A.operator';

        $this->setTables($tables);
        $this->setColumns($columns);
        $this->setOrderBy('A.date');
        $sql = $this->select($data);
        $rs  = $this->query($sql);
        return $rs;
    }


    public function getLastStatus($itemNumber, $level=NULL, $libraryUnitId=NULL)
    {
        $this->clear();
        $columns = 'A.itemNumber,
                    A.exemplaryStatusId,
                    B.level';

        $tables = '          gtcExemplaryStatusHistory A
                   LEFT JOIN gtcExemplaryStatus B
                          ON (A.exemplaryStatusId = B.exemplaryStatusId)';

        $this->setColumns($columns);
        $this->setTables($tables);

        $this->setWhere('A.itemNumber = ?');
        $args[] = $itemNumber;

        if ($libraryUnitId)
        {
        	$this->setWhere('A.libraryUnitId = ?');
        	$args[] = $libraryUnitId;
        }

        $this->setOrderBy('A.date DESC');
        $sql = $this->select($args);
        $rs  = $this->query($sql, true);

        //Procura o ltimo estado desconsiderando o estado atual
        for ($x=1; $x<sizeof($rs); $x++)
        {
            //Se no for definido nvel retorna o primeiro valor
            if (!$level)
            {
                return $rs[$x]->exemplaryStatusId;
            }
            elseif ($level == $rs[$x]->level)
            {
                return $rs[$x]->exemplaryStatusId;
            }
        }
    }


    /**
     * Enter description here...
     *
     * @param unknown_type $itemNumber
     * @return unknown
     */
    public function getDateOfLastStatus($itemNumber)
    {
        $this->clear();
        $this->setColumns('MAX(date)');
        $this->setTables('gtcExemplaryStatusHistory');
        $this->setWhere('itemNumber = ?');
        $this->setOrderBy('A.date DESC');
        $sql = $this->select(array($itemNumber));
        $rs  = $this->query($sql);
        if(!$rs)
        {
            return false;
        }

        return $rs[0][0];
    }
}
?>
