<?php
/**
 * <--- Copyright 2005-2011 de Solis - Cooperativa de Solues Livres Ltda. e
 * Univates - Centro Universitrio.
 * 
 * Este arquivo  parte do programa Gnuteca.
 * 
 * O Gnuteca  um software livre; voc pode redistribu-lo e/ou modific-lo
 * dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao
 * do Software Livre (FSF); na verso 2 da Licena.
 * 
 * Este programa  distribudo na esperana que possa ser til, mas SEM
 * NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO
 * ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU/GPL em
 * portugus para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo
 * "LICENCA.txt", junto com este programa, se no, acesse o Portal do Software
 * Pblico Brasileiro no endereo www.softwarepublico.gov.br ou escreva para a
 * Fundao do Software Livre (FSF) Inc., 51 Franklin St, Fifth Floor, Boston,
 * MA 02110-1301, USA --->
 * 
 * Handle with files.
 *
 * @author Eduardo Bonfandini [eduardo@solis.coop.br]
 *
 * @version $Id$
 *
 * \b Maintainers \n
 * Eduardo Bonfandini [eduardo@solis.coop.br]
 * Jamiel Spezia [jamiel@solis.coop.br]
 * Moises Heberle [moises@solis.coop.br]
 *
 * @since
 * Class created on 14/10/2010
 *
 **/

/**
 * Class to manipulate the basConfig table
 **/
class BusinessGnuteca3BusFile extends GnutecaBusiness
{
    const DEFAULT_PATH = '/modules/gnuteca3/html/files';
    private static $serverPath;

    public $type; //constantes do type GLOB_, pode ser um array
    public $folder;
    public $fileName;
    public $extension;
    public $files; //so os arquivos upados definidos pelo servidor
    
    function __construct()
    {
        parent::__construct();
        self::defineServerPath();
    }

    /**
     * Verifica se existe a possibilidade de gravao na pasta de arquivos.
     * Utilizado peloe depedencyCheck.
     *
     * @return <boolean>
     */
    public static function isWritable()
    {
        $path = self::getAbsoluteServerPath(true);

        return is_writable($path);
    }

    public static function defineServerPath()
    {
        self::$serverPath = getcwd();
    }

    /**
     * Retorna a pasta do servidor onde os arquivos so guardados.
     * Caminho absolute.
     *
     *
     *
     * @param <type> $complete
     * @return <type>
     */
    public static function getAbsoluteServerPath($complete = false)
    {
        self::defineServerPath();

        if ( $complete)
        {
            $aPath = explode('/', self::$serverPath);
            unset($aPath[count($aPath)-1]);
            return implode('/', $aPath) . self::DEFAULT_PATH;
        }
        else
        {
            return self::$serverPath;
        }
    }

    /**
     * Retorna o caminho absoluto de um arquivo, passando as suas partes
     *
     *
     * @param <string> $folder pasta
     * @param <string> $file nome do arquivo
     * @param <string> $ext extenso
     * @return <string>
     *
     */
    public static function getAbsoluteFilePath($folder, $file, $ext=null)
    {
        #caso exista extenso, inclui o ponto
        if ( $ext )
        {
            $ext = '.' . $ext;
        }

        return self::getAbsoluteServerPath(true).'/'.$folder.'/'.$file.$ext;
    }

    /**
     * Escreve $buffer no $baseFilename e disponibiliza para download em uma nova janela no browser
     *
     * @param (String) $folder
     * @param (String) $filename
     * @param (String) $buffer
     * @param (String) $label
     */
    public static function openDownload( $folder , $baseFilename, $buffer = null , $label = null)
    {
        $baseFilename   = self::getValidFilename( $baseFilename ); #tira espaos, acentos, e limita quantidade de caracteres
        $MIOLO          = MIOLO::getInstance();
        $module         = MIOLO::getCurrentModule();

        if ( $buffer )
        {
            #verifica se o pasta permite escrita
            if ( !is_writable(self::getAbsoluteFilePath($folder)) )
            {
                GnutecaForm::error(_M('Sem permisso para gravar arquivo na pasta', $module).' '.$folder, null, _M('Error', $module), null, true);
                return false;
            }
            
            file_put_contents(self::getAbsoluteFilePath($folder,$baseFilename), $buffer); //escreve os dados no arquivo
        }

        if ( $label )
        {
            $label = "&label=$label";
        }
        
        $MIOLO->page->onload("window.open('file.php?folder={$folder}&file={$baseFilename}{$label}')"); //abre popup de download no browser

        return true;
    }

    /**
     * Retorna um nome de arquivo vlido
     *
     * @param String $title qualquer string
     * @return String $title string com nome do arquivo formatado
     */
    public static function getValidFilename($someString)
    {
        $someString = GnutecaUtils::toASCII($someString); #tira acentos
        $someString = strtolower($someString); #coloca em minuscula
        $someString = str_replace(' ', '_', $someString); #troca espaos por underline
        $someString = substr($someString, 0, 150);  #reduz para 150 caracteres
        return $someString;
    }

    public function listFile()
    {
    }

    public function listFolder($onlyRelative = true)
    {
        $this->type = GLOB_ONLYDIR;
        $result = $this->searchFile();

        if ( is_array($result) && $onlyRelative )
        {
            foreach ( $result as $line => $folder )
            {
                $r[$line] = array($folder[2], $folder[2]); //pega s o caminho relativo
            }

            $result = $r;
        }

        return $result;
    }


    /**
     * Retorna o objeto com as informaes do arquivo
     *
     * @param <type> $filePath caminho relativo
     * @return <stdClass> retorna o objeto com as informaes do arquivo
     *
     */
    public function getFile($relativeFilePath)
    {
        $this->fileName = $relativeFilePath;
        $file = $this->searchFile(true);
        return $file[0];
    }

    /**
     * Verifica se um arquivo existe.
     * Caso passe os 3 parametros otimiza usando diretamente
     * um file_exists, caso contrrio executado busca
     *
     * @param <string> $folder pasta
     * @param <string> $file nome do arquivo
     * @param <string> $ext extensao
     * @return <boolean>
     */
    public function fileExists($folder, $file, $ext=null)
    {
        //caso tenha os 3 otimiza usando diretamente um file_exists
        if ( $folder && $file && $ext )
        {
            return file_exists($this->getAbsoluteFilePath($folder, $file, $ext));
        }
        else //caso contrrio efefua busca
        {
            $this->folder       = $folder;
            $this->fileName     = $file;
            $this->extension    = $ext;
            $result = $this->searchFile(true);

            return $result ? true : false;
        }
    }


    /**
     *
     * GLOB_MARK - Acrescenta uma barra a cada item retornado
     * GLOB_NOSORT - Retorna os arquivos conforme eles aparecem no diretrio (sem ordenao)
     * GLOB_NOCHECK - Retorna o padro da busca se nenhuma combinao de arquivo for encontrada
     * GLOB_NOESCAPE - Barras invertidas no escapam metacaracteres.
     * GLOB_BRACE - Expande {a,b,c} para combinar com 'a', 'b' ou 'c'
     * GLOB_ONLYDIR - Retorna apenas diretrios que combinem com o padro
     * GLOB_ERR - Pra em erros de leitura (como diretrios que no podem ser lidos), por padro os erros so ignorados.
     *
     *
     * @param <type> $toObject
     * @param <type> $orderBy
     * @return <type>
     *
     *
     *
     */
    public function searchFile($toObject = FALSE, $orderBy = NULL)
    {
        $absolutePath = $this->getAbsoluteServerPath(true).'/';

        if ( $this->folder )
        {
            //para no suportar subpastas
            $folder = str_replace('/','',$this->folder). '/';
        }

        if ( $this->fileName )
        {
            $fileName = $this->fileName;
        }

        if ( $this->extension )
        {
            $extension = '.'.$this->extension.'*';
        }

        $searchString = $absolutePath . $folder.$fileName."*".$extension;

        //troca % por * para compatibilidade com outros bus
        $searchString = str_replace('%','*', $searchString);

        //glob  uma funo que busca nas pastas
        $result = glob( $searchString , $this->type );

        //trata dados e busca dados extras 
        foreach ( $result as $line => $file)
        {
            $fileObj = new stdClass();
            $fileObj->absolute  = $file;

            $pathinfo = pathinfo($file);

            $fileObj->dirname       = str_replace ( '/','', str_replace( $this->getAbsoluteServerPath(true), '', $pathinfo['dirname']) );
            $fileObj->basename      = $pathinfo['basename'];
            $fileObj->filename      = $pathinfo['filename'];
            $fileObj->extension     = $pathinfo['extension'];
            $fileObj->size          = filesize($file);
            $fileObj->type          = filetype($file);
            $fileObj->mimeContent   = mime_content_type($file);

            //caso especfico do recibo do gnuteca
            if ( $fileObj->extension == 'rcpt')
            {
                $fileObj->mimeContent = 'application/gnuteca3receipt';
                $fileObj->type = 'text';
            }

            //caso especifico do csv
            if ( $fileObj->extension == 'csv')
            {
                $fileObj->mimeContent = 'text/csv';
                $fileObj->type = 'text';
            }

            //lgica do tipo
            $imageTypes = array('image/png','image/gif', 'image/jpeg', 'image/bmp');

            if ( in_array($fileObj->mimeContent, $imageTypes ) )
            {
                $fileObj->type = 'image';
            }

            $textType = array('text/plain','application/gnuteca3receipt');

            if ( in_array($fileObj->mimeContent, $textType ) )
            {
                $fileObj->type = 'text';
            }

            $mioloFolder = $fileObj->dirname ? 'folder='.$fileObj->dirname : '';

            $fileObj->mioloLink     = $this->MIOLO->getConf('home.url').'/file.php?'.$mioloFolder . '&file=' . $fileObj->basename;

            $fileObj->lastChange    = date("d/m/Y H:i:s", filectime($file) );

            if ( $toObject )
            {
                $result[$line] = $fileObj;
            }
            else
            {
                $result[$line] = array_values( (array) $fileObj );
            }
        }

        return $result;
    }

    public function insertFile()
    {
        if (is_array($this->files) )
        {
            foreach ($this->files as $line => $file )
            {
                if (is_array($file))
                {
                    $targetPath = $this->getAbsoluteServerPath(true).'/'.$this->folder.'/';
                    $targetPath = $targetPath . basename( $file['name']);
                    $targetPath = self::getValidFilename($targetPath); //tira espaos, acentos e limita a quantidade de caracteres

                    $i = 1;
                    while ( file_exists($targetPath) )
                    {
                        $targetPath = str_replace('_'.($i-1),'',$targetPath); //tira o _n anterior
                        $explode    = explode('.',"$targetPath"); //separa nome e extenso
                        $targetPath = $explode[0].'_'.$i.'.'.$explode[1]; //adiciona contador no meuio
                        $i++; //aumenta contador
                    }

                    $ok[] = copy( $file['tmp_name'] , $targetPath );

                    //remove o temporrio
                    unlink($file['tmp_name'] );
                }
                //caso vierem do setData
                else if ( is_object($file))
                {
                    //remove do disco o arquivo definido como remove
                    if ($file->remove)
                    {
                        $this->deleteFile($file->absolute);
                    }
                }
            }
        }

        return in_array(true, $ok);
    }

    public function updateFile()
    {
        $this->insertFile();
    }

    /**
     * Remove a file form disk.
     *
     * @param <string> $completePath
     * @return <boolean>
     */
    public function deleteFile($completePath)
    {
        if ( $completePath && file_exists( $completePath ) )
        {
            return unlink($completePath);
        }
    }
}
?>
