<?php
/**
 * <--- Copyright 2005-2011 de Solis - Cooperativa de Solues Livres Ltda. e
 * Univates - Centro Universitrio.
 * 
 * Este arquivo  parte do programa Gnuteca.
 * 
 * O Gnuteca  um software livre; voc pode redistribu-lo e/ou modific-lo
 * dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao
 * do Software Livre (FSF); na verso 2 da Licena.
 * 
 * Este programa  distribudo na esperana que possa ser til, mas SEM
 * NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO
 * ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU/GPL em
 * portugus para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo
 * "LICENCA.txt", junto com este programa, se no, acesse o Portal do Software
 * Pblico Brasileiro no endereo www.softwarepublico.gov.br ou escreva para a
 * Fundao do Software Livre (FSF) Inc., 51 Franklin St, Fifth Floor, Boston,
 * MA 02110-1301, USA --->
 * 
 * This file handles the connection and actions for Material Gender table
 *
 * @author Jamiel Spezia [jamiel@solis.coop.br]
 *
 * @version $Id$
 *
 * \b Maintainers \n
 * Eduardo Bonfandini [eduardo@solis.coop.br]
 * Jamiel Spezia [jamiel@solis.coop.br]
 * Luiz Gregory Filho [luiz@solis.coop.br]
 * Moises Heberle [moises@solis.coop.br]
 *
 * @since
 * Class created on 25/11/2008
 *
 **/

/**
 * Class to manipulate the gtcMaterial table
 **/
class BusinessGnuteca3BusGenericSearch2 extends GnutecaBusiness
{
    /**
     * Class Attributes
     */

    protected   $controlNumbers             = array(),
                $attributesMaterial         = null,
                $attributesMaterialList     = null,
                $attributesMaterialControl  = null,
                $attributesExemplaryControl = null,
                $extraAttributes            = null,
                $fields                     = null,
                $order                      = null,
                $crossingTerm               = null,
                $addPrefixSuffixInresult    = true,
                $accurateTerm               = false,
                $filterLibraryUnits         = false,
                $debug                      = true;

    public      $busMaterial                = null;


    /**
     * Contantes Locais que definem os nome das tabelas
     */
    const TABLE_EXEMPLARY_CONTROL   = "gtcSearchMaterialView";
    const TABLE_MATERIAL_CONTROL    = "gtcSearchMaterialView";
    const TABLE_MATERIAL            = "gtcMaterial";
    const TABLE_PREFIX_SUFFIX       = "gtcPrefixSuffix";

    /**
     * Class constructor
     **/
    function __construct()
    {
        parent::__construct();

        $this->busMaterial = $this->MIOLO->getBusiness($this->module, 'BusMaterial');

        $this->addPrefixSuffixInresult = MUtil::getBooleanValue( MATERIAL_SEARCH_USE_PREFIX_SUFFIX );
    }


    /**
     * Enter description here...
     *
     */
    public function clean()
    {
        $this->controlNumbers             = array();
        $this->attributesMaterial         = null;
        $this->attributesMaterialList     = null;
        $this->attributesMaterialControl  = null;
        $this->attributesExemplaryControl = null;
        $this->extraAttributes            = null;
        $this->fields                     = null;
        $this->order                      = null;
    }


    /**
     * Adiciona um campo para exibir o resultado
     *
     * @param char(3) $field
     * @param char(1) $subfield
     */
    public function addSearchField( $field, $subfield )
    {
        $obj->field     = $field;
        $obj->subfield  = $subfield;
        $obj->tag       = "{$field}.{$subfield}";

        $this->fields[$obj->tag] = $obj;
    }


    /**
     * Enter description here...
     *
     * @param unknown_type $tag
     */
    public function addSearchTagField( $tag )
    {
        if(!ereg(".", $tag))
        {
            return;
        }

        list($field, $subfield) = explode(".", $tag);

        $this->addSearchField($field, $subfield);
    }


    /**
     * Seta uma condiao para a tabela gtcMaterial;
     *
     * @param simple array char(3) $field
     * @param simple array char(1) $subfield
     * @param simple array varchar $arrayValue
     * @param char(AND, OR) $operator
     * @param char(LIKE, =, >, <) $condition
     */
    public function addMaterialWhere($field, $subfield, $arrayValue, $operator = "AND", $condition = "LIKE")
    {
        //garante integridade da pesquisa
        if ( ! trim($operator) )
        {
            $operator = 'AND';
        }
        if ( ! trim($condition) )
        {
            $condition = 'LIKE';
        }

        $obj->field     = is_array($field)      ? $field        : array($field);
        $obj->subfield  = is_array($subfield)   ? $subfield     : array($subfield);
        $obj->values    = is_array($arrayValue) ? $arrayValue   : array($arrayValue);
        $obj->operator  = $operator;
        $obj->condition = $condition;

        $this->attributesMaterial[] = $obj;
    }


    /**
     * Enter description here...
     *
     * @param simple array or char $tag
     * @param simple array $arrayValue
     * @param char(AND, OR) $operator
     * @param char $condition
     */
    public function addMaterialWhereByTag($tag, $arrayValue, $operator = "AND", $condition = "LIKE")
    {
        if(!strlen($tag) && !is_array($tag))
        {
            $this->addMaterialWhere("", "", $arrayValue, $operator, $condition);
            return;
        }

        //$tag = explode(',', $tag);
        $tag = preg_split('/[,+]/', $tag);
        $arrayTag = is_array($tag) ? $tag : array($tag);

        $f = array();
        $s = array();

        foreach ($arrayTag as $tag)
        {
            if(!ereg(".", $tag))
            {
                continue;
            }

            list($field, $subfield) = explode(".", $tag);

            $f[] = $field;
            $s[] = $subfield;
        }

        $this->addMaterialWhere($f, $s, $arrayValue, $operator, $condition);
    }


    /**
     * Seta uma condiao para a tabela Material Control
     *
     * @param varchar $fieldName
     * @param simple array $arrayValue
     * @param char(AND, OR) $operator
     * @param char(like, =, >, <) $condition
     */
    public function addMaterialControlWhere($fieldName, $arrayValue, $operator = "AND", $condition = "=")
    {
        $obj->fieldName = is_array($fieldName)  ? $fieldName  : array($fieldName);
        $obj->values    = is_array($arrayValue) ? $arrayValue : array($arrayValue);
        $obj->operator  = $operator;
        $obj->condition = $condition;

        $this->attributesExemplaryControl[] = $obj;
    }


     /**
     * Seta uma condiao para a tabela Exemplary Control
     *
     * @param varchar $fieldName
     * @param simple array $arrayValue
     * @param char(AND, OR) $operator
     * @param char(like, =, >, <) $condition
     */
    public function addExemplaryControlWhere($fieldName, $arrayValue, $operator = 'AND', $condition = '=')
    {
    	$obj            = new StdClass();
        $obj->fieldName = is_array($fieldName)  ? $fieldName    : array($fieldName);
        $obj->values    = is_array($arrayValue) ? $arrayValue   : array($arrayValue);
        $obj->operator  = $operator;
        $obj->condition = $condition;

        foreach ($obj->fieldName as $index => $fieldName)
        {
            $obj->fieldName[$index] = "exemplary{$fieldName}";
        }

        $this->attributesExemplaryControl[] = $obj;
    }



    /**
     * Adiciona control number na lista.
     * a pesquisa retorna o conteudo os numeros de controle existentes nesta lista.
     *
     * @param integer $controlNumber
     */
    public function addControlNumber($controlNumber)
    {
        $this->controlNumbers[] = $controlNumber;
    }



    /**
     * Adicionar condies de pesquisa por um expresso
     *
     * @param String $expression
     */
    public function addMaterialWhereByExpression( $expression )
    {
    	//tira os espaos sobresalentes no inicio e no fim da expresso.
    	$expression = trim( $expression ); //o trim  importante aqui na classe, pois pode evitar um monte de erros na pesquisa.
        // Filtra as tags e contedos - "([0-9]{3}\.[0-9a-zA-Z],?)*:"
        $searchWere = preg_split('/(([0-9]{3}\.[0-9a-zA-Z][,+]?\|?)*:| AND | OR | NOT )/', $expression, -1, PREG_SPLIT_DELIM_CAPTURE);

        $specialCondition = null;
        $specialTagSeparators = " .-";

        for ($x=0; $x<count($searchWere); $x++ )
        {
        	 $sW = $searchWere[$x];

            //Verifica se  uma tag
            if (ereg('([0-9]{3}\.[0-9a-zA-Z],?\|?)*:', $sW))
        	{
                $condition->tag = trim(trim($sW, ':'));
                $x += 1;
        	}
        	// VERIFICA SE A UMA REGRA ESPECIAL DE SEPARADORES ENTRE DUAS OU MAIS TAGS
        	elseif(ereg("\([0-9A-Za-z.|$specialTagSeparators]{1,}\)", $sW)) // \(([0-9A-Za-z.]{5})(([|]{1}).*([|]{1}))?\)
        	{
                $condition->tag = trim(trim($sW, ':'));
                $condition->tag = preg_replace("/,?\([0-9A-Za-z.|$specialTagSeparators]{1,}\),?/", '', $sW);
                $specialCondition->tags = str_replace(array($condition->tag, ','), '', $sW);
                $x += 1;
        	}
        	//Verifica se  um operador lgico
        	elseif (ereg(' AND | OR | NOT ', $sW))
        	{
                $condition->operator = trim($sW);
        	}
        	else
            {
                if ($sW)
                {
                    $condition->content = trim($sW);

                    if($specialCondition->tags)
                    {
                        $specialCondition->content = $condition->content;
                    }

                    if (!$condition->operator)
                    {
                        $condition->operator = 'AND';
                    }

                    $this->addMaterialWhereByTag($condition->tag, $condition->content, $condition->operator);
                    unset($condition);
                }
            }
        }

        if($specialCondition)
        {
            $tags = preg_match_all('/[0-9A-Za-z.]{5}/', $specialCondition->tags, $tagMatch);
            $sepa = preg_match_all("/\|[$specialTagSeparators]{1}\|/", $specialCondition->tags, $sepMatch);

            $separadores = null;
            foreach ($sepMatch[0] as $content)
            {
                $separadores[] = str_replace('|', '', $content);
            }

            $sep = implode(null, $separadores);
            $conteudos = preg_split("/[$sep]/", $specialCondition->content);
            $tag = $tagMatch[0];

            foreach($conteudos as $index => $value)
            {
                $this->addMaterialWhereByTag($tag[$index], $value, ($index > 0 ? 'AND' : 'OR'));
            }
        }
    }


    /**
     * Define the order of the search
     *
     * @param integer $field
     * @param integer  $subfield
     * @param php constant $type - SORT_ASC, SORT_DESC.
     * @param php constant $fieldType - SORT_NUMERIC, SORT_STRING.
     */
    public function setOrder( $field, $subfield, $type = SORT_ASC, $fieldType = SORT_NUMERIC )
    {
        $obj->field     = $field;
        $obj->subfield  = $subfield;
        $obj->type      = (int) $type;
        $obj->fieldType = (int) $fieldType;

        $this->order["{$field}.{$subfield}"] = $obj;
    }


    /**
     * Limpa a ordem
     *
     */
    public function cleanOrder()
    {
        $this->order = null;
    }


    /**
     * Este metodo permite filtar a consulta por diversos tipos diferentes de material,
     * definindo a planilha e o level desejado.
     *
     * @param simple array Object $arrayObject
     * @param boolean $ignore; define se  para ignorar ou nao as categorias informadas.
     */
    public function setCategoryLevel($arrayObject, $ignore = false)
    {
        $arrayObject = is_array($arrayObject) ? $arrayObject : array($arrayObject);

        $sql = "";
        foreach($arrayObject as $obj)
        {
            $condition  = $ignore ? " != " : " = ";
            $category   = strlen($obj->category)    ? " ". self::TABLE_EXEMPLARY_CONTROL .".category {$condition} '{$obj->category}' " : "";
            $level      = strlen($obj->level)       ? " ". self::TABLE_EXEMPLARY_CONTROL .".level    {$condition} '{$obj->level}' "    : "";
            $union      = strlen($category) && strlen($level) ? " AND " : "";
            $operator   = $ignore ? " AND " : " OR ";

            $sql.= "( {$category} {$union} {$level} )  {$operator}";
        }
        $sql = substr($sql, 0, strlen($sql)-4);
        $sql = " AND ({$sql})";

        $this->extraAttributes[self::TABLE_MATERIAL_CONTROL]->categoryLevel = $sql;
    }



    /**
     * Este metodo permite definir um filtro pelo tipo de material.
     *
     * @param simple array integer $arrayMaterialTypeId
     * @param boolean $ignore - define se  para ignorar ou nao os ids do array
     */
    public function setMaterialTypeId($arrayMaterialTypeId, $ignore = false)
    {
        $arrayMaterialTypeId    = is_array($arrayMaterialTypeId) ? $arrayMaterialTypeId : array($arrayMaterialTypeId);
        $ids                    = implode(", ", $arrayMaterialTypeId);
        $operator               = $ignore ? " NOT IN " : " IN ";

        // filtro apenas por obra
        if(MATERIAL_TYPE_CONTROL == 1)
        {
            // SETA TIPO DE MATERIAL
            $this->addMaterialControlWhere("materialtypeid", $arrayMaterialTypeId, "AND", trim($operator));
            return;
        }
        //filtro por obra e exemplar
        elseif(MATERIAL_TYPE_CONTROL != 2)
        {
            return ;
        }

        // SETA TIPO DE MATERIAL
        //$this->addMaterialControlWhere("materialtypeid", $arrayMaterialTypeId, "AND", trim($operator));

        $materialFilter     = $this->makeSqlToMaterialTypeConstant(MATERIAL_TYPE_FORCE_BY_MATERIAL, $ignore);
        $exemplaryFilter    = $this->makeSqlToMaterialTypeConstant(MATERIAL_TYPE_FORCE_BY_EXEMPLARY, $ignore);

        $sqlFinal          .= strlen($materialFilter)   ? "($materialFilter  AND ". self::TABLE_MATERIAL_CONTROL   .".materialtypeid            $operator ($ids) )    ". ($ignore ? " AND " : " OR ") : "";
        $sqlFinal          .= strlen($exemplaryFilter)  ? "($exemplaryFilter AND ". self::TABLE_MATERIAL_CONTROL   .".exemplarymaterialtypeid   $operator ($ids))   ". ($ignore ? " AND " : " OR ") : "";
        $sqlFinal           = substr($sqlFinal, 0, -4);

        $this->extraAttributes[self::TABLE_EXEMPLARY_CONTROL]->materialType = " AND ($sqlFinal)";
    }



    /**
     * Este metodo foi criado para transformar duas constantes em SQL para a funo que filtra tipo de material.
     *
     * @param unknown_type $constant
     * @param unknown_type $ignore
     * @return unknown
     */
    private function makeSqlToMaterialTypeConstant($constant, $ignore = false)
    {
        $materialFilter     = explode("\n", $constant);
        $operator           = $ignore ? " NOT IN " : " IN ";

        if(!is_array($materialFilter) || !count($materialFilter))
        {
            return "";
        }

        $materialCategory       = " ". self::TABLE_MATERIAL_CONTROL   .".category   $operator (<--MATERIAL_CATEGORY_IN_LOCAL-->) ";
        $materialLevel          = " ". self::TABLE_MATERIAL_CONTROL   .".level      $operator (<--MATERIAL_LEVEL_IN_LOCAL-->) ";
        $categories             = null;
        $levels                 = "";

        foreach ($materialFilter as $v)
        {
            list($category, $level) = explode(",", $v);

            if(!strlen($level))
            {
                $categories[] = $category;
                continue;
            }

            $levels.= " ( ";
            $levels.= str_replace("<--MATERIAL_CATEGORY_IN_LOCAL-->", "'$category'", $materialCategory);
            $levels.= " AND ";
            $levelX = null;

            for($x = 0; $x < strlen($level); $x++)
            {
                $levelX[] = $level[$x];
            }

            $levels.= str_replace("<--MATERIAL_LEVEL_IN_LOCAL-->", "'". implode("','", $levelX) ."'", $materialLevel);
            $levels.= ")". ($ignore ? " AND " : " OR ");
        }

        $sqlMaterial = $categories ? str_replace("<--MATERIAL_CATEGORY_IN_LOCAL-->", "'". implode("','", $categories) ."'", $materialCategory) . ($ignore ? " AND " : " OR ") : "";
        $sqlMaterial = (strlen($levels)) ? $sqlMaterial . $levels : $sqlMaterial;
        $sqlMaterial = substr($sqlMaterial, 0, -4);
        $sqlMaterial = "($sqlMaterial)";

        return $sqlMaterial;
    }





    /**
     * retorna o Sql Where para a tabela gtcMaterial
     *
     * @return string or null
     */
    protected function getMaterialWhere()
    {
        if(is_null($this->attributesMaterial))
        {
            return null;
        }

        $tableName      = self::TABLE_MATERIAL;
        $controlNumber  = " {$tableName}.controlNumber ";
        $subSql         = " SELECT DISTINCT {$controlNumber} FROM {$tableName} <HINNER_JOIN_MATEIRAL_CONTROL> <HINNER_JOIN_EXEMPLARY_CONTROL> WHERE ";
        $sqlWhere       = null;

        foreach ($this->attributesMaterial as $index => $obj)
        {
            $condition = $this->workingConditionMaterial($obj);

            switch ( trim( $obj->operator ) )
            {
                case 'OR':
                    //O operador or, como ele adiciona mais registros, no tem necessidade de execut-lo com subsql
                    $sqlWhere .= "OR  <CONDITION>";
                break;

                case 'AND':
                	if ($index == 0)
                	{
                	    //Se o termo for o primeiro, no gera o subsql por questes de desempenho
                	    $sqlWhere .= "AND <CONDITION> ";
                	    break;
                	}

                	$sqlWhere .= "AND {$controlNumber} IN ( <SUBSQL><CONDITION> <CONDITION_MATEIRAL_CONTROL> <CONDITION_EXEMPLARY_CONTROL>)";

                break;

                case 'NOT':
                    $sqlWhere .= "AND {$controlNumber} NOT IN ( <SUBSQL><CONDITION> <CONDITION_MATEIRAL_CONTROL> <CONDITION_EXEMPLARY_CONTROL>)";
                break;
            }

            $priority .= '(';
            $sqlWhere .= ')';

            $sqlWhere = str_replace(array('<SUBSQL>', '<CONDITION>'),array($subSql, $condition),$sqlWhere);
        }

        $sqlWhere = substr($sqlWhere, 4, strlen($sqlWhere));
        $sqlWhere = $priority . $sqlWhere;

        return $sqlWhere;
    }


    /**
     * Enter description here...
     *
     * @param object fields $obj
     * @return string sql where
     */
    public function workingConditionMaterial($obj)
    {
        $conditionField     = "";
        $conditionValues    = "";

        // TRABALHA AS TAGS DE PESQUISA
        if(count($obj->subfield) && count($obj->field))
        {
            $fields = array();
            foreach ($obj->field as $index => $f)
            {
                if(!strlen($f) || !strlen($obj->subfield[$index]))
                {
                    $f = "*";
                    $obj->subfield[$index] = "*";
                }

                $obj->tags["{$f}.{$obj->subfield[$index]}"] = true;
            }
        }

        // PREPARA OS VALORES PARA SEREM PESQUISADOS
        $valores = array();
        foreach ($obj->tags as $tag => $f)
        {
            foreach ($obj->values as $v)
            {
                $complement = null;

                // SE FOR CLASSIFICAO... TENTA ADICIONAR O CUTTER
                if(ereg($tag, MARC_CLASSIFICATION_TAG))
                {
                    $v = str_replace("  ", " ", $v);
                    list($v, $complement) = explode(" ", $v);
                }

                $str = $this->busMaterial->prepareSearchContent($tag, $v, $complement);
                $valores[$tag][] = str_replace("'","''", $str); //troca ' por '' para funcionar no postgres pesquisa com '
            }
        }

        $condition = "";

        //TRABALHA A CONDIO DE VALORES
        switch(strtoupper($obj->condition))
        {
            // TRABALHA WHERE CONDITION PARA START END
            case 'START':
            case 'END':

                foreach($valores as $tag => $conteudoS)
                {
                    list($field, $subField) = explode(".", $tag);
                    $condition.= " ( ";
                    if("{$field}.{$subField}" != "*.*")
                    {
                        $condition.=  self::TABLE_MATERIAL  .".fieldid = '{$field}' AND  ". self::TABLE_MATERIAL  .".subfieldid = '{$subField}' AND ";
                    }
                    $condition.= " ( ";

                    foreach ($conteudoS as $conteudo)
                    {
                        $conteudo = $this->busMaterial->prepareSearchContent($tag, $conteudo);
                        $conteudo = $obj->condition == 'START' ? "{$conteudo}%" : "%{$conteudo}";

                        $condition.= " ". self::TABLE_MATERIAL  .".searchContent LIKE '{$conteudo}' OR ";
                    }
                    $condition = substr($condition, 0, -3);
                    $condition.= " )) OR ";
                }

                $condition = substr($condition, 0, -4);
                $condition = "( $condition )";

                return $condition;



            // TRABALHA WHERE CONDITION PARA IN
            case 'IN'       :
            case 'NOT IN'  :


                foreach($valores as $tag => $conteudoS)
                {
                    list($field, $subField) = explode(".", $tag);
                    $condition.= " ( ";
                    //. self::TABLE_MATERIAL  .".fieldid = '{$field}' AND  ". self::TABLE_MATERIAL  .".subfieldid = '{$subField}' AND ";
                    if("{$field}.{$subField}" != "*.*")
                    {
                        $condition.=  self::TABLE_MATERIAL  .".fieldid = '{$field}' AND  ". self::TABLE_MATERIAL  .".subfieldid = '{$subField}' AND ";
                    }

                    $inContent = implode("', '", $conteudoS);

                    $condition.= " ". self::TABLE_MATERIAL  .".searchContent IN ('{$inContent}') ";
                    $condition.= " ) ";
                    $condition.= (strtoupper($obj->condition) == "IN") ? " OR " : " AND ";
                }

                $condition = substr($condition, 0, -4);
                $condition = "( $condition )";

                return $condition;


            // TRABALHA WHERE CONDITION PARA BETWEEN
            case "BETWEEN"  :
            case "NUMERIC BETWEEN"  :

                foreach($valores as $tag => $conteudoS)
                {
                    if(count($conteudoS) != 2)
                    {
                       //continue;
                    }

                    $condition.= " ( ";

                    if($tag != "*.*")
                    {
                        list($field, $subField) = explode(".", $tag);
                        $condition.=  self::TABLE_MATERIAL  .".fieldid = '{$field}' AND  ". self::TABLE_MATERIAL  .".subfieldid = '{$subField}' AND (";
                    }

                    $searchContent = self::TABLE_MATERIAL  .".searchContent";
                    $conteudo1      = "'{$conteudoS[0]}'";
                    $conteudo2      = "'{$conteudoS[1]}'";

                    if(strtoupper($obj->condition) == "NUMERIC BETWEEN")
                    {
                        $searchContent1  = "getSearchContentToYearCompare(". self::TABLE_MATERIAL  .".searchContent, FALSE)::integer";
                        $searchContent2  = "getSearchContentToYearCompare(". self::TABLE_MATERIAL  .".searchContent, TRUE)::integer";
                        $conteudo1      = "getSearchContentToYearCompare('{$conteudoS[0]}', TRUE)::integer";
                        $conteudo2      = "getSearchContentToYearCompare('{$conteudoS[1]}', TRUE)::integer";
                    }

                    switch ($tag)
                    {
                        // CONDIO ESPECIAL PARA 260.c = data de publicao
                        case MARC_PUBLICATION_DATE_TAG :
                            $condition.= "CASE WHEN char_length(getSearchContentToYearCompare(". self::TABLE_MATERIAL  .".searchContent, FALSE)) >= 1 THEN ";
                            $condition.= "(";
                            $condition.= "      ($searchContent1 >= $conteudo1 ";
                            $condition.= "          AND ";
                            $condition.= "       $searchContent2 <= $conteudo2) ";
                            $condition.= "  OR  compareYearPeriod(". self::TABLE_MATERIAL  .".searchContent, getSearchContentToYearCompare('{$conteudoS[0]}', FALSE), getSearchContentToYearCompare('{$conteudoS[1]}', TRUE)) ";
                            $condition.= ") ELSE FALSE END ";
                            break;

                        default:
                            $condition.= " ($searchContent >= $conteudo1 AND $searchContent <= $conteudo2 )";
                            break;
                    }

                    $condition.= ")) OR ";
                }

                $condition = substr($condition, 0, -4);
                $condition = "( $condition )";

                return $condition;


            // TRABALHA WHERE CONDITION PARA LIKE
            case 'ILIKE'     :
            case 'LIKE'      :
            case 'NOT LIKE'  :
            case 'NOT ILIKE' :

                $obj->condition = str_replace("ILIKE", "LIKE", $obj->condition);

                foreach($valores as $tag => $conteudoS)
                {
                    $condition.= " ( ";
                    list($field, $subField) = explode(".", $tag);
                    if("{$field}.{$subField}" != "*.*")
                    {
                        $condition.=  self::TABLE_MATERIAL  .".fieldid = '{$field}' AND  ". self::TABLE_MATERIAL  .".subfieldid = '{$subField}' AND ";
                    }
                    $condition.= " ( ";

                    foreach ($conteudoS as $conteudo)
                    {
                        if($this->crossingTerm === true)
                    	{
                            $condition.= $this->makeCrossingTerms($conteudo, $tag, $obj->condition);
                            $condition.= " OR ";
                    	}
                        else
                    	{
                    	    if(!$this->accurateTerm && !ereg("%", $conteudo))
                        	{
                                $conteudo = str_replace(" ", "%", $conteudo);
                                $conteudo = "%{$conteudo}%";
                        	}

                            $condition.= " ". self::TABLE_MATERIAL  .".searchContent {$obj->condition} '{$conteudo}' OR ";

                    	}
                    }

                    $condition = substr($condition, 0, -3);
                    $condition.= " )) ";
                    $condition.= ($obj->condition == "LIKE") ? " OR " : " AND ";
                }

                $condition = substr($condition, 0, -4);
                $condition = "( $condition )";

                return $condition;



            default:

                foreach ($obj->field as $i => $fi)
                {
                    $tag        = "{$fi}.{$obj->subfield[$i]}";
                    $field      = $fi;
                    $subField   = $obj->subfield[$i];

                    $conditionValues.= "(".self::TABLE_MATERIAL  .".fieldid = '{$field}' AND  ". self::TABLE_MATERIAL  .".subfieldid = '{$subField}' AND (";
                    foreach ($obj->values as $v)
                    {
                        $v = $this->busMaterial->prepareSearchContent($tag, $v);
                        $v = str_replace("'","''", $v); //troca ' por '' para funcionar no postgres
                        $conditionValues.= " (". self::TABLE_MATERIAL  .".searchContent {$obj->condition} '{$v}') OR ";
                    }

                    $conditionValues = substr($conditionValues, 0, -3);
                    $conditionValues.= ")) OR ";
                }

                $conditionValues = substr($conditionValues, 0, -3);
                return "( $conditionValues )";
        }

    }



    /**
     * Enter description here...
     *
     * @param unknown_type $value
     * @param unknown_type $tag
     * @param unknown_type $condition
     */
    private function makeCrossingTerms($values, $tag, $condition)
    {
        $values             = explode(" ", $values);
        $conditionValues    = "";

        foreach ($values as $v)
        {
            $v = $this->busMaterial->prepareSearchContent($tag, $v);
            $v = "%{$v}%";
            $conditionValues.= " ". self::TABLE_MATERIAL  .".searchContent {$condition} '{$v}' AND ";
        }

        $conditionValues = substr($conditionValues, 0, -4);
        return $conditionValues;
    }


    /**
     * Enter description here...
     *
     * @return string
     */
    protected function getExemplaryControlWhere($temMaterialControl = false)
    {
        if(is_null($this->attributesExemplaryControl))
        {
            return null;
        }

        $sqlModel = '<operator>   ( <condition> <continue> )';
        $sqlWhere = null;

        foreach ($this->attributesExemplaryControl as $obj)
        {
            // Trabalha o bloco de condio para consulta
            $condition = $this->workingConditionExemplaryControl($obj);

            //substitui o operador e a condiao no modelo
            $op = ($obj->operator == "NOT") ? "AND" : $obj->operator;
            $where = str_replace(array('<operator>', '<condition>'), array($op, $condition), $sqlModel);

            // incrementa o sqlWhere
            $sqlWhere = is_null($sqlWhere) ? $where : str_replace('<continue>', $where, $sqlWhere);
        }

        // remove o conteudo indevido
        $sqlWhere = str_replace('<continue>', '<extra_sql>', $sqlWhere);

        if(!$temMaterialControl)
        {
            $sqlWhere = substr($sqlWhere, 4, strlen($sqlWhere));
        }

        return $sqlWhere;
    }



    /**
     * Enter description here...
     *
     * @param object fields $obj
     * @return string sql where
     */
    public function workingConditionExemplaryControl($obj)
    {
        $fields     = $obj->fieldName;
        $values     = $obj->values;


        // CONDIO ESPEXIAL PARA FILTRO DE PLANILHAS
        if($fields[0] == 'categoryLevel')
        {
            $v = $values[0];
            list($cat, $lev) = explode(",", $v);

            $operator = ((trim($obj->condition) == "=") ? 'AND' : 'OR');

            return " ( ". self::TABLE_EXEMPLARY_CONTROL .".category {$obj->condition} '{$cat}' {$operator} ". self::TABLE_EXEMPLARY_CONTROL .".level {$obj->condition} '{$lev}' ) ";
        }


        $conditionFields  = "";

        foreach ($fields as $index => $f)
        {
            $parcialCondition = "";

            switch(strtoupper($obj->condition))
            {
                case "IN" :
                    $v = implode(", ", $values);
                    $parcialCondition.= " ". self::TABLE_EXEMPLARY_CONTROL    .".{$f}  IN ($v) ";
                break;

                case "NOT":
                case "NOT IN":
                    $v = implode(", ", $values);
                    $parcialCondition.= " ". self::TABLE_EXEMPLARY_CONTROL    .".{$f}  NOT IN ($v) ";
                break;

                case "BETWEEN"  :
                    if(count($values) == 2 && strlen($values[0]) && strlen($values[1]))
                    {
                        $parcialCondition.= " (";
                        $parcialCondition.= "    ". self::TABLE_EXEMPLARY_CONTROL  .".{$f} >= '{$values[0]}' ";
                        $parcialCondition.= "    AND ";
                        $parcialCondition.= "    ". self::TABLE_EXEMPLARY_CONTROL  .".{$f} <= '{$values[1]}' ";
                        $parcialCondition.= " )";
                    }
                break;

                default:
                    foreach ($values as $v)
                    {
                        $parcialCondition.= " ". self::TABLE_EXEMPLARY_CONTROL    .".{$f} {$obj->condition} '$v' OR ";
                    }
                    $parcialCondition = substr($parcialCondition, 0, strlen($parcialCondition)-3);
                break;
            }

            $conditionFields.= " ($parcialCondition) OR ";
        }
        $conditionFields = substr($conditionFields, 0, strlen($conditionFields) -3);

        return "($conditionFields)";
    }


    /**
     * Executa o sql e seta os numeros de controle.
     *
     */
    public function executeSearch( $limit = null )
    {
        if(!$this->attributesMaterial && !$this->attributesMaterialControl && !$this->attributesExemplaryControl && !$this->attributesMaterialList && !$this->extraAttributes)
        {
            return;
        }

        $sqlSelect   = "SELECT DISTINCT <fistTableName>.controlNumber ";
        $sqlFrom     = null;
        $sqlWhere    = null;

        $materialWhere          = $this->getMaterialWhere();
        //$materialControlWhere   = $this->getMaterialControlWhere($materialWhere);
        $exemplaryControlWhere  = $this->getExemplaryControlWhere($materialWhere || $materialControlWhere);

        if($materialWhere)
        {
            $sqlSelect  = str_replace("<fistTableName>", self::TABLE_MATERIAL, $sqlSelect);
            $sqlFrom    = " FROM ". self::TABLE_MATERIAL ." ";
            $sqlWhere   = " WHERE $materialWhere";
        }

        // ADICIONA JOIN COM GTCEXEMPLARYCONTROL
        if($exemplaryControlWhere || isset($this->extraAttributes[self::TABLE_EXEMPLARY_CONTROL]) || isset($this->extraAttributes["all"]))
        {
            if(!strlen($exemplaryControlWhere))
            {
                $exemplaryControlWhere = "<extra_sql>";
            }

            $sqlSelect  = is_null($sqlFrom) ? str_replace("<fistTableName>", self::TABLE_EXEMPLARY_CONTROL  , $sqlSelect) : $sqlSelect;
            $sqlFrom    = !($materialWhere) && !($materialControlWhere) ? " FROM ". self::TABLE_EXEMPLARY_CONTROL   ." " : " $sqlFrom INNER JOIN ". self::TABLE_EXEMPLARY_CONTROL   ." USING (controlNumber) ";
            $sqlWhere   = !($materialWhere) && !($materialControlWhere) ? " WHERE $exemplaryControlWhere " : " $sqlWhere $exemplaryControlWhere ";

            // ADICIONA AS CONDIES ESPECIAIS
            if(isset($this->extraAttributes[self::TABLE_EXEMPLARY_CONTROL]))
            {
                foreach ($this->extraAttributes[self::TABLE_EXEMPLARY_CONTROL] as $sql)
                {
                    $sqlWhere = str_replace('<extra_sql>', " $sql <extra_sql>", $sqlWhere);
                }
            }

            $sqlWhere   = str_replace(array("<HINNER_JOIN_EXEMPLARY_CONTROL>", "<CONDITION_EXEMPLARY_CONTROL>", '<extra_sql>'), array(" INNER JOIN ". self::TABLE_EXEMPLARY_CONTROL  ." USING (controlNumber) ", $exemplaryControlWhere, ""), $sqlWhere);
        }

        // Adiciona condio extra de ambas as tabelas.
        if(isset($this->extraAttributes["all"]))
        {
            $sqlWhere.= " AND (<extra_sql>) ";

            foreach ($this->extraAttributes["all"] as $sql)
            {
                $sqlWhere = str_replace('<extra_sql>', " $sql <extra_sql>", $sqlWhere);
            }
        }


        // MONTA O SQL FINAL
        $sqlFinal = "$sqlSelect $sqlFrom $sqlWhere";

        // FAZ UM REPLACE DA TAGS QUE SOBRARAM
        $sqlFinal = str_replace(array("<HINNER_JOIN_MATEIRAL_CONTROL>", "<HINNER_JOIN_EXEMPLARY_CONTROL>", "<CONDITION_MATEIRAL_CONTROL>", "<CONDITION_EXEMPLARY_CONTROL>", "<extra_sql>"), "", $sqlFinal);

        if ( $limit )
        {
        	$sqlFinal .= "LIMIT $limit";
        }

        $this->startTime = $this->currentTime();

        $result = parent::query($sqlFinal);

        if ($result)
        {
            foreach ($result as $r)
            {
                $this->addControlNumber($r[0]);
            }
        }

        return $result;
    }



    /**
     * Enter description here...
     *
     * @return unknown
     */
    public function getWorkSearch()
    {
        if ( !is_array( $this->controlNumbers ) || ! count( $this->controlNumbers ))
        {
            return false;
        }

        // remove valores duplicados
        $this->controlNumbers = array_unique($this->controlNumbers);
        $controlNumber = implode(',', $this->controlNumbers);

        $select = "SELECT   TABLE_MATERIAL.controlNumber,
                            TABLE_MATERIAL.fieldId,
                            TABLE_MATERIAL.subfieldId,
                            <--CONTENT-->,
                            TABLE_MATERIAL.line,
                            TABLE_MATERIAL.searchcontent,
                            TABLE_MATERIAL.indicator1,
                            TABLE_MATERIAL.indicator2,
                            TABLE_MATERIAL.prefixId,
                            TABLE_MATERIAL.suffixId
                    FROM    TABLE_MATERIAL
                   WHERE    TABLE_MATERIAL.controlNumber in ($controlNumber)
                ORDER BY    TABLE_MATERIAL.controlNumber,
                            TABLE_MATERIAL.line";

        //adiciona o prefixo e sufixo caso estado definido que assim deve ser
        if ( $this->addPrefixSuffixInresult )
        {
            $args = "(CASE WHEN TABLE_MATERIAL.prefixId IS NOT NULL THEN
                                    (SELECT TABLE_PREFIX_SUFFIX.content FROM TABLE_PREFIX_SUFFIX WHERE TABLE_MATERIAL.prefixId = TABLE_PREFIX_SUFFIX.prefixsuffixid )
                                ELSE '' END)
                                || TABLE_MATERIAL.content  ||
                               (CASE WHEN TABLE_MATERIAL.suffixId IS NOT NULL THEN
                                    (SELECT TABLE_PREFIX_SUFFIX.content FROM TABLE_PREFIX_SUFFIX WHERE TABLE_MATERIAL.suffixId = TABLE_PREFIX_SUFFIX.prefixsuffixid )
                                ELSE '' END) as content";
        }
        else
        {
            $args = 'TABLE_MATERIAL.content';
        }

        $select = str_replace('<--CONTENT-->', $args, $select);
        $select = str_replace('TABLE_MATERIAL',self::TABLE_MATERIAL, $select);
        $select = str_replace('TABLE_PREFIX_SUFFIX', self::TABLE_PREFIX_SUFFIX, $select);

        parent::clear();
        //Seta as colunas para retorno em objeto
        parent::setColumns("controlNumber,fieldId,subFieldId,content,line,searchContent,indicator1,indicator2,prefixId,suffixId");

        $this->result = parent::query($select, true);

        if (!$this->result)
        {
            return false;
        }

        $data = $this->result;

        // FORMATA O CONTEUDO
        $data = $this->formatData($data);
        // APLICAR ORDENAO
        $data = $this->applyOrder($data);
        // FILTRA OS CAMPOS
        $data = $this->filterFields($data);

        return $data;
    }


    /**
     * este metodo aplica a ordenao no array
     *
     * @param array $data
     */
    private function applyOrder($data)
    {
        if(is_null($this->order))
        {
            return $data;
        }

        $i = $this->currentTime();

        $orderType      = null;
        $fieldType      = null;
        $order          = array();
        $orderFinal     = array();

        //Percorre todos os registros encontrados
        foreach ($data as $controlNumber => $contentData)
        {
            //Monta um array de ordenacao
            foreach ($this->order as $tag => $contentOrder)
            {
                $searchContent = $contentData[$tag][0]->searchContent;
                if(!strlen($searchContent))
                {
                    //Verifica caso nao encontre conteudo com a primeira tag do order, tenta a segunda at encontrar alguma
                    continue;
                }

                switch ($tag)
                {
                    // ISSO SERVE PARA DESPREZAR CARACTERES NO TTULO
                    case MARC_SECUNDARY_TITLE_TAG:
                    case MARC_TITLE_TAG:
                        $indicator2 = strlen($contentData[$tag][0]->indicator2) ? $contentData[$tag][0]->indicator2 : 0;
                        $searchContent = substr($searchContent, $contentData[$tag][0]->indicator2, strlen($searchContent));
                        break;
                }

                $order[$tag]["$controlNumber"] = $searchContent;
                
                //Se encontrou a tag, vai para o proximo registro (de fora)
                continue 2;
            }

            //Caso nao encontre, coloca "zzzzzz" para ficar no final
            $order[$tag]["$controlNumber"] = "zzzzzzzzzz";
        }

        foreach ($this->order as $tag => $contentOrder)
        {
            if(!count($order[$tag]))
            {
                continue;
            }
            if(is_null($orderType))
            {
                 $orderType = $contentOrder->type;
            }
            if(is_null($fieldType))
            {
                $fieldType = $contentOrder->fieldType;
            }

            if($contentOrder->type == SORT_ASC)
            {
                asort($order[$tag], $contentOrder->fieldType);
            }
            else
            {
                arsort($order[$tag], $contentOrder->fieldType);
            }

            $orderFinal = $orderFinal + $order[$tag];
        }
        
        $tempData = null;

        foreach ($orderFinal as $controlNumber => $content)
        {
            $tempData[$controlNumber] = $data[$controlNumber];
        }

        $total = $this->Total_Time($i, $this->currentTime());

        return $tempData;
    }


    /**
     * Este mnetodo formata o conteudo da pesquisa para o formato utilizado pelo usuario
     *
     * @param original array search $originalData
     * @return new format array
     */
    private function formatData($originalData)
    {
        $i = $this->currentTime();

        foreach ($originalData as $res)
        {
            $tag = "{$res->fieldId}.{$res->subFieldId}";

            //Grava o nmero de controle do registro
            $data[$res->controlNumber]['CONTROLNUMBER'] = $res->controlNumber;
            $data[$res->controlNumber][0]               = $res->controlNumber; //usado para conseguir pegar o control number na grid

        	//Dados - [sequencia][tag][line]
            $data[$res->controlNumber][$tag][$res->line] = $res;
        }

        $total = $this->Total_Time($i, $this->currentTime());

        return $data;
    }


    /**
     * Este mtodo serve para filtrar os campos que foram solicitados pela pesquisa
     *
     * @param array data search $data
     * @return  array data search filtered
     */
    private function filterFields($data)
    {
        if (!$this->fields)
        {
            return $data;
        }

        $i = $this->currentTime();

        $dataOk = array();
        $x = 0;

        foreach ($data as $key => $dt)
        {
            $dataOK[$x][0]               = $dt['CONTROLNUMBER']; //usado para conseguir pegar o control number na grid
        	$dataOk[$x]['ORDER']         = $dt['ORDER'];
        	$dataOk[$x]['CONTROLNUMBER'] = $dt['CONTROLNUMBER'];
            foreach ($this->fields as $field)
            {
                $dataOk[$x][$field->field . '.' . $field->subfield] = $dt[$field->field . '.' . $field->subfield];
            }

            $x++;
        }

        $total = $this->Total_Time($i, $this->currentTime());

        return $dataOk;
    }


    /**
     * Seta se a busca sera por um termo exato
     *
     * @param boolean $status
     */
    public function setAccurateTerm($status = true)
    {
        $this->accurateTerm = $status;
    }



    /**
     * seta uma consulta com termos cruzados
     *
     * @param boolean $status
     */
    public function setCrossingTerm($status = true)
    {
        $this->crossingTerm = $status;
    }


    /**
     * Concatena prefixos e suffixos com o resultado.
     *
     * @param boolean  $status
     */
    public function addPrefixSuffixInResult($status = true)
    {
        $this->addPrefixSuffixInresult = $status;
    }


    /**
     * Enter description here...
     *
     * @return unknown
     */
    public function currentTime()
    {
        return time() + microtime();
    }


    /**
     * Enter description here...
     *
     * @param unknown_type $ini_t
     * @param unknown_type $end_t
     * @return unknown
     */
    public function Total_Time($ini_t, $end_t)
    {
        return round($end_t - $ini_t, 4);
    }


    public function setDebug($status = true)
    {
        $this->debug = $status;
    }
}
?>