<?php
/**
 * <--- Copyright 2005-2011 de Solis - Cooperativa de Solues Livres Ltda. e
 * Univates - Centro Universitrio.
 * 
 * Este arquivo  parte do programa Gnuteca.
 * 
 * O Gnuteca  um software livre; voc pode redistribu-lo e/ou modific-lo
 * dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao
 * do Software Livre (FSF); na verso 2 da Licena.
 * 
 * Este programa  distribudo na esperana que possa ser til, mas SEM
 * NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO
 * ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU/GPL em
 * portugus para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo
 * "LICENCA.txt", junto com este programa, se no, acesse o Portal do Software
 * Pblico Brasileiro no endereo www.softwarepublico.gov.br ou escreva para a
 * Fundao do Software Livre (FSF) Inc., 51 Franklin St, Fifth Floor, Boston,
 * MA 02110-1301, USA --->
 * 
 * This file handles the connection and actions for gtcKardexControl table
 *
 * @author Luiz Gilberto Gregory [luiz@solis.coop.br]
 *
 * @version $Id$
 *
 * \b Maintainers \n
 * Eduardo Bonfandini [eduardo@solis.coop.br]
 * Jamiel Spezia [jamiel@solis.coop.br]
 * Luiz Gregory Filho [luiz@solis.coop.br]
 * Moises Heberle [moises@solis.coop.br]
 * Jader Osvino Fiegenbaum [jader@solis.coop.br]
 *
 * @since
 * Class created on 05/12/2008
 *
 **/

class BusinessGnuteca3BusKardexControl extends GnutecaBusiness
{
    public $MIOLO;
    public $module;
    public $cols;
    public $pkeys;
    public $pkeysWhere;
    public $fullColumns;

    public  $controlNumber,
            $codigoDeAssinante,
            $libraryUnitId,
            $acquisitionType,
            $vencimentoDaAssinatura,
            $dataDaAssinatura,
            $entranceDate,
            $line;

    public  $numberType,
            $number,
            $subscriberCode,
            $fiscalNote,
            $titleS,
            $expressionS,
            $startDate,
            $endDate;

   public   $businessGenericSearch2,
            $businessMaterial,
            $businessTag,
            $businessMaterialControl,
            $businessSearchableField;

    /**
     * Class constructor
     **/
    function __construct()
    {
        parent::__construct();

        $this->MIOLO    = MIOLO::getInstance();
        $this->module   = MIOLO::getCurrentModule();


        $this->table    = 'gtckardexcontrol';

        $this->pkeys    = 'controlNumber,
                           codigoDeAssinante';

        $this->cols     = 'libraryUnitId,
                           acquisitionType,
                           vencimentoDaAssinatura,
                           dataDaAssinatura,
                           entranceDate,
                           line';

        $this->fullColumns =  str_replace(array("\n", " "), "", $this->pkeys . ',' . $this->cols);

        $this->businessGenericSearch2   = $this->MIOLO->getBusiness( $this->module, 'BusGenericSearch2');
        $this->businessMaterial         = $this->MIOLO->getBusiness( $this->module, 'BusMaterial');
        $this->businessMaterialControl  = $this->MIOLO->getBusiness( $this->module, 'BusMaterialControl');
        $this->businessTag              = $this->MIOLO->getBusiness( $this->module, 'BusTag');
        $this->businessSearchableField  = $this->MIOLO->getBusiness( $this->module, 'BusSearchableField');
    }


    /**
     * Return a specific record from the database
     *
     * @param $itemNumber (integer): Primary key of the record to be retrieved
     *
     * @return (object): Return an object of the type handled by the class
     *
     **/
    public function getKardexControl()
    {

    }


    /**
     * Enter description here...
     *
     * @param unknown_type $controlNumber
     * @param unknown_type $libraryUnitId
     * @param unknown_type $null
     * @return unknown
     */
    public function getKardexOfMaterial($controlNumber, $libraryUnitId = null)
    {

        $this->clear();
        $this->setTables($this->table);
        $this->setColumns($this->fullColumns);
        $this->setWhere("controlNumber = ?");

        $data[] = $controlNumber;

        if($libraryUnitId)
        {
            $libraryUnitId = is_array($libraryUnitId) ? $libraryUnitId : array($libraryUnitId);
            $libraryUnitId = implode("','", $libraryUnitId);
            if(strlen($libraryUnitId))
            {
                $this->setWhere("libraryUnitId IN ('$libraryUnitId')");
            }
        }

        $sql = $this->select($data);
        return $this->query($sql, true);
    }


    /**
     * Enter description here...
     *
     */
    public function getControlNumber($codigoDeAssinante)
    {
        $this->clear();
        $this->setTables($this->table);
        $this->setColumns("controlNumber");
        $this->setWhere("codigoDeAssinante = ?");
        $sql =$this->select(array($codigoDeAssinante));
        $r = $this->query();

        if(!$r)
        {
            return false;
        }

        return $r[0][0];
    }


    /**
     * Do a search on the database table handled by the class
     *
     * @param $filters (object): Search filters
     *
     * @return (array): An array containing the search results
     **/
    public function searchKardexControl()
    {
        $this->businessGenericSearch2->clean();

        $this->businessGenericSearch2->addSearchTagField(MARC_KARDEX_SUBSCRIBER_ID_TAG);
        $this->businessGenericSearch2->addSearchTagField(MARC_KARDEX_LIBRARY_UNIT_ID_TAG);
        $this->businessGenericSearch2->addSearchTagField(MARC_TITLE_TAG);

        if(!$this->number           &&
           !$this->subscriberCode   &&
           !$this->fiscalNote       &&
           !$this->titleS           &&
           !$this->expressionS      &&
           !$this->startDate        &&
           !$this->endDate)
        {
            $this->expressionS = '%';
        }

        if($this->expressionS)
        {
            $exp = $this->businessSearchableField->parseExpression( $this->expressionS );
            $this->businessGenericSearch2->addMaterialWhereByExpression($exp);
        }

        if($this->subscriberCode)
        {
            $this->businessGenericSearch2->addMaterialWhere(MARC_KARDEX_FIELD, MARC_KARDEX_SUBSCRIBER_ID_SUBFIELD, $this->subscriberCode);
        }

        if($this->fiscalNote)
        {
            $this->businessGenericSearch2->addMaterialWhere(MARC_KARDEX_FIELD, MARC_KARDEX_FISCAL_NOTE_SUBFIELD, $this->fiscalNote);
        }

        if($this->titleS)
        {
            $this->businessGenericSearch2->addMaterialWhereByTag(MARC_TITLE_TAG, $this->titleS);
        }

        if($this->startDate && $this->endDate)
        {
            $startDate  = new GnutecaDate2($this->startDate);
            $endDate    = new GnutecaDate2($this->endDate);
            $values     = array($startDate->getDate(GnutecaDate2::getDBMask('date')), $endDate->getDate(GnutecaDate2::getDBMask('date')));
            $this->businessGenericSearch2->addMaterialWhereByTag(MARC_KARDEX_SIGNATURE_END_TAG, $values, 'AND', 'between');
        }

        if($this->startDate && !$this->endDate)
        {
            $this->businessGenericSearch2->addMaterialWhereByTag(MARC_KARDEX_SIGNATURE_END_TAG, GnutecaDate::identifyDateMask($this->startDate, GnutecaDate::getDBMask("DATE")), 'AND', '>=');
        }

        if(!$this->startDate && $this->endDate)
        {
           $this->businessGenericSearch2->addMaterialWhereByTag(MARC_KARDEX_SIGNATURE_END_TAG, GnutecaDate::identifyDateMask($this->endDate, GnutecaDate::getDBMask("DATE")), 'AND', '<=');
        }

        list($sp, $le) = explode("-", SPREADSHEET_CATEGORY_COLECTION);
        $this->businessGenericSearch2->addMaterialControlWhere("category",  $sp);
        $this->businessGenericSearch2->addMaterialControlWhere("level",     $le);

        //Adiciona condio quando  informado o nmero de controle ou o nmero do tombo
        if(!empty($this->number))
        {
            switch ($this->numberType)
            {
                case "cn" :
                    $cn = $this->number;
                break;
                case "sc" :
                    $cn = $this->getControlNumber($this->number);
                break;
                case "wn" :
                    $cn = $this->businessMaterial->getControlNumberByWorkNumber($this->number);
                break;
            }

            $this->businessGenericSearch2->addMaterialControlWhere('controlnumber', $cn);

        }
        else
        {
           $this->businessGenericSearch2->addPrefixSuffixInResult();
        }

        //busca os nmeros de controle
        $this->businessGenericSearch2->executeSearch();
        //executa a busca com os nmeros de controle da funo "executeSearch"
        $data = $this->businessGenericSearch2->getWorkSearch();

        if(!$data)
        {
            return false;
        }

        // TRABALHA A EXIBIO DOS DADOS NA GRID
        foreach ($data as $l => $values)
        {
            $gridData[$l][0] = $values['CONTROLNUMBER'];

            $div1 =  new MDiv(null, $this->businessTag->getTagNameByTag(MARC_TITLE_TAG)                     .":".    $values[MARC_TITLE_TAG][0]->content);
            $div2 =  new MDiv(null, $this->businessTag->getTagNameByTag(MARC_KARDEX_SUBSCRIBER_ID_TAG)      .":".    $values[MARC_KARDEX_SUBSCRIBER_ID_TAG][0]->content);
            $div3 =  new MDiv(null, $this->businessTag->getTagNameByTag(MARC_KARDEX_LIBRARY_UNIT_ID_TAG)    .":".    $values[MARC_KARDEX_LIBRARY_UNIT_ID_TAG][0]->content);
            $gridData[$l][1] = $div1->generate() . $div2->generate() . $div3->generate();
        }

        return $gridData;
    }


    /**
     * Insert a new record
     *
     * @param $data (object): An object of the type handled by the class
     *
     * @return True if succed, otherwise False
     *
     **/
    public function insertKardexControl()
    {
        $this->clear();
        $this->setColumns($this->fullColumns);
        $this->setTables($this->table);
        $sql = $this->insert( $this->associateData($this->fullColumns) );

        $rs  = $this->execute($sql);

        return $rs;
    }


    /**
     * Update data from a specific record
     *
     * @param $data (object): Data which will replace the old record data
     *
     * @return (boolean): True if succeed, otherwise False
     *
     **/
    public function updateKardexControl()
    {
        $this->clear();
        $this->setTables($this->table);
        $this->setColumns($this->cols);
        $this->setWhere('controlNumber = ? AND codigoDeAssinante = ?');
        $sql = $this->update( $this->associateData($this->cols . ',' . $this->pkeys) );
        $rs  = $this->execute($sql);
        return  $rs;
    }


    /**
     * Delete a record
     *
     * @param $itemNumber (string): Primary key for deletion
     *
     * @return (boolean): True if succeed, otherwise False
     *
     **/
    public function deleteKardexControl($controlNumber, $codigoDeAssinante)
    {
        $this->clear();
        $this->setTables($this->table);
        $this->setWhere('controlNumber = ? AND codigoDeAssinante = ?');
        $sql = $this->delete( array($controlNumber, $codigoDeAssinante) );
        $rs  = $this->execute($sql);
        return $rs;
    }


    public function deleteAllKardexByMaterialControl($controlNumber)
    {
        $this->clear();
        $this->setTables($this->table);
        $this->setWhere('controlNumber = ?');
        $sql = $this->delete( array($controlNumber) );
        $ok[] = $this->execute($sql);

        $this->businessMaterial->controlNumber = $controlNumber;
        $ok[] = $this->businessMaterial->deleteMaterial();
        $ok[] = $this->businessMaterialControl->deleteMaterialControl($controlNumber);

        return array_search(false, $ok) == false;
    }


    public function clean()
    {
        $this->this->controlNumber      =
        $this->codigoDeAssinante        =
        $this->libraryUnitId            =
        $this->acquisitionType          =
        $this->vencimentoDaAssinatura   =
        $this->dataDaAssinatura         =
        $this->entranceDate             =
        $this->line                     = null;
    }


    function getReferenced($controlNumberFather)
    {
        $sql = "SELECT A.controlNumber as cn
                  FROM gtcmaterialcontrol A
                 WHERE A.controlnumberfather    = '$controlNumberFather'";

        return $this->query($sql);
    }
}
?>