<?php
/**
 * <--- Copyright 2005-2011 de Solis - Cooperativa de Solues Livres Ltda. e
 * Univates - Centro Universitrio.
 * 
 * Este arquivo  parte do programa Gnuteca.
 * 
 * O Gnuteca  um software livre; voc pode redistribu-lo e/ou modific-lo
 * dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao
 * do Software Livre (FSF); na verso 2 da Licena.
 * 
 * Este programa  distribudo na esperana que possa ser til, mas SEM
 * NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO
 * ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU/GPL em
 * portugus para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo
 * "LICENCA.txt", junto com este programa, se no, acesse o Portal do Software
 * Pblico Brasileiro no endereo www.softwarepublico.gov.br ou escreva para a
 * Fundao do Software Livre (FSF) Inc., 51 Franklin St, Fifth Floor, Boston,
 * MA 02110-1301, USA --->
 * 
 * @author Luiz G Gregory Filho [luiz@solis.coop.br]
 *
 * $version: $Id$
 *
 * \b Maintainers \n
 * Eduardo Bonfandini [eduardo@solis.coop.br]
 * Jamiel Spezia [jamiel@solis.coop.br]
 * Luiz Gregory Filho [luiz@solis.coop.br]
 * Moises Heberle [moises@solis.coop.br]
 *
 * @since
 * Class created on 05/ago/2008
 *
 **/
class BusinessGnuteca3BusLibraryGroup extends GnutecaBusiness
{
    public $MIOLO;
    public $module;

    public  $libraryGroupId,
            $description,
            $observation;

    public  $libraryGroupIdS,
            $descriptionS;

    /**
     * Constructor Method
     */

    function __construct()
    {
        parent::__construct();

        $this->MIOLO    = MIOLO::getInstance();
        $this->module   = MIOLO::getCurrentModule();

        $this->setData(null);
        $this->setColumns();
        $this->setTables();
    }

    /**
     * Seta as tabelas
     *
     * @param (String || Array) $tables
     */

    public function setTables($tables = null)
    {
        if(is_null($table))
        {
            $table = "gtclibrarygroup";
        }

        parent::setTables($table);
    }

    /**
     * Este mtodo seta as colunas da tabela.
     *
     * @param (String || Array) $columns
     */

    public function setColumns($type = null)
    {
        switch($type)
        {
            case 'insert':
            case 'update':
                $columns = array
                (
                    'description',
                    'observation'
                );
                break;

            case 'list':
                $columns = array
                (
                    'librarygroupid',
                    'description',
                );
                break;

            case 'select':
            default:
                $columns = array
                (
                    'librarygroupid',
                    'description',
                    'observation'
                );
                break;
        }

        parent::setColumns($columns);
    }

    /**
     * Seta as condies do sql
     *
     * @return void
     */

    public function getWhereCondition()
    {
        $where = "";

        if(!empty($this->libraryGroupIdS) || !empty($this->libraryGroupId))
        {
            $where.= " libraryGroupId = ? AND ";
        }
        if(!empty($this->descriptionS))
        {
            $where.= " lower(description) LIKE lower(?) AND ";
        }

        if(strlen($where))
        {
            $where = substr($where, 0, strlen($where) - 4);
            parent::setWhere($where);
        }
    }

    /**
     * Trabalha o Data Object retornado do form
     *
     * transforma em um array para enviar para o where condition do sql
     *
     * @return (Array) $args
     */

    private function getDataConditionArray()
    {
        $args = array();

        if(!empty($this->libraryGroupIdS))
        {
            $args[] = $this->libraryGroupIdS;
        }
        if(!empty($this->libraryGroupId))
        {
            $args[] = $this->libraryGroupId;
        }
        if(!empty($this->descriptionS))
        {
            $this->descriptionS = trim($this->descriptionS);
            $this->descriptionS = str_replace(" ", "%", $this->descriptionS);
            $args[] = "%{$this->descriptionS}%";
        }

        return $args;
    }

    /**
     * Do a search on the database table handled by the class
     *
     * @return (array): An array containing the search results
     */

    public function searchLibraryGroup()
    {
      /*  parent::clear();

        $this->setTables();
        $this->setColumns();
        $this->getWhereCondition();
        $sql = parent::select($this->getDataConditionArray());
        return parent::query();*/
        unset($this->libraryGroupId); //estava ocorrendo erro pos-busca no formulario
        $this->clear();
        $filters = array(
            'libraryGroupId'     => 'equals',
            'description'        => 'ilike',
            'ttagsags'           => 'ilike'
        );
        return $this->autoSearch($filters, $object);
    }

    /**
     * Insert a new record
     *
     * @return True if succed, otherwise False
     */

    public function insertLibraryGroup()
    {
        parent::clear();

        $this->setTables();
        $this->setColumns('insert');

        $data = array
        (
            $this->description,
            $this->observation
        );

        $sql = parent::insert($data);

        return parent::Execute();
    }

    /**
     * Atualiza um determinado registro
     *
     * @return True if succed, otherwise False
     */

    public function updateLibraryGroup()
    {
        parent::clear();

        $this->getWhereCondition();

        $this->setTables();
        $this->setColumns('update');

        $data = array
        (
            $this->description,
            $this->observation,
            $this->libraryGroupId
        );

        $sql = parent::update($data);

        return parent::Execute();
    }

    /**
     * retorna um determinado registro
     *
     * @param (int) $labelLayoutId - Id do registro
     * @return (Array)
     */

    public function getLibraryGroup($libraryGroupId)
    {
        parent::clear();

        $this->setTables();
        $this->setColumns('update');

        $this->libraryGroupId = $libraryGroupId;

        $this->getWhereCondition();
        parent::select($this->getDataConditionArray());

        $result = parent::query();

        if(!$result)
        {
            return false;
        }

        list
        (
            $this->description,
            $this->observation
        ) = $result[0];

        return $result[0];
    }

    /**
     * Delete a record
     *
     * @param $labelLayoutId (int): Primary key for deletion
     *
     * @return (boolean): True if succeed, otherwise False
     *
     */

    public function deleteLibraryGroup($libraryGroupId)
    {
        parent::clear();

        $this->setTables();

        $this->libraryGroupId = $libraryGroupId;
        $this->getWhereCondition();

        parent::delete(array($libraryGroupId));
        //vd($this->sql);

        return parent::Execute();
    }

    /**
     * List all records from the table handled by the class
     *
     * @param: (String) $orderBy - set order by selected records
     *
     * @return (array): Return an array with the entire table
     */

    public function listLibraryGroup($orderBy = "description")
    {
        parent::clear();

        $this->setTables();
        $this->setColumns("list");

        parent::setOrderBy($orderBy);
        parent::select();

        return parent::query();
    }

}
?>
