<?php

/**
 * <--- Copyright 2005-2011 de Solis - Cooperativa de Solues Livres Ltda. e
 * Univates - Centro Universitrio.
 * 
 * Este arquivo  parte do programa Gnuteca.
 * 
 * O Gnuteca  um software livre; voc pode redistribu-lo e/ou modific-lo
 * dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao
 * do Software Livre (FSF); na verso 2 da Licena.
 * 
 * Este programa  distribudo na esperana que possa ser til, mas SEM
 * NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO
 * ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU/GPL em
 * portugus para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo
 * "LICENCA.txt", junto com este programa, se no, acesse o Portal do Software
 * Pblico Brasileiro no endereo www.softwarepublico.gov.br ou escreva para a
 * Fundao do Software Livre (FSF) Inc., 51 Franklin St, Fifth Floor, Boston,
 * MA 02110-1301, USA --->
 * 
 *
 * @author Luiz G Gregory Filho [luiz@solis.coop.br]
 *
 * $version: $Id$
 *
 * \b Maintainers \n
 * Eduardo Bonfandini [eduardo@solis.coop.br]
 * Jamiel Spezia [jamiel@solis.coop.br]
 * Luiz Gregory Filho [luiz@solis.coop.br]
 * Moises Heberle [moises@solis.coop.br]
 *
 * @since
 * Class created on 29/07/2008
 *
 **/

/**
 * Class to manipulate
 **/

class BusinessGnuteca3BusMarcTagListing extends GnutecaBusiness
{
    /**
     * Attributes
     */

    public  $MIOLO;
    public  $marcTagListingId;
    public  $description_;
    public  $marc_options;
    public  $marcTagListingIdS,
            $descriptionS,
            $leaderTags,
            $indicador;
    public  $businessMarcTagListingOption;

    /**
     * Constructor Method
     */

    function __construct()
    {
        parent::__construct();

        $this->MIOLO    = MIOLO::getInstance();
        $this->module   = MIOLO::getCurrentModule();

        $this->businessMarcTagListingOption = $this->MIOLO->getBusiness('gnuteca3', 'BusMarcTagListingOption');

        $this->setData(null);
        $this->setColumns();
        $this->setTables();
    }

    /**
     * Seta as tabelas
     *
     * @param (String || Array) $tables
     */

    public function setTables($tables = 1)
    {
        switch($tables)
        {
            case 1 :
                $table = "gtcmarctaglisting";
                break;

            case 2 :
                $table = "gtcmarctaglistingoption";
                break;
        }

        parent::setTables($table);
    }

    /**
     * Este mtodo seta as colunas da tabela.
     *
     * @param (String || Array) $columns
     */

    public function setColumns($type = "All")
    {
        switch($type)
        {
            case "All" :
                $columns = array
                (
                    'marcTagListingId',
                    'description',
                );
                break;

            case "select" :
                $columns = array
                (
                    'description',
                );
                break;

            case "options":
                $columns = array
                (
                    'marcTagListingId',
                    'option',
                    'description',
                );
                break;

            case "selectOption":
                $columns = array
                (
                    'option',
                    'description',
                );
                break;
        }

        parent::setColumns($columns);
    }


    /**
     * Seta as condies do sql
     *
     * @return void
     */
    public function getWhereCondition()
    {
        $where = "";

        if(!empty($this->marcTagListingId))
        {
            $where.= " marcTagListingId = ? AND ";
        }

        if(!empty($this->marcTagListingIdS))
        {
            $where.= " lower(marcTagListingId) LIKE lower(?) AND ";
        }

        if(!empty($this->descriptionS))
        {
            $where.= " lower(description) LIKE lower(?) AND ";
        }

        if($this->leaderTags)
        {
            $where.= " lower(marcTagListingId) LIKE lower('000-%') AND ";
        }

        if(!empty($this->indicador))
        {
            $where.= " marcTagListingId LIKE ('{$this->indicador}-I%') AND ";
        }

        if(strlen($where))
        {
            $where = substr($where, 0, strlen($where) - 4);
            parent::setWhere($where);
        }
    }


    /**
     * Trabalha o Data Object retornado do form
     *
     * transforma em um array para enviar para o where condition do sql
     *
     * @return (Array) $args
     */
    private function getDataConditionArray()
    {
        $args = array();

        if(!empty($this->marcTagListingId))
        {
            $args[] = $this->marcTagListingId;
        }
        if(!empty($this->marcTagListingIdS))
        {
            $this->marcTagListingIdS = trim($this->marcTagListingIdS);
            $this->marcTagListingIdS = str_replace(" ", "%", $this->marcTagListingIdS);
            $args[] = "%{$this->marcTagListingIdS}%";
        }
        if(!empty($this->descriptionS))
        {
            $this->descriptionS = trim($this->descriptionS);
            $this->descriptionS = str_replace(" ", "%", $this->descriptionS);
            $args[] = "%{$this->descriptionS}%";
        }

        return $args;
    }


    /**
     * Do a search on the database table handled by the class
     *
     * @return (array): An array containing the search results
     */
    public function searchMarcTagListing()
    {
        parent::clear();
        $this->setTables();
        $this->setColumns();
        $this->getWhereCondition();
        parent::setOrderBy("marcTagListingId");
        $sql = parent::select($this->getDataConditionArray());
        return parent::query($sql);
    }


    /**
     * Insert a new record
     *
     * @return True if succed, otherwise False
     */
    public function insertMarcTagListing()
    {
        parent::clear();

        $this->setTables();
        $this->setColumns();

        $data = array
        (
            $this->marcTagListingId,
            $this->description_,
        );

        $sql = parent::insert($data);

        $ok = parent::Execute();

        if(!$ok)
        {
            return false;
        }

        return $this->insertMarcTagListingOptions();
    }


    /**
     * insere as opoes de listagem do marc
     *
     * @return (boolean)
     */
    private function insertMarcTagListingOptions()
    {
        return $this->businessMarcTagListingOption->insertMarcTagListingOptions($this->marcTagListingId, $this->marc_options);
    }


    /**
     * Atualiza um determinado registro
     *
     * @return True if succed, otherwise False
     */
    public function updateMarcTagListing()
    {
        parent::clear();

        $this->marcTagListingId = $this->marcTagListingId;
        $this->getWhereCondition();

        $this->setTables();
        $this->setColumns("select");

        $data = array
        (
            $this->description_,
            $this->marcTagListingId
        );

        $sql = parent::update($data);

        $ok = parent::Execute();

        if(!$ok)
        {
            return false;
        }

        $this->removeMarcTagListOptions($this->marcTagListingId);

        return $this->insertMarcTagListingOptions();
    }


    /**
     * Remove as opoes da marc tag listing
     *
     * @param varChar $marcTagListingId
     * @return boolean
     */
    public function removeMarcTagListOptions($marcTagListingId)
    {
       return $this->businessMarcTagListingOption->removeMarcTagListOptions($marcTagListingId);
    }


    /**
     * retorna um determinado registro
     *
     * @param (int) $marcTagListingId - Id do registro
     * @return (Array)
     */
    public function getMarcTagListing($marcTagListingId)
    {
        parent::clear();

        $this->setTables();
        $this->setColumns("All");

        $this->marcTagListingId = $marcTagListingId;

        $this->getWhereCondition();
        parent::select($this->getDataConditionArray());

        $result = parent::query();

        if(!$result)
        {
            return false;
        }

        list
        (
            $this->marcTagListingId,
            $this->description_

        ) = $result[0];

        return $result[0];
    }


    /**
     * retorna as opoes de um determinado marc tag
     *
     * @param (int) $marcTagListingId - Id do registro
     * @return (Array)
     */
    public function getMarcTagListingOptions($marcTagListingId)
    {
        $this->marc_options = $this->businessMarcTagListingOption->getMarcTagListingOptions($marcTagListingId);
        return $this->marc_options;
    }


    /**
     * Delete a record
     *
     * @param $marcTagListingId (int): Primary key for deletion
     *
     * @return (boolean): True if succeed, otherwise False
     *
     */
    public function deleteMarcTagListing($marcTagListingId)
    {
        $ok = $this->removeMarcTagListOptions($marcTagListingId);

        if(!$ok)
        {
            return false;
        }

        parent::clear();

        $this->setTables();

        $this->marcTagListingId = $marcTagListingId;
        $this->getWhereCondition();

        parent::delete(array($marcTagListingId));

        return parent::Execute();
    }

    /**
     * seta na classe os valores das opoes
     *
     * @param (Array de Objectos) $op
     */
    public function setMarcOptions($op)
    {
        $this->marc_options = $op;
    }


    /**
     * Retorna os campos do Leader
     */
    public function getMarcLeaderTags($marcTagListingId)
    {
        if(!$marcTagListingId)
        {
            return false;
        }

        $this->leaderTags = true;
        $this->marcTagListingId = $marcTagListingId;
        $r = $this->searchMarcTagListing();

        $this->leaderTags = false;

        if(!$r)
        {
            return false;
        }

        $marcLeaderTags->name    = $r[0][1];
        $marcLeaderTags->options = $this->businessMarcTagListingOption->getMarcTagListingOptions($r[0][0]);

        return $marcLeaderTags;
    }


    /**
     *
     */
    public function getTagOptions($marcTagListingId)
    {
        $this->marcTagListingId = $marcTagListingId;
        $r = $this->searchMarcTagListing();

        if(!$r)
        {
            return false;
        }

        $marcLeaderTags->name    = $r[0][1];
        $marcLeaderTags->options = $this->businessMarcTagListingOption->getMarcTagListingOptions($r[0][0]);

        return $marcLeaderTags;
    }

    /**
     *
     */
    public function getIndicadores($etiqueta)
    {
        $this->indicador        = $etiqueta;
        $this->marcTagListingId = null;

        $r = $this->searchMarcTagListing(1);
        $this->indicador    = false;

        if(!$r)
        {
            return false;
        }

        foreach($r as $i => $v)
        {
            $ind[$v[0]]->name    = $v[1];
            $ind[$v[0]]->options = $this->businessMarcTagListingOption->getMarcTagListingOptions($v[0]);
        }

        return $ind;
    }
}
?>
