<?php
/**
 * <--- Copyright 2005-2011 de Solis - Cooperativa de Solues Livres Ltda. e
 * Univates - Centro Universitrio.
 * 
 * Este arquivo  parte do programa Gnuteca.
 * 
 * O Gnuteca  um software livre; voc pode redistribu-lo e/ou modific-lo
 * dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao
 * do Software Livre (FSF); na verso 2 da Licena.
 * 
 * Este programa  distribudo na esperana que possa ser til, mas SEM
 * NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO
 * ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU/GPL em
 * portugus para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo
 * "LICENCA.txt", junto com este programa, se no, acesse o Portal do Software
 * Pblico Brasileiro no endereo www.softwarepublico.gov.br ou escreva para a
 * Fundao do Software Livre (FSF) Inc., 51 Franklin St, Fifth Floor, Boston,
 * MA 02110-1301, USA --->
 * 
 * This file handles the connection and actions for gtcMarcTagListingOption table
 *
 * @author Moises Heberle [moises@solis.coop.br]
 *
 * @version $Id$
 *
 * \b Maintainers \n
 * Eduardo Bonfandini [eduardo@solis.coop.br]
 * Jamiel Spezia [jamiel@solis.coop.br]
 * Luiz Gregory Filho [luiz@solis.coop.br]
 * Moises Heberle [moises@solis.coop.br]
 *
 * @since
 * Class created on 26/09/2008
 *
 **/

class BusinessGnuteca3BusMarcTagListingOption extends GnutecaBusiness
{
    public $MIOLO;
    public $module;
    public $cols;

    public $marcTagListingId;

    /**
     * Class constructor
     **/
    public function __construct()
    {
        parent::__construct();
        $this->MIOLO    = MIOLO::getInstance();
        $this->module   = MIOLO::getCurrentModule();
        $this->tables   = 'gtcMarcTagListingOption';
        $this->cols     = 'marcTagListingId,
                           option,
                           description';
    }


    /**
     * List values
     *
     * @param $marcTagListingId (integer)
     *
     * @return (array)
     */
    public function listMarcTagListingOption($marcTagListingId, $showDefault = FALSE)
    {
        $this->clear();
        $this->setTables($this->tables);
        $this->setColumns('option, description');
        $this->setWhere('marcTagListingId = ?');
        $sql    = $this->select(array($marcTagListingId));
        $query  = $this->query($sql);
        if (($query) && ($showDefault))
        {
            $query = array_merge(array('DF' => _M('Default', $this->module)), $query);
        }
        return $query;
    }


    /**
     * insere as opoes de listagem do marc
     *
     * @return (boolean)
     */
    public function insertMarcTagListingOptions($marcTagListingId, $marc_options)
    {
        foreach ($marc_options as $i => $values)
        {
        	if ($values->removeData)
        	{
        		continue;
        	}
            parent::clear();
            $this->setTables($this->tables);
            $this->setColumns($this->cols);

            $data = array
            (
                $marcTagListingId,
                $values->option,
                $values->description,
            );

            $sql = parent::insert($data);

            $ok[] = parent::Execute();
        }

        return (array_search(false, $ok) === false);
    }


    /**
     * Remove as opoes da marc tag listing
     *
     * @param varChar $marcTagListingId
     * @return boolean
     */
    public function removeMarcTagListOptions($marcTagListingId)
    {
        parent::clear();

        parent::setTables($this->tables);

        $this->marcTagListingId = $marcTagListingId;
        $this->getWhereCondition();

        parent::delete($this->getDataConditionArray());

        return parent::Execute();
    }

    /**
     * Trabalha o Data Object retornado do form
     *
     * transforma em um array para enviar para o where condition do sql
     *
     * @return (Array) $args
     */
    private function getDataConditionArray()
    {
        $args = array();

        if(!empty($this->marcTagListingId))
        {
            $args[] = $this->marcTagListingId;
        }

        return $args;
    }

   /**
     * Seta as condies do sql
     *
     * @return void
     */
    public function getWhereCondition()
    {
        $where = "";

        if(!empty($this->marcTagListingId))
        {
            $where.= " marcTagListingId = ? AND ";
        }

        if(strlen($where))
        {
            $where = substr($where, 0, strlen($where) - 4);
            parent::setWhere($where);
        }
    }

    /**
     * retorna as opoes de um determinado marc tag
     *
     * @param (int) $marcTagListingId - Id do registro
     * @return (Array)
     */
    public function getMarcTagListingOptions($marcTagListingId)
    {
        parent::clear();

        parent::setTables($this->tables);
        parent::setColumns('option, description');

        $this->marcTagListingId = $marcTagListingId;

        $this->getWhereCondition();

        parent::setOrderBy("option, marcTagListingId");

        $sql = parent::select($this->getDataConditionArray());

        $result = parent::query(null, 1);

        if(!$result)
        {
            return false;
        }

        return $result;
    }


    public function getMarcTagListingOption($marcTagListingId, $option)
    {
    	$this->clear();
    	$this->setColumns($this->cols);
    	$this->setTables($this->tables);
    	$this->setWhere('marcTagListingId = ?');
    	$this->setWhere('option = ?');
    	$sql = $this->select(array($marcTagListingId, $option));
    	$rs  = $this->query($sql, true);
    	return $rs[0];
    }
}
?>
