<?php
/**
 * <--- Copyright 2005-2011 de Solis - Cooperativa de Solues Livres Ltda. e
 * Univates - Centro Universitrio.
 * 
 * Este arquivo  parte do programa Gnuteca.
 * 
 * O Gnuteca  um software livre; voc pode redistribu-lo e/ou modific-lo
 * dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao
 * do Software Livre (FSF); na verso 2 da Licena.
 * 
 * Este programa  distribudo na esperana que possa ser til, mas SEM
 * NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO
 * ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU/GPL em
 * portugus para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo
 * "LICENCA.txt", junto com este programa, se no, acesse o Portal do Software
 * Pblico Brasileiro no endereo www.softwarepublico.gov.br ou escreva para a
 * Fundao do Software Livre (FSF) Inc., 51 Franklin St, Fifth Floor, Boston,
 * MA 02110-1301, USA --->
 * 
 * This file handles the connection and actions for basPerson table
 *
 * @author Moises Heberle [moises@solis.coop.br]
 *
 * @version $Id$
 *
 * \b Maintainers \n
 * Eduardo Bonfandini [eduardo@solis.coop.br]
 * Jamiel Spezia [jamiel@solis.coop.br]
 * Luiz Gregory Filho [luiz@solis.coop.br]
 * Moises Heberle [moises@solis.coop.br]
 *
 * @since
 * Class created on 06/08/2008
 *
 **/
class BusinessGnuteca3BusPerson extends GnutecaBusiness
{
    public $colsNoId;
    public $fullColumns;
    public $MIOLO;
    public $module;
    public $busBond;
    public $busPenalty;
    public $busLibraryUnitConfig;
    public $bondOrderBy;
    public $busPersonOperationProcess;
    public $busPhone;

    public $personId;
    public $personName;
    public $city;
    public $zipCode;
    public $location;
    public $number;
    public $complement;
    public $neighborhood;
    public $email;
    public $password;
    public $operationProcess;

    public $personIdS;
    public $personNameS;
    public $cityS;
    public $zipCodeS;
    public $locationS;
    public $numberS;
    public $complementS;
    public $neighborhoodS;
    public $emailS;
    public $passwordS;

    public $bond;
    public $penalty;
    public $personLibraryUnit;
    public $phone;


    /**
     * Class constructor
     **/
    public function __construct()
    {
        parent::__construct();
        $this->MIOLO    = MIOLO::getInstance();
        $this->tables   = 'basPerson';
        $this->colsNoId = 'name as personName,
                           city,
                           zipCode,
                           location,
                           number,
                           complement,
                           neighborhood,
                           email,
                           password';
        $this->fullColumns = 'personId, ' . $this->colsNoId;
        $this->busBond              = $this->MIOLO->getBusiness($this->module, 'BusBond');
        $this->busPenalty           = $this->MIOLO->getBusiness($this->module, 'BusPenalty');
        $this->busPersonLibraryUnit = $this->MIOLO->getBusiness($this->module, 'BusPersonLibraryUnit');
        $this->busLibraryUnitConfig = $this->MIOLO->getBusiness($this->module, 'BusLibraryUnitConfig');
        $this->busPersonOperationProcess = $this->MIOLO->getBusiness($this->module, 'BusPersonOperationProcess');
        $this->busPhone = $this->MIOLO->getBusiness($this->module, 'BusPhone');
    }


    /**
     * List all records from the table handled by the class
     *
     * @param: None
     *
     * @returns (array): Return an array with the entire table
     *
     **/
    public function listPerson()
    {
        return 'Not implemented';
    }


    /**
     * Return a specific record from the database
     *
     * @param $personId (integer): Primary key of the record to be retrieved
     *
     * @return (object): Return an object of the type handled by the class
     *
     **/
    public function getPerson($personId, $return=FALSE, $onlyActive)
    {
    	if (!$personId || !is_numeric($personId))
    	{
    		return false;
    	}
    	else
    	{
	        $data = array($personId);

	        $this->clear();
	        $this->setColumns($this->fullColumns);
	        $this->setTables($this->tables);
	        $this->setWhere('personId = ?');
	        $sql = $this->select($data);
	        $rs  = $this->query($sql, TRUE);

	        if ($rs)
	        {
		        if ( !$return )
		        {
			        $this->setData( $rs[0] );
			        $this->email = $rs[0]->email; //o setData para o campo email por algum motivo nao esta funcionando corretamente

			        $this->busBond->personIdS = $personId;
			        $this->bond = $this->busBond->searchBond(TRUE, $this->bondOrderBy);

			        $this->busPenalty->personIdS = $personId;
			        $this->penalty = $this->busPenalty->searchPenalty(TRUE, NULL, FALSE);

			        $this->busPersonLibraryUnit->personIdS = $personId;
			        $this->personLibraryUnit = $this->busPersonLibraryUnit->searchPersonLibraryUnit(TRUE);
			        
			        $this->busPhone->personIdS = $personId;
			        $this->phone = $this->busPhone->searchPhone(true);
			        
			        return $this;
		        }
		        else
		        {
		        	$result  = $rs[0];
			        $this->busBond->personIdS = $personId;
			        
			        //mostra somente link ativo
			        if ( $onlyActive )
			        {
                        $this->busBond->byActive = true;
			        }
			        
			        $result->bond  = $this->busBond->searchBond(TRUE);

			        $this->busPenalty->personIdS = $personId;
			        $result->penalty = $this->busPenalty->searchPenalty(TRUE, NULL, TRUE);

			        $this->busPersonLibraryUnit->personIdS = $personId;
			        $result->personLibraryUnit = $this->busPersonLibraryUnit->searchPersonLibraryUnit(TRUE);
			        
			        return $result;
		        }
	        }
    	}
    }


    /**
     * Do a search on the database table handled by the class
     *
     * @param $filters (object): Search filters
     *
     * @return (array): An array containing the search results
     **/
    public function searchPerson()
    {
        $this->clear();

        if ( $v = $this->personIdS )
        {
            $this->setWhere('personId = ?');
            $data[] = $v;
        }
        if ( $v = $this->personNameS )
        {
            $this->setWhere('lower(name) LIKE lower(?)');
            $data[] = $v . '%';
        }
        if ( $v = $this->cityS )
        {
            $this->setWhere('lower(city) LIKE lower(?)');
            $data[] = $v . '%';
        }
        if ( $v = $this->zipCodeS )
        {
            $this->setWhere('zipCode = ?');
            $data[] = $v;
        }
        if ( $v = $this->locationS )
        {
            $this->setWhere('lower(location) LIKE lower(?)');
            $data[] = $v . '%';
        }
        if ( $v = $this->numberS )
        {
            $this->setWhere('number = ?');
            $data[] = $v;
        }
        if ( $v = $this->complementS )
        {
            $this->setWhere('lower(complement) LIKE lower(?)');
            $data[] = $v . '%';
        }
        if ( $v = $this->neighborhoodS )
        {
            $this->setWhere('lower(neighborhood) LIKE lower(?)');
            $data[] = $v . '%';
        }
        if ( $v = $this->emailS )
        {
            $this->setWhere('lower(email) LIKE lower(?)');
            $data[] = $v . '%';
        }

        $this->setColumns($this->fullColumns);
        $this->setTables($this->tables);
        $this->setOrderBy('personId');
        $sql = $this->select($data);
        $rs  = $this->query($sql);
        return $rs;
    }


    /**
     * Insert a new record
     *
     * @param $data (object): An object of the type handled by the class
     *
     * @return True if succed, otherwise False
     *
     **/
    public function insertPerson()
    {
        $manual = false;
        //se no for informado cdigo no formulrio, pega o nextval
        if ( !$this->personId )
        {
            $this->personId = $this->getNextId();
        }
        else
        {
            $manual = true;
        }
        
        $columns = 'personId,
                    name,
                    city,
                    zipCode,
                    location,
                    number,
                    complement,
                    neighborhood,
                    email,
                    password';
        $this->clear();
        $this->setColumns($columns);
        $this->setTables($this->tables);

        $sql = $this->insert( $this->associateData($this->fullColumns) );
        $rs  = $this->execute($sql);

        if ($this->bond)
        {
            foreach ($this->bond as $value)
            {
                $this->busBond->setData($value);
                $this->busBond->personId = $this->personId;
                $this->busBond->insertBond();
            }
        }

        if ($this->penalty)
        {
            foreach ($this->penalty as $value)
            {
                $this->busPenalty->setData($value);
                $this->busPenalty->personId = $this->personId;
                $this->busPenalty->insertPenalty();
            }
        }
        
        if ( $this->phone )
        {
        	foreach ( $this->phone as $key=>$value )
        	{
        		$this->busPhone->setData($value);
        		$this->busPhone->personId = $this->personId;
        		$this->busPhone->insertPhone();
        	}
        }

        if ($this->personLibraryUnit)
        {
            foreach ($this->personLibraryUnit as $value)
            {
                $this->busPersonLibraryUnit->setData($value);
                $this->busPersonLibraryUnit->personId = $this->personId;
                $this->busPersonLibraryUnit->insertPersonLibraryUnit();
            }
        }
        
        if ( $this->operationProcess )
        {
        	$this->busPersonOperationProcess->personId = $this->personId;
        	$this->busPersonOperationProcess->operationProcess = $this->operationProcess;
        	$this->busPersonOperationProcess->insertPersonOperationProcess();
        }

        //se foi especificado codigo manual, atualiza a tabela de sequncia da pessoa
        if ( $manual )
        {
            $this->updateSequenceId();
        }
        
        return $rs;
    }


    /**
     * Update data from a specific record
     *
     * @param $data (object): Data which will replace the old record data
     *
     * @return (boolean): True if succeed, otherwise False
     *
     **/
    public function updatePerson()
    {
        $this->clear();
        //Quando parmetro CHANGE_WRITE_PERSON estiver configurado para no gravar dados do usurio. No deve atualizar view de Pessoas.
        if (MUtil::getBooleanValue(CHANGE_WRITE_PERSON) == FALSE)
        {
            $rs = '1';
        }
        else
        {
            $this->setColumns('
                name,
                city,
                zipCode,
                location,
                number,
                complement,
                neighborhood,
                email,
                password
            ');
            $this->setTables($this->tables);
            $this->setWhere('personId = ?');
            $sql = $this->update( $this->associateData( $this->colsNoId . ', personId' ) );
            $rs  = $this->execute($sql);
        }

        if ( $this->phone )
        {
            foreach ( $this->phone as $key=>$value )
            {
                $this->busPhone->setData($value);
                $this->busPhone->personId = $this->personId;
                $this->busPhone->updatePhone();
            }
        }
        
        if ($this->bond)
        {
            foreach ($this->bond as $value)
            {
                $this->busBond->setData($value);
                $this->busBond->personId = $this->personId;
                $this->busBond->updateBond();
            }
        }

        if ($this->penalty)
        {
            foreach ($this->penalty as $value)
            {
                $this->busPenalty->setData($value);
                $this->busPenalty->personId = $this->personId;
                $this->busPenalty->updatePenalty();
            }
        }


        if ($this->personLibraryUnit)
        {
            $this->busPersonLibraryUnit->personIdS = $this->personId;
            $search = $this->busPersonLibraryUnit->searchPersonLibraryUnit(TRUE);
            if ($search)
            {
                foreach ($search as $value)
                {
                    $this->busPersonLibraryUnit->deletePersonLibraryUnit($value->libraryUnitId, $value->personId);
                }
            }
            foreach ($this->personLibraryUnit as $value)
            {
                if (!$value->removeData)
                {
                    $this->busPersonLibraryUnit->setData($value);
                    $this->busPersonLibraryUnit->personId = $this->personId;
                    $this->busPersonLibraryUnit->insertPersonLibraryUnit();
                }
            }
        }
        
        if ( $this->operationProcess )
        {
        	$this->busPersonOperationProcess->personId = $this->personId;
        	$this->busPersonOperationProcess->operationProcess = $this->operationProcess;
        	$this->busPersonOperationProcess->updatePersonOperationProcess();
        }

        return $rs;
    }


    /**
     * Delete a record
     *
     * @param $moduleConfig (string): Primary key for deletion
     * @param $parameter (string): Primary key for deletion
     *
     * @return (boolean): True if succeed, otherwise False
     *
     **/
    public function deletePerson($personId)
    {
        $data = array($personId);

        $this->clear();
        $this->setTables($this->tables);
        $this->setWhere('personId = ?');
        $sql = $this->delete($data);
        $rs  = $this->execute($sql);
        return $rs;
    }


    /**
     * Get constants for a specified module
     *
     * @param $moduleConfig (string): Name of the module to load values from
     *
     * @return (array): An array of key pair values
     *
     **/
    public function getPersonValues($personId)
    {
        $data = array($personId);

        $this->clear();
        $this->setColumns('personId');
        $this->setTables($this->tables);
        $this->setWhere('personId = ?');
        $sql = $this->select($data);
        $rs  = $this->query($sql);
        return $rs;
    }

    /**
     * Enter description here...
     *
     * @param unknown_type $personId
     * @return unknown
     */
    public function getBasicPersonInformations($personId)
    {
        $this->clear();
        $this->setColumns('personId, name, email');
        $this->setTables($this->tables);
        $this->setWhere('personId = ?');
        $sql = $this->select(array($personId));
        $rs  = $this->query($sql, true);
        return $rs[0];
    }


    public function getEmail($personId)
    {
        $data = array($personId);

        $this->clear();
        $this->setColumns('email');
        $this->setTables($this->tables);
        $this->setWhere('personId = ?');
        $sql = $this->select($data);
        $rs  = $this->query($sql);

        return $rs ? $rs[0][0] : false;
    }



    public function getPassword($personId)
    {
        $data = array($personId);

        $this->clear();
        $this->setColumns('password');
        $this->setTables($this->tables);
        $this->setWhere('personId = ?');
        $sql = $this->select($data);
        $rs  = $this->query($sql);

        return $rs;
    }


    public function getNextId()
    {
        $query = $this->query("SELECT NEXTVAL('seq_personId')");
        return $query[0][0];
    }

    /**
     * Mtodo que atualiza o id da sequncia com o maior cdigo de aluno
     *
     * @param $id
     */
    public function updateSequenceId()
    {
        return $this->execute("SELECT setval('seq_personid', (SELECT max(personId) FROM basPerson))");
    }


    /**
     * Make user login (authentication)
     *
     * @param int $user the user id (personId)
     * @param string $password the password of user
     * @return true if success
     */
    public function authenticate($user, $password)
    {
        $data = array($user, $password);

        $this->clear();
        $this->setColumns('name');
        $this->setTables($this->tables);
        $this->setWhere('personId = ?');
        $this->setWhere('password = ?');
        $sql = $this->select($data);
        $rs  = $this->query($sql);

        return $rs;
    }


    /**
     * Change the user password
     *
     * @param int $user the user code (personId)
     * @param string $password the user password
     * @param string $retype the retype of password to verify
     * @return true if change
     */
    public function changePassword($user, $password, $retype)
    {
    	if ( $password != $retype || !$password || !$retype)
    	{
    		return false;
    	}
    	else
    	{
    		if ( !$user )
    		{
    			return false;
    		}
    		else
    		{
	    		$this->getPerson($user);
	    		if ($this->personId)
	    		{
			        $this->clear();
			        $this->setColumns('password');
			        $this->setTables($this->tables);
			        $this->setWhere('personId = ?');
			        $this->password = $password;
			        $sql = $this->update( $this->associateData('password, personId' ) );
			        $rs  = $this->execute($sql);
			        return $rs;
	    		}
    		}
    	}
    }

    /**
     * Verify if user is in operation process (is making a process)
     *
     * @param integer $personId the user code
     * @return true if is in operation
     */
    public function isOperationProcess($personId)
    {
        if (!$personId)
        {
            return false;
        }
        //testa se tem registro, caso no, retorna false
        $personOperationProcess = $this->busPersonOperationProcess->getPersonOperationProcess($personId, true);
        
        $diff = 0;
        if ( $personOperationProcess )
        {
	        $operationProcess = new GDate($personOperationProcess->operationProcess);
	        $now = GDate::now();
	        $diff = $now->diffDates($operationProcess, GDate::ROUND_DOWN);

            if ( ($diff->seconds / 60) < OPERATION_PROCESS_TIME )
            {
                return true;
            }
            else
            {
                return false;
            }
        }
        else
        {
            return false;
        }
    }

    /**
     * Remove o processo de operao para uma pessoa
     *
     * @param unknown_type $personId
     * @return unknown
     */
    public function removeOperationProcess($personId)
    {
    	if (!$personId || !is_numeric($personId) ) //se no existir ou se no for nmero
    	{
    		return false;
    	}
    	else
    	{
    		return $this->busPersonOperationProcess->deletePersonOperationProcess($personId);
    	}
    }

    /**
     * Define o processo de operao para uma pessoa
     *
     * @param unknown_type $personId
     * @return unknown
     */
    public function setOperationProcess($personId)
    {
        if (!$personId)
        {
            return FALSE;
        }
        
        $data = array();
        $data[] = $personId;
        
        //pega hora e data atual em timestamp
        $this->clear();
        $this->setColumns('now()');
        $resultData = $this->query($this->select());
        $dateTime = $resultData[0][0];
        
        $this->busPersonOperationProcess->personId = $personId;
        $this->busPersonOperationProcess->operationProcess = $dateTime;
        
        $operationProcess = $this->busPersonOperationProcess->getPersonOperationProcess($personId, true);
        //atualiza se j existe processo, seno insere
        if ( strlen($operationProcess->operationProcess) > 0 )
        {
        	$this->busPersonOperationProcess->updatePersonOperationProcess();
        }
        else 
        {
        	$this->busPersonOperationProcess->insertPersonOperationProcess();
        }
    }


    public function checkAccessLibraryUnit($personId, $libraryUnitId)
    {
        $className = $this->busLibraryUnitConfig->getValueLibraryUnitConfig($libraryUnitId, 'CLASS_USER_ACCESS_IN_THE_LIBRARY');
        if (empty($className))
        {
            return true;
        }
        else
        {
            $bus = MIOLO::getInstance()->getBusiness($this->module, $className);
            return $bus->checkAccess($personId, $libraryUnitId);
        }
    }
}
?>
