<?php
/**
 * <--- Copyright 2005-2011 de Solis - Cooperativa de Solues Livres Ltda. e
 * Univates - Centro Universitrio.
 * 
 * Este arquivo  parte do programa Gnuteca.
 * 
 * O Gnuteca  um software livre; voc pode redistribu-lo e/ou modific-lo
 * dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao
 * do Software Livre (FSF); na verso 2 da Licena.
 * 
 * Este programa  distribudo na esperana que possa ser til, mas SEM
 * NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO
 * ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU/GPL em
 * portugus para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo
 * "LICENCA.txt", junto com este programa, se no, acesse o Portal do Software
 * Pblico Brasileiro no endereo www.softwarepublico.gov.br ou escreva para a
 * Fundao do Software Livre (FSF) Inc., 51 Franklin St, Fifth Floor, Boston,
 * MA 02110-1301, USA --->
 * 
 *
 * This file handles the connection and actions for basConfig table
 *
 * @author Eduardo Bonfandini [eduardo@solis.coop.br]
 *
 * @version $Id$
 *
 * \b Maintainers \n
 * Eduardo Bonfandini   [eduardo@solis.coop.br]
 * Jamiel Spezia        [jamiel@solis.coop.br]
 * Luiz Gregory Filho   [luiz@solis.coop.br]
 * Moises Heberle       [moises@solis.coop.br]
 * Sandro R. Weisheimer [sandrow@solis.coop.br]
 * Jader Osvino Fiegenbaum [jader@solis.coop.br]
 *
 * @since
 * Class created on 23/10/2008
 *
 **/
class BusinessGnuteca3BusPersonConfig extends GnutecaBusiness
{
    public $personId;
    public $parameter;
    public $value;
    public $configData;
    public $busPreference;

    const NOT_SEND_MAIL = 1;
    const PRINT_AND_SEND_RECEIPT = 2;
    const PRINT_RECEIPT = 3;
    const SEND_RECEIPT = 4;


    /**
     * Class constructor
     **/
    function __construct()
    {
        parent::__construct();
        $this->defineTables();
        $this->busPreference = $this->MIOLO->getBusiness($this->module, 'BusPreference');
    }


    /**
    * Define or redefine the class atributes;
    */
    function defineTables()
    {
        $this->setTables('gtcPersonConfig');
        $this->setId('personId, parameter');
        $this->setColumnsNoId('value');
    }


    /**
     * List all records from the table handled by the class
     *
     * @param: None
     *
     * @returns (array): Return an array with the entire table
     *
     **/
    public function listPersonConfig($object=FALSE)
    {
        $this->defineTables();
        return $this->autoList();
    }


    /**
     * Return a specific record from the database
     *
     * @param $moduleConfig (integer): Primary key of the record to be retrieved
     * @param $parameter (integer): Primary key of the record to be retrieved
     *
     * @return (object): Return an object of the type handled by the class
     *
     **/
    public function getPersonConfig($id)
    {
        $this->defineTables();
        $this->clear;
        //here you can pass how many where you want
        return $this->autoGet($id);
    }


    /**
     * Do a search on the database table handled by the class
     *
     * @param $filters (object): Search filters
     *
     * @return (array): An array containing the search results
     **/
    public function searchPersonConfig($object=false, $object)
    {
        $this->defineTables();
        $this->clear();

        //here you can pass how many where you want, or use filters

        $filters  = array(
                    'personId'              => 'equals',
                    'parameter'  			=> 'equals',
                    'value'                 => 'equals',
                    );
        return $this->autoSearch($filters, $object);
    }


    /**
     * Insert a new record
     *
     * @param $data (object): An object of the type handled by the class
     *
     * @return True if succed, otherwise False
     *
     **/
    public function insertPersonConfig()
    {
    	unset($this->parameter);
    	$search = $this->searchPersonConfig(true, true);
    	foreach ((array)$search as $v)
    	{
    		$parameters[] = $v->parameter;
    	}

        if ($this->configData && $this->personId)
        {
            //$this->deletePersonConfig($this->personId);
            $this->defineTables();
            foreach ($this->configData as $line => $info)
            {
                if ($line != 'btnSave' && strlen($info) > 0) 
                {
                    $this->parameter = $line;
                    $this->value     = $info;
                    $ok = (in_array($line, $parameters)) ? $this->autoUpdate() : $this->autoInsert();
                }
            }
        }
        return $ok;
    }


    /**
     * Update data from a specific record
     *
     * @param $data (object): Data which will replace the old record data
     *
     * @return (boolean): True if succeed, otherwise False
     *
     **/
    public function updatePersonConfig()
    {
        $this->defineTables();
        return $this->autoUpdate();
    }


    /**
     * Delete a record
     *
     * @param $moduleConfig (string): Primary key for deletion
     * @param $parameter (string): Primary key for deletion
     *
     * @return (boolean): True if succeed, otherwise False
     *
     **/
    public function deletePersonConfig($personId)
    {
        $this->clear();
        $this->setTables($this->tables);
        $this->setWhere('personId = ?');
        $sql = $this->delete( $personId );
        $rs  = $this->execute($sql);
        return $rs;
    }


    /**
     * Get value of person config
     *
     * @param $personId (integer)
     * @param $parameter (String)
     *
     * @return (String): Value
     */
    public function getValuePersonConfig($personId, $parameter)
    {
        $parameter = trim(strtoupper($parameter));
    	$ignorePersonConfig = FALSE;
        $user_config = explode("\n", USER_CONFIG);

        foreach ((array)$user_config as $v)
        {
            list($var, $perm) = explode('=', $v);

            //desconsidera as observaes, que esto depois do |
            $perm = explode('|',$perm );
            $perm = strtoupper(trim($perm[0]));

            $correctParameter = '';
            if (in_array($parameter, array('MARK_SEND_RETURN_MAIL_RECEIPT', 'MARK_PRINT_RECEIPT_RETURN')))
            {
            	$correctParameter = 'CONFIGURE_RECEIPT_RETURN';
            }

            if (in_array($parameter, array('MARK_SEND_LOAN_MAIL_RECEIPT', 'MARK_PRINT_RECEIPT_LOAN')))
            {
                $correctParameter = 'CONFIGURE_RECEIPT_LOAN';
            }

            if ( ((strtoupper($var) == $correctParameter) || (strtoupper($var) == $parameter)) && (in_array($perm, array('R', 'I'))))
            {
            	$ignorePersonConfig = TRUE;
            }
        }

        if ( (!$ignorePersonConfig) && ($personId) )
        {
	        $this->personId = $personId;
	        $this->parameter= $parameter;
	        $search			= $this->searchPersonConfig(null, true);
		    $result			= $search[0]->value;
        }

        //caso no encontre preferncia do usurio pega preferncia geral, na basConfig
	    if ( strlen($result) == 0 )
	    {
	    	$result = constant( $parameter );
	    }

	    return $result;
    }



    /**
     * Altera os registro de um usurio por outro
     *
     * @param integer $currentPersonId
     * @param integer $newPersonId
     * @return boolean
     */
    public function updatePersonId($currentPersonId, $newPersonId)
    {
        $this->clear();
        $this->setColumns("personId");
        $this->setTables($this->tables);
        $this->setWhere(' personId = ?');
        $sql = $this->update(array($newPersonId, $currentPersonId));
        $rs  = $this->execute($sql);
        return $rs;
    }




    public function getFields($onlyValues = FALSE, $personId = null)
    {
    	//Mensagem a ser mostrada no topo da tela
        if ( !$onlyValues )//hamado na circulao de material, no precisa dessa label
        {
            $fields[] = new MLabel(_M(LABEL_PERSON_CONFIG, $this->module));
        }

        if (!$personId)
        {
            $personId = BusinessGnuteca3BusAuthenticate::getUserCode();
        }

        $user_config = explode ("\n", USER_CONFIG);
        foreach ((array)$user_config as $v)
        {
        	//Separa a legenda do parmetro
            list($var1, $label) = explode('|', $v);
            //Parmetro vai em $var e valor em $perm
            list($var, $perm) = explode('=', $var1);
            $var = strtoupper(trim($var));
            $perm = strtoupper(trim($perm));
            if (!($var) || !($perm))
            {
                continue;
            }

            $value = $this->getValuePersonConfig($personId, $var);

            if ($var == "USER_DELAYED_LOAN")
            {
                //Separa valor em 2
                $delayedLoan   = explode(';', $value);
                $quantidade    = $delayedLoan[0];
                $periodo       = $delayedLoan[1];
            }
            switch ($var)
            {
                case 'USER_SEND_DELAYED_LOAN':
                    $fld = $sel = new GnutecaSelection($var, $value, '', GnutecaUtils::listYesNo(), NULL, NULL, NULL, TRUE);
                    $lab = new MLabel('Enviar e-mail de atraso', $this->module);
                    $lab->setWidth(FIELD_LABEL_SIZE);
                    $img = new MImage('imgHelp', null, $this->MIOLO->getUI()->getImage($this->module, 'help-16x16.png') );
                    $img->addAttribute('title', $label);
                    $img->addAttribute('alt', $label);
                    $img->addAttribute('onclick', 'this.title');
                    $fld = new MHContainer('', array($lab, $fld, $img));
                    //Para aparecer no profile (F10 -> F6), ref. ticket #3339
                    if (MIOLO::_REQUEST('action') == 'main:verifyUser')
                    {
                        $fld->label = $label;
                        $fld->value = GnutecaUtils::getYesNo($value);
                    }
                    break;

                case 'USER_DELAYED_LOAN':
                    $lblDL          = new MLabel(_M('Delayed loan', $this->module) . ':');
                    $lblDL->setWidth(FIELD_LABEL_SIZE);
                    $lblQuant       = new MLabel(_M('Quantidade', $this->module) . ':');
                    $Quant          = new MTextField('quantidade', $quantidade, '', FIELD_ID_SIZE);
                    $Quant->setReadOnly( ($perm == 'W') ? FALSE : TRUE );
                    $lblPer         = new MLabel(_M('Perodo', $this->module) . ':');
                    $Per            = new MTextField('periodo', $periodo, '', FIELD_ID_SIZE);
                    $Per->setReadOnly( ($perm == 'W') ? FALSE : TRUE );
                    $Ajuda = new MImage('imgHelp', null, $this->MIOLO->getUI()->getImage($this->module, 'help-16x16.png') );
                    $Ajuda->addAttribute('title', _M($label, $this->module));
                    $Ajuda->addAttribute('alt', _M($label, $this->module));
                    $fld = $sel = new MHContainer('hctDL', array($lblDL, $lblQuant, $Quant, $lblPer, $Per, $Ajuda));

                    //Para aparecer no profile (F10 -> F6), ref. ticket #5911
                    if (MIOLO::_REQUEST('action') == 'main:verifyUser')
                    {
	                    $fld->label = $label;
	                    $fld->value = _M('Quantidade: @1', $this->module, $quantidade) . ' - ' . _M('Perodo: @1', $this->module, $periodo);
                    }
	                break;

                case 'USER_SEND_NOTIFY_AQUISITION':
                    $fld =  $sel = new GnutecaSelection($var, $value, '', GnutecaUtils::listYesNo(), NULL, NULL, NULL, TRUE);
                    $lab = new MLabel('Enviar e-mail de novas aquisies', $this->module);
                    $lab->setWidth(FIELD_LABEL_SIZE);
                    $img = new MImage('imgHelp', null, $this->MIOLO->getUI()->getImage($this->module, 'help-16x16.png') );
                    $img->addAttribute('title', _M($label, $this->module));
                    $img->addAttribute('alt', _M($label, $this->module));
                    $fld = new MHContainer('', array($lab, $fld, $img));
                    //Para aparecer no profile (F10 -> F6), ref. ticket #3339
                    if (MIOLO::_REQUEST('action') == 'main:verifyUser')
                    {
                        $fld->label = $label;
                        $fld->value = GnutecaUtils::getYesNo($value);
                    }
                    break;

                case 'USER_NOTIFY_AQUISITION':
                    $fld = $sel = new MTextField($var, $value, '', FIELD_DESCRIPTION_SIZE);
                    $lab = new MLabel(_M('Notify acquisition', $this->module));
                    $lab->setWidth(FIELD_LABEL_SIZE);
                    $img = new MImage('imgHelp', null, $this->MIOLO->getUI()->getImage($this->module, 'help-16x16.png') );
                    $img->addAttribute('title', _M($label, $this->module));
                    $img->addAttribute('alt', _M($label, $this->module));
                    $fld = new MHContainer('', array($lab, $fld, $img));
                    if (MIOLO::_REQUEST('action') == 'main:verifyUser')
                    {
                        $fld->label = $label;
                        $fld->value = $value;
                    }

                    break;

                case 'USER_SEND_DAYS_BEFORE_EXPIRED':
                    $fld =  $sel = new GnutecaSelection($var, $value, '', GnutecaUtils::listYesNo(), NULL, NULL, NULL, TRUE);
                    $lab = new MLabel('Enviar e-mail antes do vencimento', $this->module);
                    $lab->setWidth(FIELD_LABEL_SIZE);
                    $img = new MImage('imgHelp', null, $this->MIOLO->getUI()->getImage($this->module, 'help-16x16.png') );
                    $img->addAttribute('title', _M($label, $this->module));
                    $img->addAttribute('alt', _M($label, $this->module));
                    $fld = new MHContainer('', array($lab, $fld, $img));
                    //Para aparecer no profile (F10 -> F6), ref. ticket #3339
                    if (MIOLO::_REQUEST('action') == 'main:verifyUser')
                    {
                        $fld->label = $label;
                        $fld->value = GnutecaUtils::getYesNo($value);
                    }
                    break;

                case 'USER_DAYS_BEFORE_EXPIRED':
                    $fld = $sel = new MTextField($var, $value, '', FIELD_DESCRIPTION_SIZE);
                    $lab = new MLabel(_M('Days before expired', $this->module));
                    $lab->setWidth(FIELD_LABEL_SIZE);
                    $img = new MImage('imgHelp', null, $this->MIOLO->getUI()->getImage($this->module, 'help-16x16.png') );
                    $img->addAttribute('alt', _M($label, $this->module));
                    $fld = new MHContainer('', array($lab, $fld, $img));
                    if (MIOLO::_REQUEST('action') == 'main:verifyUser')
                    {
                        $fld->label = $label;
                        $fld->value = $value;
                    }
                    break;

                case 'CONFIGURE_RECEIPT_LOAN':
                    $opts = $this->listReceiptConfigure();
                    $print = $this->getValuePersonConfig($personId, 'MARK_PRINT_RECEIPT_LOAN');
                    $send = $this->getValuePersonConfig($personId, 'MARK_SEND_LOAN_MAIL_RECEIPT');
                    $value = $this->getReceiptConfig($print, $send);

                    $fld =  $sel = new GnutecaSelection($var, $value, '', $opts, NULL, NULL, NULL, TRUE);
                    $lab = new MLabel('Comprovantes de emprstimo', $this->module);
                    $lab->setWidth(FIELD_LABEL_SIZE);
                    $img = new MImage('imgHelp', null, $this->MIOLO->getUI()->getImage($this->module, 'help-16x16.png') );
                    $img->addAttribute('title', _M($label, $this->module));
                    $img->addAttribute('alt', _M($label, $this->module));
                    $fld = new MHContainer('', array($lab, $fld, $img));
                    if (MIOLO::_REQUEST('action') == 'main:verifyUser')
                    {
                        $fld->label = $label;
                        $fld->value = $this->getNameOfReceiptConfigure($value);
                    }
                    break;


                case 'CONFIGURE_RECEIPT_RETURN':
                    $opts = $this->listReceiptConfigure();
                    $print = $this->getValuePersonConfig($personId, 'MARK_PRINT_RECEIPT_RETURN');
                    $send = $this->getValuePersonConfig($personId, 'MARK_SEND_RETURN_MAIL_RECEIPT');
                    $value = $this->getReceiptConfig($print, $send);

                    $fld = $sel = new GnutecaSelection($var, $value, '', $opts, NULL, NULL, NULL, TRUE);
                    $lab = new MLabel('Comprovantes de devoluo', $this->module);
                    $lab->setWidth(FIELD_LABEL_SIZE);
                    $img = new MImage('imgHelp', null, $this->MIOLO->getUI()->getImage($this->module, 'help-16x16.png') );
                    $img->addAttribute('title', _M($label, $this->module));
                    $img->addAttribute('alt', _M($label, $this->module));
                    $fld = new MHContainer('', array($lab, $fld, $img));
                    if (MIOLO::_REQUEST('action') == 'main:verifyUser')
                    {
                        $fld->label = $label;
                        $fld->value = $this->getNameOfReceiptConfigure($value);
                    }
                    break;

                case 'USER_SEND_RECEIPT_RENEW_WEB':
                    $fld = $sel = new GnutecaSelection($var, $value, '', GnutecaUtils::listYesNo(), NULL, NULL, NULL, TRUE);
                    $lab = new MLabel('Enviar comprovante de renovao web', $this->module);
                    $lab->setWidth(FIELD_LABEL_SIZE);
                    $img = new MImage('imgHelp', null, $this->MIOLO->getUI()->getImage($this->module, 'help-16x16.png') );
                    $img->addAttribute('title', _M($label, $this->module));
                    $img->addAttribute('alt', _M($label, $this->module));
                    $fld = new MHContainer('', array($lab, $fld, $img));
                    //Para aparecer no profile (F10 -> F6), ref. ticket #3339
                    if (MIOLO::_REQUEST('action') == 'main:verifyUser')
                    {
                        $fld->label = $label;
                        $fld->value = GnutecaUtils::getYesNo($value);
                    }
                    break;
            }
            switch ($perm)
            {
                case 'R':
                    $sel->setReadOnly(true);
                    $fields[] = $fld;
                    break;
                case 'W':
                    $fields[] = $fld;
                    break;
                case 'I':
                default:
            }
        }
        //Return array data for fill table raw
        if ($onlyValues)
        {
        	$data = array();
            foreach ($fields as $v)
            {
                $data[] = array($v->label, $v->getValue());
            }
            return $data;
        }
        return $fields;
    }


    /**
     * Mtodo que pega o valor da configurao de recibo
     * @param $print
     * @param $send
     * @return value of receipt configure
     */
    private function getReceiptConfig($print, $send)
    {
        $print = MUtil::getBooleanValue($print);
        $send = MUtil::getBooleanValue($send);

    	if ( !$print && !$send)
        {
            $value = self::NOT_SEND_MAIL;
        }
        elseif ( $print && $send )
        {
            $value = self::PRINT_AND_SEND_RECEIPT;
        }
        elseif ( $print )
        {
            $value = self::PRINT_RECEIPT;
        }
        else
        {
            $value = self::SEND_RECEIPT;
        }

        return $value;
    }


    /**
     * Mtodo que pega as configuraes de impresso e envio de recibo
     * @param valor da configurao
     * @return (object)
     */
    public function getPrintAndSendReceiptConfigure($value)
    {
    	$data = new stdClass();
    	switch ($value)
    	{
    		case self::NOT_SEND_MAIL:
    			$data->print = DB_FALSE;
    			$data->send = DB_FALSE;
    			break;

            case self::PRINT_AND_SEND_RECEIPT:
    			$data->print = DB_TRUE;
    			$data->send = DB_TRUE;
    			break;

            case self::PRINT_RECEIPT:
            	$data->print = DB_TRUE;
            	$data->send = DB_FALSE;
            	break;

            case self::SEND_RECEIPT:
            	$data->print = DB_FALSE;
            	$data->send = DB_TRUE;
            	break;
    	}
    	return $data;
    }


    /**
     * Mtodo que retorna lista de opes de configurao de recibo
     * @return (array) lista de opes
     */
    private function listReceiptConfigure()
    {
    	if (MUtil::getBooleanValue(MARK_DONT_PRINT_SEND_RECEIPT))
    	{
	    	return array(self::NOT_SEND_MAIL=>_M("Don't print and send receipt", $this->module),
	    	             self::PRINT_AND_SEND_RECEIPT => _M('Print and send receipt', $this->module),
                         self::PRINT_RECEIPT => _M('Print receipt', $this->module),
	    	             self::SEND_RECEIPT => _M('Send receipt by e-mail', $this->module));
    	}
    	else
    	{
            return array(self::PRINT_AND_SEND_RECEIPT => _M('Print and send receipt', $this->module),
                         self::PRINT_RECEIPT => _M('Print receipt', $this->module),
                         self::SEND_RECEIPT => _M('Send receipt by e-mail', $this->module));

    	}
    }


    /**
     * Mtodo que retorna o nome da opo
     * @param valor
     */
    private function getNameOfReceiptConfigure( $get=null)
    {
    	if (!$get)
    	{
    		return null;
    	}
    	$config = $this->listReceiptConfigure();
    	return $config[$get];
    }
}
?>
