<?php
/**
 * <--- Copyright 2005-2011 de Solis - Cooperativa de Solues Livres Ltda. e
 * Univates - Centro Universitrio.
 * 
 * Este arquivo  parte do programa Gnuteca.
 * 
 * O Gnuteca  um software livre; voc pode redistribu-lo e/ou modific-lo
 * dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao
 * do Software Livre (FSF); na verso 2 da Licena.
 * 
 * Este programa  distribudo na esperana que possa ser til, mas SEM
 * NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO
 * ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU/GPL em
 * portugus para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo
 * "LICENCA.txt", junto com este programa, se no, acesse o Portal do Software
 * Pblico Brasileiro no endereo www.softwarepublico.gov.br ou escreva para a
 * Fundao do Software Livre (FSF) Inc., 51 Franklin St, Fifth Floor, Boston,
 * MA 02110-1301, USA --->
 * 
 *
 * This file handles the connection and actions for basConfig table
 *
 * @author Moises Heberle [moises@solis.coop.br]
 *
 * @version $Id$
 *
 * \b Maintainers \n
 * Eduardo Bonfandini [eduardo@solis.coop.br]
 * Jamiel Spezia [jamiel@solis.coop.br]
 * Luiz Gregory Filho [luiz@solis.coop.br]
 * Moises Heberle [moises@solis.coop.br]
 * Sandro Roberto Weisheimer[sandrow@solis.coop.br]
 *
 * @since
 * Class created on 29/08/2008
 *
 **/


/**
 * Class to manipulate the basConfig table
 **/
class BusinessGnuteca3BusRenew extends GnutecaBusiness
{
    public $pkeys;
    public $colsNoId;

    public $renewId;
    public $loanId;
    public $renewTypeId;
    public $renewDate;
    public $returnForecastDate;
    public $operator;

    public $renewIdS;
    public $loanIdS;
    public $renewTypeIdS;
    public $renewDateS;
    public $beginRenewDateS;
    public $endRenewDateS;
    public $beginReturnForecastDateS;
    public $endReturnForecastDateS;
    public $operatorS;
    public $libraryUnitIdS;
    public $itemNumberS;
    public $personIdS;


    /**
     * Class constructor
     **/
    function __construct()
    {
        parent::__construct();
        $this->tables   = 'gtcRenew';
        $this->pkeys    = 'renewId';
        $this->colsNoId = 'loanId,
                           renewTypeId,
                           renewDate,
                           returnForecastDate,
                           operator';
        $this->columns  = $this->pkeys . ',' . $this->colsNoId;
    }


    /**
     * List all records from the table handled by the class
     *
     * @param None
     *
     * @returns (array): Return an array with the entire table
     *
     **/
    public function listRenew()
    {
        $this->clear();
        $this->setColumns($this->columns);
        $this->setTables($this->tables);
        $sql = $this->select();
        $rs  = $this->query($sql);
        return $rs;
    }


    /**
     * Return a specific record from the database
     *
     * @param $moduleConfig (integer): Primary key of the record to be retrieved
     * @param $parameter (integer): Primary key of the record to be retrieved
     *
     * @return (object): Return an object of the type handled by the class
     *
     **/
    public function getRenew($renewId)
    {
        $data[] = $renewId;

        $this->clear();
        $this->setColumns($this->columns);
        $this->setTables($this->tables);
        $this->setWhere('renewId = ?');
        $sql = $this->select($data);
        $rs  = $this->query($sql, TRUE);
        $this->setData($rs[0]);
        return $this;
    }


    /**
     * Do a search on the database table handled by the class
     *
     * @param $filters (object): Search filters
     *
     * @return (array): An array containing the search results
     **/
    public function searchRenew($toObject = false)
    {
        $this->clear();

        if ($v = $this->renewIdS)
        {
            $this->setWhere('A.renewId = ?');
            $data[] = $v;
        }
        if ( $this->personIdS )
        {
            $this->setWhere('P.personId = ?');
            $data[] = $this->personIdS;
        }
        if ($v = $this->loanIdS)
        {
            $this->setWhere('A.loanId = ?');
            $data[] = $v;
        }
        if ($v = $this->renewTypeIdS)
        {
            $this->setWhere('A.renewTypeId = ?');
            $data[] = $v;
        }
        if ($v = $this->renewDateS)
        {
            $this->setWhere('date(A.renewDate) = ?');
            $data[] = $v;
        }
        if ($v = $this->beginRenewDateS)
        {
            $this->setWhere('date(A.renewDate) >= ?');
            $data[] = $v;
        }
        if ($v = $this->endRenewDateS)
        {
            $this->setWhere('date(A.renewDate) <= ?');
            $data[] = $v;
        }
        if ($v = $this->beginReturnForecastDateS)
        {
            $this->setWhere('A.returnForecastDate >= ?');
            $data[] = $v;
        }
        if ($v = $this->endReturnForecastDateS)
        {
            $this->setWhere('A.returnForecastDate <= ?');
            $data[] = $v;
        }
        if ($v = $this->operatorS)
        {
            $this->setWhere('lower(A.operator) LIKE lower(?)');
            $data[] = '%' . $v . '%';
        }
        if ($this->libraryUnitIdS)
        {
            $this->setWhere('L.libraryUnitId in (' . $this->libraryUnitIdS . ')' );
        }
        if ($this->itemNumberS)
        {
        	$this->setWhere('L.itemNumber = ?');
        	$data[] = $this->itemNumberS;
        }

        $tables = '     gtcRenew        A
             LEFT JOIN  gtcRenewType    B
                    ON  (A.renewTypeId = B.renewTypeId)
             LEFT JOIN  gtcLoan         L
                    ON  (A.loanId = L.loanId)
             LEFT JOIN  basPerson       P
                    ON  (P.personId = L.personId)';
        $columns = 'A.renewId,
                    A.loanId,
                    P.personId,
                    P.name AS personName,
                    L.itemNumber,
                    NULL AS tomo,
                    NULL AS data,
                    B.description AS renewType,
                    A.renewDate,
                    A.returnForecastDate,
                    A.operator';

        $this->setTables($tables);
        $this->setColumns($columns);
        $this->setOrderBy('renewId');
        $sql = $this->select($data);
        $rs  = $this->query($sql, $toObject);
        return $rs;
    }


    /**
     * Insert a new record
     *
     * @param $data (object): An object of the type handled by the class
     *
     * @return True if succed, otherwise False
     *
     **/
    public function insertRenew()
    {
        $data = $this->associateData( $this->colsNoId );

        $this->clear();
        $this->setColumns($this->colsNoId);
        $this->setTables($this->tables);
        $sql = $this->insert($data);

        $rs  = $this->execute($sql);

        return $rs;
    }


    /**
     * Update data from a specific record
     *
     * @param $data (object): Data which will replace the old record data
     *
     * @return (boolean): True if succeed, otherwise False
     *
     **/
    public function updateRenew()
    {
        $data = $this->associateData( $this->colsNoId . ',' . $this->pkeys );

        $this->clear();
        $this->setColumns($this->colsNoId);
        $this->setTables($this->tables);
        $this->setWhere('renewId = ?');
        $sql = $this->update($data);
        $rs  = $this->execute($sql);
        return $rs;
    }


    /**
     * Delete a record
     *
     * @param $moduleConfig (string): Primary key for deletion
     * @param $parameter (string): Primary key for deletion
     *
     * @return (boolean): True if succeed, otherwise False
     *
     **/
    public function deleteRenew($renewId)
    {
        $data[] = $renewId;

        $this->clear();
        $this->setTables($this->tables);
        $this->setWhere('renewId = ?');
        $sql = $this->delete($data);
        $rs  = $this->execute($sql);
        return $rs;
    }

    //Retorna quantidade de renovaes de um emprstimo
    public function getCountRenew($loanId)
    {
    	if (!$loanId)
    	{
    		return false;
    	}
        $sql = "select count (loanid) from gtcrenew where loanid = '{$loanId}'";
        $rs  = $this->query($sql);
        return $rs[0][0];
    }
}
?>
