<?php
/**
 * <--- Copyright 2005-2011 de Solis - Cooperativa de Solues Livres Ltda. e
 * Univates - Centro Universitrio.
 * 
 * Este arquivo  parte do programa Gnuteca.
 * 
 * O Gnuteca  um software livre; voc pode redistribu-lo e/ou modific-lo
 * dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao
 * do Software Livre (FSF); na verso 2 da Licena.
 * 
 * Este programa  distribudo na esperana que possa ser til, mas SEM
 * NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO
 * ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU/GPL em
 * portugus para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo
 * "LICENCA.txt", junto com este programa, se no, acesse o Portal do Software
 * Pblico Brasileiro no endereo www.softwarepublico.gov.br ou escreva para a
 * Fundao do Software Livre (FSF) Inc., 51 Franklin St, Fifth Floor, Boston,
 * MA 02110-1301, USA --->
 * 
 *
 * @author Eduardo Bonfandini [eduardo@solis.coop.br]
 *
 * @version $Id$
 *
 * \b Maintainers \n
 * Eduardo Bonfandini   [eduardo@solis.coop.br]
 * Jamiel Spezia        [jamiel@solis.coop.br]
 * Luiz Gregory Filho   [luiz@solis.coop.br]
 * Moises Heberle       [moises@solis.coop.br]
 *
 * @since
 * Class created on 06/01/2009
 *
 **/
class BusinessGnuteca3BusReport extends GnutecaBusiness
{
	public $busReportParameter;

	public $reportId;
    public $Title;
    public $description;
    public $permission;
    public $reportSql;
    public $reportSubSql;
    public $script;
    public $model;
    public $isActive;
    public $reportGroup;
    public $parameters;

    public $reportIdS;
    public $TitleS;
    public $titleS;
    public $descriptionS;
    public $permissionS;
    public $reportSqlS;
    public $reportSubSqlS;
    public $scriptS;
    public $modelS;
    public $isActiveS;
    public $reportGroupS;


    /**
     * Class constructor
     **/
    function __construct()
    {
        $module = 'gnuteca3';
    	$this->busReportParameter = MIOLO::getInstance()->getBusiness($module, 'BusReportParameter');
        parent::__construct();
        $this->defineTables();
    }


    /**
    * Define or redefine the class atributes;
    */
    function defineTables()
    {
        $this->setTables('gtcReport');
        $this->setId('reportId');
        $this->setColumnsNoId('Title, description, permission, reportSql, reportSubSql, script, model, isActive, reportGroup');
    }


    /**
     * List all records from the table handled by the class
     *
     * @param: None
     *
     * @returns (array): Return an array with the entire table
     *
     **/
    public function listReport($object=FALSE)
    {
        $this->defineTables();
        return $this->autoList();
    }


    /**
     * Return a specific record from the database
     *
     * @param $moduleConfig (integer): Primary key of the record to be retrieved
     * @param $parameter (integer): Primary key of the record to be retrieved
     *
     * @return (object): Return an object of the type handled by the class
     *
     **/
    public function getReport($id)
    {
        $this->defineTables();
        $this->clear;
        $result     = $this->autoGet($id);
        $parameters = $this->getParameters($id);
        $result->parameters = $parameters;
        $this->parameters = $parameters;
        return $result;
    }


    /**
     * Get all parameters of a report
     *
     * @param integer $reportId
     * @return array array of objects
     */
    public function getParameters($reportId)
    {
    	$this->busReportParameter->reportIdS = $reportId;
        $parameters = $this->busReportParameter->searchReportParameter(true);
        return $parameters;
    }


    /**
     * Do a search on the database table handled by the class
     *
     * @param $filters (object): Search filters
     *
     * @return (array): An array containing the search results
     **/
    public function searchReport($toObject = false, $checkAccess = false)
    {

        $this->clear();
        $this->defineTables();
        $this->setOrderBy('title');

        if ( $this->reportIdS )
        {
            $this->setWhere('reportId = ?');
            $data[] = $this->reportIdS;
        }
        if ( $this->titleS )
        {
            $this->setWhere('lower(title) LIKE lower(?)');
            $data[] = $this->titleS . '%';
        }
        if ( $this->descriptionS )
        {
            $this->setWhere('lower(description) LIKE lower(?)');
            $data[] = $this->descriptionS . '%';
        }
        if ( $this->permissionS )
        {
            $this->setWhere('permission = ?');
            $data[] = $this->permissionS;
        }
        if ( $this->isActiveS )
        {
            $this->setWhere('isActive = ?');
            $data[] = $this->isActiveS;
        }
        if ( $this->reportGroupS )
        {
            $this->setWhere('reportGroup = ?');
            $data[] = $this->reportGroupS;
        }

        //Mostra os relatrios de acordo com as permisses do usurio
        if ($checkAccess)
        {
        	if (!GnutecaPerms::checkAccess('gtcAdminReportBasic', null, false))
        	{
                $this->setWhere("permission != 'basic'" );
        	}
        	if (!GnutecaPerms::checkAccess('gtcAdminReportIntermediary', null, false))
        	{
                $this->setWhere("permission != 'intermediary'" );
        	}
        	if (!GnutecaPerms::checkAccess('gtcAdminReportAdvanced', null, false))
        	{
                $this->setWhere("permission != 'advanced'" );
        	}
        }
        
        $sql = $this->select($data);
        $rs  = $this->query($sql, $object);

        return $rs;    
    }


    /**
     * Insert a new record
     *
     * @param $data (object): An object of the type handled by the class
     *
     * @return True if succed, otherwise False
     *
     **/
    public function insertReport()
    {
    	$this->defineTables();
    	$this->reportId = $this->getNextval();
    	$result = $this->autoInsert();
        
    	if ($result)
    	{
    		$this->insertParameters( $this->reportId );
    	}
        
    	return $result;
    }

    /**
     * Insert an array of object that are in session
     *
     * @param integer $reportId
     * @return boolean true if succeded.
     */
    public function insertParameters($reportId)
    {
        //FIXME corrigir isto no pode pegar sesso dentro do bus fere MVC
        $parameters = $_SESSION['GnutecaRepetitiveField']['parameters'];
        
        if ($parameters)
        {
            foreach ($parameters as $line => $info)
            {
                if ( !$info->removeData ) //informaes removidas do gnutecaRepetitiveField
                {
                	$this->busReportParameter->lastValue       = '';
                	$this->busReportParameter->defaultValue    = '';
                	$this->busReportParameter->options         = '';
                    $this->busReportParameter->setData($info);
                    $this->busReportParameter->reportId = $reportId;
                    unset( $this->busReportParameter->reportParameterId );
                    $result = $this->busReportParameter->insertReportParameter();
                }
                else
                {
                    $result = true;
                }
            }
        }
        else
        {
            $result = true;
        }
        
        return $result;
    }


    /**
     * Return the next available id
     *
     * @return integer the next available id
     */
    public function getNextval()
    {
    	$sql    = "select nextval('seq_reportid');";
    	$result = $this->query($sql);
    	return $result[0][0];
    }

   
    /**
     * Update data from a specific record
     *
     * @param $data (object): Data which will replace the old record data
     *
     * @return (boolean): True if succeed, otherwise False
     *
     **/
    public function updateReport()
    {
        $this->defineTables();

        $ok = $this->autoUpdate();

        if ( $ok && $this->parameters )
        {
            $ok = $this->busReportParameter->deleteReportParameterByReport($this->reportId);
            $ok = $this->insertParameters($this->reportId);
        }
        
        return $ok;
    }


    /**
     * Delete a record
     *
     * @param $moduleConfig (string): Primary key for deletion
     * @param $parameter (string): Primary key for deletion
     *
     * @return (boolean): True if succeed, otherwise False
     *
     **/
    public function deleteReport( $reportId )
    {
    	$this->busReportParameter->deleteReportParameterByReport($reportId);
        $this->defineTables();
        return $this->autoDelete( $reportId );
    }


    /**
     * Mtodo que gera os SQL's de exportao
     *
     */
    public function exportReport( $reportId )
    {
        $report = $this->getReport($reportId);
        $this->setData($report);

        $this->clear();
        $this->setTables($this->tables);
        $this->setColumns($this->columnsNoId);
        $sqlReport = $this->insert($this->associateData($this->columnsNoId) ) . ';'; //sql do report

        //sql dos parmetros
        $parameters = $report->parameters;
        $sqlParameter = array();
        if ($parameters)
        {
            foreach ($parameters as $line => $info)
            {
                $this->busReportParameter->setData($info);
                $sqlParameter[] = $this->busReportParameter->exportReportParameter();
            }
        }
        $sqlReport .= implode("\n", $sqlParameter);

        return $sqlReport;
    }

}
?>