<?php
/**
 * <--- Copyright 2005-2011 de Solis - Cooperativa de Solues Livres Ltda. e
 * Univates - Centro Universitrio.
 * 
 * Este arquivo  parte do programa Gnuteca.
 * 
 * O Gnuteca  um software livre; voc pode redistribu-lo e/ou modific-lo
 * dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao
 * do Software Livre (FSF); na verso 2 da Licena.
 * 
 * Este programa  distribudo na esperana que possa ser til, mas SEM
 * NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO
 * ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU/GPL em
 * portugus para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo
 * "LICENCA.txt", junto com este programa, se no, acesse o Portal do Software
 * Pblico Brasileiro no endereo www.softwarepublico.gov.br ou escreva para a
 * Fundao do Software Livre (FSF) Inc., 51 Franklin St, Fifth Floor, Boston,
 * MA 02110-1301, USA --->
 * 
 *
 * Return type business
 *
 * @author Sandro Roberto Weisheimer [sandrow@solis.coop.br]
 *
 * @version $Id$
 *
 * \b Maintainers \n
 * Eduardo Bonfandini   [eduardo@solis.coop.br]
 * Jamiel Spezia        [jamiel@solis.coop.br]
 * Luiz Gregory Filho   [luiz@solis.coop.br]
 * Moises Heberle       [moises@solis.coop.br]
 * Sandro Roberto Weisheimer [sandrow@solis.coop.br]
 *
 * @since
 * Class created on 25/05/2009
 *
 **/
class BusinessGnuteca3BusReturnType extends GnutecaBusiness
{
   // public $colsNoId;

    public $returnTypeId;
    public $description;

    public $returnTypeIdS;
    public $typeIdS;
    public $descriptionS;


    public function __construct()
    {
        parent::__construct();
        $this->tables   = 'gtcReturnType';
        $this->colsNoId = 'description';
        $this->columns  = 'returnTypeId, ' . $this->colsNoId;

    }


    /**
     * List all records from the table handled by the class
     *
     * @param: None
     *
     * @returns (array): Return an array with the entire table
     *
     **/
    public function listReturnType($object=FALSE)
    {
        $this->clear();
        $this->setColumns($this->columns);
        $this->setTables($this->tables);
        $this->setOrderBy('returnTypeId');
        $sql = $this->select();
        $rs  = $this->query($sql, $object);
        return $rs;
    }


    /**
     * Return a specific record from the database
     *
     * @param $moduleConfig (integer): Primary key of the record to be retrieved
     * @param $parameter (integer): Primary key of the record to be retrieved
     *
     * @return (object): Return an object of the type handled by the class
     *
     **/
    public function getReturnType($returnTypeId)
    {
        $data = array($returnTypeId);

        $this->clear();
        $this->setColumns($this->columns);
        $this->setTables($this->tables);
        $this->setWhere('returnTypeId = ?');
        $sql = $this->select($data);
        $rs  = $this->query($sql, $toObject=true);
        $this->setData($rs[0]);

        return $this;
    }

    /**
     * @param int $returnTypeId
     * @return string $description
     *
     * Funo retorna a descrio do tipo de retorno de material.
     * Function returns the description of loan return type.
     */
    public function getReturnTypeDescription($returnTypeId)
    {
        $data = array($returnTypeId);
        $this->clear();
        $this->setColumns('description');
        $this->setTables($this->tables);
        $this->setWhere('returnTypeId = ?');
        $sql = $this->select($data);
        $rs  = $this->query($sql, $toObject=true);

        return $rs[0]->description;
    }
    /**
     * Do a search on the database table handled by the class
     *
     * @param $filters (object): Search filters
     *
     * @return (array): An array containing the search results
     **/
    public function searchReturnType()
    {
        $this->clear();

        if ($v = $this->typeIdS)
        {
            $this->setWhere('returnTypeId = ?');
            $data[] = $v;
        }
        if ($v = $this->returnTypeIdS)
        {
            $this->setWhere('returnTypeId = ?');
            $data[] = $v;
        }
        if ($v = $this->descriptionS)
        {
            $this->setWhere('lower(description) LIKE lower(?)');
            $data[] = $v . '%';
        }

        $this->setColumns($this->columns);
        $this->setTables($this->tables);
        $this->setOrderBy('returnTypeId');
        $sql = $this->select($data);
        $rs  = $this->query($sql);

        return $rs;
    }


    /**
     * Insert a new record
     *
     * @param $data (object): An object of the type handled by the class
     *
     * @return True if succed, otherwise False
     *
     **/
    public function insertReturnType()
    {
        $this->clear();
        $this->setColumns($this->colsNoId);
        $this->setTables($this->tables);
        $data= array($this->description);
        $sql = $this->insert($data);
        $rs  = $this->execute($sql);

        return $rs;
    }


    /**
     * Update data from a specific record
     *
     * @param $data (object): Data which will replace the old record data
     *
     * @return (boolean): True if succeed, otherwise False
     *
     **/
    public function updateReturnType()
    {
        $data= array($this->description,
                     $this->returnTypeId);
        $this->clear();
        $this->setColumns($this->colsNoId);
        $this->setTables($this->tables);
        $this->setWhere('returnTypeId = ?');
        $sql = $this->update($data);
        $rs  = $this->execute($sql);

        return $rs;
    }


    /**
     * Delete a record
     *
     * @param $moduleConfig (string): Primary key for deletion
     * @param $parameter (string): Primary key for deletion
     *
     * @return (boolean): True if succeed, otherwise False
     *
     **/
    public function deleteReturnType($returnTypeId)
    {
        $data = array($returnTypeId);

        $this->clear();
        $this->setTables($this->tables);
        $this->setWhere('returnTypeId = ?');
        $sql = $this->delete($data);
        $rs  = $this->execute($sql);

        return $rs;
    }

}
?>
